@@ -13,6 +13,18 @@
 
 =========================================================================*/
 
+#ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif /* HAVE_SYS_TYPES_H */
+
+#ifdef HAVE_STDINT_H
+#  include <stdint.h>
+#endif /* HAVE_STDINT_H */
+
+#ifdef HAVE_STDDEF_H
+#  include <stddef.h>
+#endif /* HAVE_STDDEF_H */
+
 #define TYPE_LONG_LONG long long
 
 typedef VTK_TEST_COMPARE_TYPE_1 Type1;
