@@ -599,7 +599,15 @@ void TestVoidReturnOuter()
 # pragma warning (disable:858) // type qualifier on return is meaningless
 #endif
 
+
+// aCC warns "type qualifier on return type is meaningless" - just omit the
+// function on aCC builds since there is no way to suppress the warning via
+// pragmas...
+// gcc>=4.3 says type qualifiers ignored on function return type
+#if !defined(__HP_aCC)
 void const TestVoidConstReturn() {}
+#endif
+
 
 #if defined(__INTEL_COMPILER)
 # pragma warning (pop)
@@ -623,6 +631,26 @@ static int TestDriverDebugReport(int type, char* message, int* retVal)
 }
 #endif
 
+
+//----------------------------------------------------------------------------
+
+/* Test setlocale  */
+#include <locale.h>
+int TestSetLocale()
+{
+  char *oldLocale = strdup(setlocale(LC_NUMERIC,NULL));
+  setlocale(LC_NUMERIC,"English");
+  
+  // restore the local
+  if (oldLocale)
+    {
+    setlocale(LC_NUMERIC,oldLocale);
+    free(oldLocale);
+    return 1;
+    }
+  return 0;
+}
+
 //----------------------------------------------------------------------------
 
 #define DO_TEST(x) \
@@ -649,6 +677,8 @@ int main()
 #ifndef VTK_CXX_GCC_2 // avoid strange exception problem on debian gcc 2.95
   DO_TEST(TestException);
 #endif
+  DO_TEST(TestSetLocale);
+  
 #if defined(_MSC_VER) && defined(_DEBUG)
   // just call the code to shut up a linker warning
   int retVal = 0;
