@@ -14,6 +14,7 @@
 =========================================================================*/
 #include "vtkInformation.h"
 
+#include "vtkCommand.h"
 #include "vtkGarbageCollector.h"
 #include "vtkInformationDataObjectKey.h"
 #include "vtkInformationDoubleKey.h"
@@ -24,6 +25,7 @@
 #include "vtkInformationIntegerKey.h"
 #include "vtkInformationIntegerPointerKey.h"
 #include "vtkInformationIntegerVectorKey.h"
+#include "vtkInformationIterator.h"
 #include "vtkInformationKeyVectorKey.h"
 #include "vtkInformationObjectBaseKey.h"
 #include "vtkInformationRequestKey.h"
@@ -39,7 +41,7 @@
 
 #include "vtkInformationInternals.h"
 
-vtkCxxRevisionMacro(vtkInformation, "$Revision: 1.26 $");
+vtkCxxRevisionMacro(vtkInformation, "$Revision: 1.30 $");
 vtkStandardNewMacro(vtkInformation);
 
 //----------------------------------------------------------------------------
@@ -87,6 +89,39 @@ void vtkInformation::PrintSelf(ostream& os, vtkIndent indent)
     }
 }
 
+//----------------------------------------------------------------------------
+// call modified on superclass
+void vtkInformation::Modified()
+{
+  this->Superclass::Modified();
+}
+
+//----------------------------------------------------------------------------
+// Update MTime and invoke a modified event with
+// the information key as call data
+void vtkInformation::Modified(vtkInformationKey* key)
+{
+  this->MTime.Modified();
+  this->InvokeEvent(vtkCommand::ModifiedEvent, key);
+}
+
+//----------------------------------------------------------------------------
+// Return the number of keys as a result of iteration.
+int vtkInformation::GetNumberOfKeys()
+{
+  vtkSmartPointer<vtkInformationIterator> infoIterator = 
+    vtkSmartPointer<vtkInformationIterator>::New();
+  infoIterator->SetInformation( this );
+
+  int numberOfKeys = 0;
+  for (infoIterator->InitTraversal(); !infoIterator->IsDoneWithTraversal();
+    infoIterator->GoToNextItem())
+    {
+    numberOfKeys++;
+    }
+  return numberOfKeys;
+}
+
 //----------------------------------------------------------------------------
 // grow the table by a factor of 2
 void vtkInformation::ExpandTable()
@@ -199,7 +234,7 @@ void vtkInformation::SetAsObjectBase(vtkInformationKey* key,
     this->Internal->Values[hash] = newvalue;
     }
   
-  this->Modified();
+  this->Modified(key);
 }
 
 //----------------------------------------------------------------------------
@@ -257,16 +292,30 @@ void vtkInformation::Copy(vtkInformation* from, int deep)
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationKey* key, int)
+                               vtkInformationKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationDataObjectKey* key, int)
+                               vtkInformationDataObjectKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -300,51 +349,100 @@ void vtkInformation::CopyEntry(vtkInformation* from,
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationIntegerKey* key, int)
+                               vtkInformationIntegerKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationRequestKey* key, int)
+                               vtkInformationRequestKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationIntegerVectorKey* key, int)
+                               vtkInformationIntegerVectorKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationDoubleVectorKey* key, int)
+                               vtkInformationDoubleVectorKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationStringKey* key, int)
+                               vtkInformationStringKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationUnsignedLongKey* key, int)
+                               vtkInformationUnsignedLongKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkInformation::CopyEntry(vtkInformation* from, 
-                               vtkInformationStringVectorKey* key, int)
+                               vtkInformationStringVectorKey* key, int deep)
 {
-  key->ShallowCopy(from, this);
+  if (!deep)
+    {
+    key->ShallowCopy(from, this);
+    }
+  else
+    {
+    key->DeepCopy(from, this);
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -359,17 +457,31 @@ void vtkInformation::CopyEntries(vtkInformation* from,
     }
 }
 
+//----------------------------------------------------------------------------
+int vtkInformation::Has(vtkInformationKey* key)
+{
+  // Use the virtual interface in case this is a special-cased key.
+  return key->Has(this)?1:0;
+}
+
+//----------------------------------------------------------------------------
+void vtkInformation::Remove(vtkInformationKey* key)
+{
+  // Use the virtual interface in case this is a special-cased key.
+  key->Remove(this);
+}
+
 void vtkInformation::Set(vtkInformationRequestKey* key)
 {
   key->Set(this);
 }
 void vtkInformation::Remove(vtkInformationRequestKey* key)
 {
-  key->Remove(this);
+  key->vtkInformationRequestKey::Remove(this);
 }
 int vtkInformation::Has(vtkInformationRequestKey* key)
 {
-  return key->Has(this);
+  return key->vtkInformationRequestKey::Has(this);
 }
 
 //----------------------------------------------------------------------------
@@ -380,7 +492,7 @@ int vtkInformation::Has(vtkInformationRequestKey* key)
     }                                                                       \
   void vtkInformation::Remove(vtkInformation##name##Key* key)               \
     {                                                                       \
-    key->Remove(this);                                                      \
+    key->vtkInformation##name##Key::Remove(this);                           \
     }                                                                       \
   type vtkInformation::Get(vtkInformation##name##Key* key)                  \
     {                                                                       \
@@ -388,7 +500,7 @@ int vtkInformation::Has(vtkInformationRequestKey* key)
     }                                                                       \
   int vtkInformation::Has(vtkInformation##name##Key* key)                   \
     {                                                                       \
-    return key->Has(this);                                                  \
+    return key->vtkInformation##name##Key::Has(this);                       \
     }
 VTK_INFORMATION_DEFINE_SCALAR_PROPERTY(IdType, vtkIdType);
 VTK_INFORMATION_DEFINE_SCALAR_PROPERTY(Integer, int);
@@ -432,11 +544,11 @@ VTK_INFORMATION_DEFINE_SCALAR_PROPERTY(ObjectBase, vtkObjectBase*);
     }                                                                       \
   void vtkInformation::Remove(vtkInformation##name##VectorKey* key)         \
     {                                                                       \
-    key->Remove(this);                                                      \
+    key->vtkInformation##name##VectorKey::Remove(this);                     \
     }                                                                       \
   int vtkInformation::Has(vtkInformation##name##VectorKey* key)             \
     {                                                                       \
-    return key->Has(this);                                                  \
+    return key->vtkInformation##name##VectorKey::Has(this);                 \
     }
 VTK_INFORMATION_DEFINE_VECTOR_PROPERTY(Integer, int);
 VTK_INFORMATION_DEFINE_VECTOR_PROPERTY(Double, double);
@@ -463,11 +575,11 @@ int vtkInformation::Length(vtkInformationStringVectorKey* key)
   }
 void vtkInformation::Remove(vtkInformationStringVectorKey* key)
   {
-  key->Remove(this);
+  key->vtkInformationStringVectorKey::Remove(this);
   }
 int vtkInformation::Has(vtkInformationStringVectorKey* key)
   {
-  return key->Has(this);
+  return key->vtkInformationStringVectorKey::Has(this);
   }
 
 VTK_INFORMATION_DEFINE_VECTOR_PROPERTY(Key, vtkInformationKey*);
@@ -522,11 +634,11 @@ VTK_INFORMATION_DEFINE_VECTOR_VALUE_PROPERTY(Double, double);
     }                                                                       \
   void vtkInformation::Remove(vtkInformation##name##PointerKey* key)         \
     {                                                                       \
-    key->Remove(this);                                                      \
+    key->vtkInformation##name##PointerKey::Remove(this);                    \
     }                                                                       \
   int vtkInformation::Has(vtkInformation##name##PointerKey* key)             \
     {                                                                       \
-    return key->Has(this);                                                  \
+    return key->vtkInformation##name##PointerKey::Has(this);                \
     }
 VTK_INFORMATION_DEFINE_POINTER_PROPERTY(Integer, int);
 #undef VTK_INFORMATION_DEFINE_POINTER_PROPERTY
