@@ -16,7 +16,7 @@
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkCone, "$Revision: 1.26 $");
+vtkCxxRevisionMacro(vtkCone, "$Revision: 1.28 $");
 vtkStandardNewMacro(vtkCone);
 
 // Construct cone with angle of 45 degrees.
@@ -26,26 +26,24 @@ vtkCone::vtkCone()
 }
 
 // Evaluate cone equation.
-double vtkCone::EvaluateFunction(double x[3])
+double vtkCone::EvaluateFunction( double x[3] )
 {
-  double tanTheta = (double) 
-    tan((double)this->Angle*vtkMath::DegreesToRadians());
-  return x[1]*x[1] + x[2]*x[2] - x[0]*x[0]*tanTheta*tanTheta;
+  double tanTheta = tan( vtkMath::RadiansFromDegrees( this->Angle) );
+  return x[1] * x[1] + x[2] * x[2] - x[0] * x[0] * tanTheta * tanTheta;
 }
 
 // Evaluate cone normal.
-void vtkCone::EvaluateGradient(double x[3], double g[3])
+void vtkCone::EvaluateGradient( double x[3], double g[3] )
 {
-  double tanTheta = (double) 
-    tan((double)this->Angle*vtkMath::DegreesToRadians());
-  g[0] = -2.0*x[0]*tanTheta*tanTheta;
-  g[1] = 2.0*x[1];
-  g[2] = 2.0*x[2];
+  double tanTheta = tan( vtkMath::RadiansFromDegrees( this->Angle) );
+  g[0] = -2.0 * x[0] * tanTheta * tanTheta;
+  g[1] = 2.0 * x[1];
+  g[2] = 2.0 * x[2];
 }
 
-void vtkCone::PrintSelf(ostream& os, vtkIndent indent)
+void vtkCone::PrintSelf( ostream& os, vtkIndent indent )
 {
-  this->Superclass::PrintSelf(os,indent);
+  this->Superclass::PrintSelf( os, indent );
 
   os << indent << "Angle: " << this->Angle << "\n";
 }
