@@ -37,7 +37,7 @@
 
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkDemandDrivenPipeline, "$Revision: 1.57 $");
+vtkCxxRevisionMacro(vtkDemandDrivenPipeline, "$Revision: 1.61 $");
 vtkStandardNewMacro(vtkDemandDrivenPipeline);
 
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, DATA_NOT_GENERATED, Integer);
@@ -46,6 +46,7 @@ vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_DATA, Request);
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_DATA_NOT_GENERATED, Request);
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_DATA_OBJECT, Request);
 vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_INFORMATION, Request);
+vtkInformationKeyMacro(vtkDemandDrivenPipeline, REQUEST_REGENERATE_INFORMATION, Integer);
 
 //----------------------------------------------------------------------------
 vtkDemandDrivenPipeline::vtkDemandDrivenPipeline()
@@ -171,7 +172,8 @@ int vtkDemandDrivenPipeline::ProcessRequest(vtkInformation* request,
   if(this->Algorithm && request->Has(REQUEST_DATA_OBJECT()))
     {
     // if we are up to date then short circuit
-    if (this->PipelineMTime < this->DataObjectTime.GetMTime())
+    if (this->PipelineMTime < this->DataObjectTime.GetMTime()
+      && ! request->Has(REQUEST_REGENERATE_INFORMATION()))      
       {
       return 1;
       }
@@ -183,7 +185,8 @@ int vtkDemandDrivenPipeline::ProcessRequest(vtkInformation* request,
     
     // Make sure our output data type is up-to-date.
     int result = 1;
-    if(this->PipelineMTime > this->DataObjectTime.GetMTime())
+    if(this->PipelineMTime > this->DataObjectTime.GetMTime()
+       || request->Has(REQUEST_REGENERATE_INFORMATION()))
       {
       // Request data type from the algorithm.
       result = this->ExecuteDataObject(request,inInfoVec,outInfoVec);
@@ -212,7 +215,8 @@ int vtkDemandDrivenPipeline::ProcessRequest(vtkInformation* request,
   if(this->Algorithm && request->Has(REQUEST_INFORMATION()))
     {
     // if we are up to date then short circuit
-    if (this->PipelineMTime < this->InformationTime.GetMTime())
+    if (this->PipelineMTime < this->InformationTime.GetMTime()
+        && ! request->Has(REQUEST_REGENERATE_INFORMATION()))      
       {
       return 1;
       }
@@ -224,7 +228,9 @@ int vtkDemandDrivenPipeline::ProcessRequest(vtkInformation* request,
 
     // Make sure our output information is up-to-date.
     int result = 1;
-    if(this->PipelineMTime > this->InformationTime.GetMTime())
+    if(this->PipelineMTime > this->InformationTime.GetMTime()
+      || request->Has(REQUEST_REGENERATE_INFORMATION()))
+
       {
       // Make sure input types are valid before algorithm does anything.
       if(!this->InputCountIsValid(inInfoVec) || 
@@ -550,10 +556,7 @@ void vtkDemandDrivenPipeline::ExecuteDataStart(vtkInformation* request,
         {
         vtkInformation* outInfo = outputs->GetInformationObject(i);
         vtkDataObject* output = outInfo->Get(vtkDataObject::DATA_OBJECT());
-        // We want to pass the field data unless it is a table.
-        // Since a table's data is stored in the field data, we want
-        // table algorithms to start with an empty field data.
-        if(output && !output->IsA("vtkTable"))
+        if(output)
           {
           output->GetFieldData()->PassData(input->GetFieldData());
           }
