@@ -33,7 +33,7 @@
 #include "vtkTriangleStrip.h"
 #include "vtkVertex.h"
 
-vtkCxxRevisionMacro(vtkPolyData, "$Revision: 1.10 $");
+vtkCxxRevisionMacro(vtkPolyData, "$Revision: 1.13 $");
 vtkStandardNewMacro(vtkPolyData);
 
 //----------------------------------------------------------------------------
@@ -239,13 +239,6 @@ void vtkPolyData::CopyStructure(vtkDataSet *ds)
     this->Links->UnRegister(this);
     this->Links = NULL;
     }
-
-  // Reset this information to mantain the functionality that was present when
-  // CopyStructure called Initialize, which incorrectly wiped out attribute
-  // data.  Someone MIGHT argue that this isn't the right thing to do.
-  this->Information->Set(vtkDataObject::DATA_PIECE_NUMBER(), -1);
-  this->Information->Set(vtkDataObject::DATA_NUMBER_OF_PIECES(), 0);
-  this->Information->Set(vtkDataObject::DATA_NUMBER_OF_GHOST_LEVELS(), 0);
 }
 
 //----------------------------------------------------------------------------
@@ -855,9 +848,9 @@ void vtkPolyData::Initialize()
 
   if(this->Information)
     {
-  this->Information->Set(vtkDataObject::DATA_PIECE_NUMBER(), -1);
-  this->Information->Set(vtkDataObject::DATA_NUMBER_OF_PIECES(), 0);
-  this->Information->Set(vtkDataObject::DATA_NUMBER_OF_GHOST_LEVELS(), 0);
+    this->Information->Set(vtkDataObject::DATA_PIECE_NUMBER(), -1);
+    this->Information->Set(vtkDataObject::DATA_NUMBER_OF_PIECES(), 0);
+    this->Information->Set(vtkDataObject::DATA_NUMBER_OF_GHOST_LEVELS(), 0);
     }
 }
 
@@ -2048,19 +2041,55 @@ void  vtkPolyData::RemoveDeletedCells()
   vtkCellData *newCellData;
   newCellData = vtkCellData::New();
   newCellData->CopyAllocate(this->CellData, this->GetNumberOfCells());
-  vtkCellArray *newPolys;
   vtkIdType inCellId=0, outCellId=0;
   vtkIdType npts=0;
   vtkIdType *pts=0;
+  vtkIdType c = 0;
 
+  if (this->Verts)
+    {
+    vtkCellArray* newVerts = vtkCellArray::New();
+    newVerts->Allocate(this->Verts->GetSize());
+    for (this->Verts->InitTraversal(); this->Verts->GetNextCell(npts, pts); c++)
+      {
+      if (this->Cells->GetCellType(c)!=VTK_EMPTY_CELL)
+        { // Keep the cell.
+        newVerts->InsertNextCell(npts, pts);
+        newCellData->CopyData(this->CellData, inCellId, outCellId);
+        ++outCellId;
+        } // Keep this cell.
+      ++inCellId;
+      } // for all cells
+    this->SetVerts(newVerts);
+    newVerts->Delete();
+    }
+
+  if (this->Lines)
+    {
+    vtkCellArray* newLines = vtkCellArray::New();
+    newLines->Allocate(this->Lines->GetSize());
+    for (this->Lines->InitTraversal(); this->Lines->GetNextCell(npts, pts); c++)
+      {
+      if (this->Cells->GetCellType(c)!=VTK_EMPTY_CELL)
+        { // Keep the cell.
+        newLines->InsertNextCell(npts, pts);
+        newCellData->CopyData(this->CellData, inCellId, outCellId);
+        ++outCellId;
+        } // Keep this cell.
+      ++inCellId;
+      } // for all cells
+    this->SetLines(newLines);
+    newLines->Delete();
+    }
+  
   if (this->Polys)
     {
+    vtkCellArray *newPolys;
     newPolys = vtkCellArray::New();
     newPolys->Allocate(this->Polys->GetSize());
-    vtkIdType c = 0;
     for (this->Polys->InitTraversal(); this->Polys->GetNextCell(npts, pts); c++)
       {
-        if (this->Cells->GetCellType(c)!=VTK_EMPTY_CELL)
+      if (this->Cells->GetCellType(c)!=VTK_EMPTY_CELL)
         { // Keep the cell.
         newPolys->InsertNextCell(npts, pts);
         newCellData->CopyData(this->CellData, inCellId, outCellId);
@@ -2068,11 +2097,34 @@ void  vtkPolyData::RemoveDeletedCells()
         } // Keep this cell.
       ++inCellId;
       } // for all cells
-   this->SetPolys(newPolys);
-   newPolys->Delete();
-   newPolys = NULL;
-   }
+    this->SetPolys(newPolys);
+    newPolys->Delete();
+    }
+  
+  if (this->Strips)
+    {
+    vtkCellArray* newStrips = vtkCellArray::New();
+    newStrips->Allocate(this->Strips->GetSize());
+    for (this->Strips->InitTraversal(); this->Strips->GetNextCell(npts, pts); c++)
+      {
+      if (this->Cells->GetCellType(c)!=VTK_EMPTY_CELL)
+        { // Keep the cell.
+        newStrips->InsertNextCell(npts, pts);
+        newCellData->CopyData(this->CellData, inCellId, outCellId);
+        ++outCellId;
+        } // Keep this cell.
+      ++inCellId;
+      } // for all cells
+    this->SetStrips(newStrips);
+    newStrips->Delete();
+    }
+
+  
   // Save the results.
+  if(inCellId != outCellId)
+    { // we have deleted cells so need to update this->Cells
+    this->BuildCells();
+    }
   this->CellData->ShallowCopy(newCellData);
   newCellData->Delete();
 }
