@@ -23,7 +23,7 @@
 #include "vtkPropCollection.h"
 #include "vtkViewport.h"
 
-vtkCxxRevisionMacro(vtkPropAssembly, "$Revision: 1.4 $");
+vtkCxxRevisionMacro(vtkPropAssembly, "$Revision: 1.7 $");
 vtkStandardNewMacro(vtkPropAssembly);
 
 // Construct object with no children.
@@ -72,9 +72,14 @@ int vtkPropAssembly::RenderTranslucentPolygonalGeometry(vtkViewport *ren)
   vtkAssemblyPath *path;
   double fraction;
   int renderedSomething=0;
-
-  fraction = this->AllocatedRenderTime / 
-    static_cast<double>(this->Parts->GetNumberOfItems());
+  
+  // Make sure the paths are up-to-date
+  this->UpdatePaths();
+  
+  double numberOfItems =  static_cast<double>(this->Parts->GetNumberOfItems());
+  fraction = numberOfItems >= 1.0 ? 
+    this->AllocatedRenderTime / numberOfItems : this->AllocatedRenderTime;
+    
   
   // render the Paths
   vtkCollectionSimpleIterator sit;
@@ -101,6 +106,9 @@ int vtkPropAssembly::HasTranslucentPolygonalGeometry()
   vtkAssemblyPath *path;
   int result=0;
   
+  // Make sure the paths are up-to-date
+  this->UpdatePaths();
+  
   // render the Paths
   vtkCollectionSimpleIterator sit;
   for ( this->Paths->InitTraversal(sit); !result && (path = this->Paths->GetNextPath(sit)); )
@@ -122,9 +130,13 @@ int vtkPropAssembly::RenderVolumetricGeometry(vtkViewport *ren)
   vtkAssemblyPath *path;
   double fraction;
   int renderedSomething=0;
-
-  fraction = this->AllocatedRenderTime / 
-    static_cast<double>(this->Parts->GetNumberOfItems());
+  
+   // Make sure the paths are up-to-date
+  this->UpdatePaths();
+  
+  double numberOfItems =  static_cast<double>(this->Parts->GetNumberOfItems());
+  fraction = numberOfItems >= 1.0 ? 
+    this->AllocatedRenderTime / numberOfItems : this->AllocatedRenderTime;
   
   // render the Paths
   vtkCollectionSimpleIterator sit;
@@ -150,12 +162,14 @@ int vtkPropAssembly::RenderOpaqueGeometry(vtkViewport *ren)
   vtkAssemblyPath *path;
   double fraction;
   int   renderedSomething=0;
+  double numberOfItems = 0.0;
 
   // Make sure the paths are up-to-date
   this->UpdatePaths();
 
-  fraction = this->AllocatedRenderTime / 
-    static_cast<double>(this->Parts->GetNumberOfItems());
+  numberOfItems =  static_cast<double>(this->Parts->GetNumberOfItems());
+  fraction = numberOfItems >= 1.0 ? 
+    this->AllocatedRenderTime / numberOfItems : this->AllocatedRenderTime;
   
   // render the Paths
   vtkCollectionSimpleIterator sit;
@@ -181,12 +195,14 @@ int vtkPropAssembly::RenderOverlay(vtkViewport *ren)
   vtkAssemblyPath *path;
   double fraction;
   int   renderedSomething=0;
+  double numberOfItems = 0.0;
 
   // Make sure the paths are up-to-date
   this->UpdatePaths();
 
-  fraction = this->AllocatedRenderTime / 
-    static_cast<double>(this->Parts->GetNumberOfItems());
+  numberOfItems =  static_cast<double>(this->Parts->GetNumberOfItems());
+  fraction = numberOfItems >= 1.0 ? 
+    this->AllocatedRenderTime / numberOfItems : this->AllocatedRenderTime;
   
   vtkCollectionSimpleIterator sit;
   for ( this->Paths->InitTraversal(sit); (path = this->Paths->GetNextPath(sit)); )
@@ -233,7 +249,7 @@ double *vtkPropAssembly::GetBounds()
   for ( this->Parts->InitTraversal(pit); 
         (part=this->Parts->GetNextProp(pit)); )
     {
-    if ( part->GetVisibility() )
+    if ( part->GetVisibility() && part->GetUseBounds() )
       {
       bounds = part->GetBounds();
       
