@@ -26,7 +26,7 @@
 #include "vtkPolygon.h"
 #include "vtkQuadric.h"
 
-vtkCxxRevisionMacro(vtkTriangle, "$Revision: 1.4.10.1 $");
+vtkCxxRevisionMacro(vtkTriangle, "$Revision: 1.8 $");
 vtkStandardNewMacro(vtkTriangle);
 
 //----------------------------------------------------------------------------
@@ -357,6 +357,12 @@ static LINE_CASES lineCases[] = {
 
 static int edges[3][2] = { {0,1}, {1,2}, {2,0} };
 
+//----------------------------------------------------------------------------
+int *vtkTriangle::GetEdgeArray(int edgeId)
+{
+  return edges[edgeId];
+}
+
 //----------------------------------------------------------------------------
 void vtkTriangle::Contour(double value, vtkDataArray *cellScalars, 
                           vtkPointLocator *locator,
@@ -981,9 +987,11 @@ int vtkTriangle::PointInTriangle(double x[3], double p1[3], double p2[3],
   vtkMath::Cross (x2, v21, n2);
   vtkMath::Cross (x3, v32, n3);
 
-  //  Check whether normals go in same direction
-  //
-  if ( (vtkMath::Dot(n1,n2) >= 0.0) && (vtkMath::Dot(n2,n3) >= 0.0) )
+  //  Check whether ALL the three normals go in same direction
+  // 
+  if ( (vtkMath::Dot(n1,n2) >= 0.0) && 
+       (vtkMath::Dot(n2,n3) >= 0.0) && 
+       (vtkMath::Dot(n1,n3) >= 0.0) )
     {
     return 1;
     }
