@@ -21,7 +21,7 @@
 #include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkTrivialProducer, "$Revision: 1.11 $");
+vtkCxxRevisionMacro(vtkTrivialProducer, "$Revision: 1.13 $");
 vtkStandardNewMacro(vtkTrivialProducer);
 
 // This compile-time switch determines whether the update extent is
@@ -103,8 +103,9 @@ int vtkTrivialProducer::FillInputPortInformation(int, vtkInformation*)
 }
 
 //----------------------------------------------------------------------------
-int vtkTrivialProducer::FillOutputPortInformation(int, vtkInformation*)
+int vtkTrivialProducer::FillOutputPortInformation(int, vtkInformation* info)
 {
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkDataObject");
   return 1;
 }
 
@@ -134,6 +135,19 @@ vtkTrivialProducer::ProcessRequest(vtkInformation* request,
       outputInfo->Set(vtkStreamingDemandDrivenPipeline::WHOLE_EXTENT(),
                       extent, 6);
       }
+    else if(dataInfo->Get(vtkDataObject::DATA_EXTENT_TYPE()) == VTK_TIME_EXTENT)
+      {
+      // The time extent is captured in TIME_STEPS and/or TIME_RANGE.
+      double time = 0.0;
+      if (dataInfo->Has(vtkDataObject::DATA_TIME_STEPS()))
+        {
+        time = dataInfo->Get(vtkDataObject::DATA_TIME_STEPS())[0];
+        }
+      double timeRange[2];
+      timeRange[0] = timeRange[1] = time;
+      outputInfo->Set(vtkStreamingDemandDrivenPipeline::TIME_STEPS(),
+                      timeRange, 2);
+      }
     }
 #if VTK_TRIVIAL_PRODUCER_CHECK_UPDATE_EXTENT
   if(request->Has(vtkStreamingDemandDrivenPipeline::REQUEST_UPDATE_EXTENT()))
