@@ -56,7 +56,7 @@
 #include "vtkBiQuadraticQuadraticWedge.h"
 #include "vtkBiQuadraticQuadraticHexahedron.h"
 
-vtkCxxRevisionMacro(vtkUnstructuredGrid, "$Revision: 1.13 $");
+vtkCxxRevisionMacro(vtkUnstructuredGrid, "$Revision: 1.14 $");
 vtkStandardNewMacro(vtkUnstructuredGrid);
 
 vtkUnstructuredGrid::vtkUnstructuredGrid ()
@@ -356,13 +356,6 @@ void vtkUnstructuredGrid::CopyStructure(vtkDataSet *ds)
       this->Locations->Register(this);
       }
     }
-
-  // Reset this information to maintain the functionality that was present when
-  // CopyStructure called Initialize, which incorrectly wiped out attribute
-  // data.  Someone MIGHT argue that this isn't the right thing to do.
-  this->Information->Set(vtkDataObject::DATA_PIECE_NUMBER(), -1);
-  this->Information->Set(vtkDataObject::DATA_NUMBER_OF_PIECES(), 0);
-  this->Information->Set(vtkDataObject::DATA_NUMBER_OF_GHOST_LEVELS(), 0);
 }
 
 //----------------------------------------------------------------------------
