@@ -25,7 +25,7 @@
 #include "vtkPolyData.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkAppendPolyData, "$Revision: 1.98 $");
+vtkCxxRevisionMacro(vtkAppendPolyData, "$Revision: 1.104 $");
 vtkStandardNewMacro(vtkAppendPolyData);
 
 //----------------------------------------------------------------------------
@@ -142,7 +142,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   vtkIdType *pPolys;
   vtkIdType npts = 0;
   vtkIdType ptId, cellId;
-  
+
   vtkDebugMacro(<<"Appending polydata");
 
   // loop over all data sets, checking to see what point data is available.
@@ -153,6 +153,8 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   int countPD=0;
   int countCD=0;
 
+  vtkIdType numVerts = 0, numLines = 0, numStrips = 0;
+
   int numInputs = this->GetNumberOfInputConnections(0);
   vtkInformation *inInfo;
 
@@ -178,7 +180,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   // These are used to determine which fields are available for appending
   vtkDataSetAttributes::FieldList ptList(countPD);
   vtkDataSetAttributes::FieldList cellList(countCD);  
-  
+
   countPD = countCD = 0;
   for (idx = 0; idx < numInputs; ++idx)
     {
@@ -202,18 +204,24 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
           }
         ++countPD;
         } // for a data set that has points
-        
+
       // Although we cannot have cells without points ... let's not nest.
       if (ds->GetNumberOfCells() > 0 )  
         {
         // keep track of the size of the poly cell array
         if (ds->GetPolys())
           {
-          numPolys += ds->GetPolys()->GetNumberOfCells();
           sizePolys += ds->GetPolys()->GetNumberOfConnectivityEntries();
           }
         numCells += ds->GetNumberOfCells();
-        
+        // Count the cells of each type.
+        // This is used to ensure that cell data is copied at the correct
+        // locations in the output.
+        numVerts += ds->GetNumberOfVerts();
+        numLines += ds->GetNumberOfLines();
+        numPolys += ds->GetNumberOfPolys();
+        numStrips += ds->GetNumberOfStrips();
+
         inCD = ds->GetCellData();
         if ( countCD == 0 )
           {
@@ -254,7 +262,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
         pointtype = ds->GetPoints()->GetData()->GetDataType();
         }
       ttype = ds->GetPoints()->GetData()->GetDataType();
-      
+
       if ( ttype != pointtype )
         {
         AllSame = 0;
@@ -280,47 +288,79 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   newPolys = vtkCellArray::New();
   pPolys = newPolys->WritePointer(numPolys, sizePolys);
 
+  if (!pPolys && sizePolys > 0)
+    {
+    vtkErrorMacro(<<"Memory allocation failed in append filter");
+    return 0;
+    }
+
   // These are created manually for faster execution
   // Uses the properties of the last input
+  vtkDataArray *inDA=0;
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::SCALARS) > -1 )
     {
+    inDA=inPD->GetScalars();
     outputPD->CopyScalarsOff();
-    newPtScalars = inPD->GetScalars()->NewInstance();
-    newPtScalars->SetNumberOfComponents(inPD->GetScalars()->GetNumberOfComponents());
-    newPtScalars->SetName(inPD->GetScalars()->GetName());
+    newPtScalars = inDA->NewInstance();
+    newPtScalars->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtScalars->SetName(inDA->GetName());
     newPtScalars->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtScalars->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::VECTORS) > -1 )
     {
+    inDA=inPD->GetVectors();
     outputPD->CopyVectorsOff();
-    newPtVectors = inPD->GetVectors()->NewInstance();
-    newPtVectors->SetNumberOfComponents(inPD->GetVectors()->GetNumberOfComponents());
-    newPtVectors->SetName(inPD->GetVectors()->GetName());
+    newPtVectors = inDA->NewInstance();
+    newPtVectors->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtVectors->SetName(inDA->GetName());
     newPtVectors->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtVectors->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::TENSORS) > -1 )
     {
+    inDA=inPD->GetTensors();
     outputPD->CopyTensorsOff();
-    newPtTensors = inPD->GetTensors()->NewInstance();
-    newPtTensors->SetNumberOfComponents(inPD->GetTensors()->GetNumberOfComponents());
-    newPtTensors->SetName(inPD->GetTensors()->GetName());
+    newPtTensors = inDA->NewInstance();
+    newPtTensors->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtTensors->SetName(inDA->GetName());
     newPtTensors->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtTensors->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::NORMALS) > -1 )
     {
+    inDA=inPD->GetNormals();
     outputPD->CopyNormalsOff();
-    newPtNormals = inPD->GetNormals()->NewInstance();
-    newPtNormals->SetNumberOfComponents(inPD->GetNormals()->GetNumberOfComponents());
-    newPtNormals->SetName(inPD->GetNormals()->GetName());
+    newPtNormals = inDA->NewInstance();
+    newPtNormals->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtNormals->SetName(inDA->GetName());
     newPtNormals->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtNormals->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::TCOORDS) > -1 )
     {
+    inDA=inPD->GetTCoords();
     outputPD->CopyTCoordsOff();
-    newPtTCoords = inPD->GetTCoords()->NewInstance();
-    newPtTCoords->SetNumberOfComponents(inPD->GetTCoords()->GetNumberOfComponents());
-    newPtTCoords->SetName(inPD->GetTCoords()->GetName());
+    newPtTCoords = inDA->NewInstance();
+    newPtTCoords->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtTCoords->SetName(inDA->GetName());
     newPtTCoords->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtTCoords->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
 
   // Allocate the point and cell data
@@ -329,7 +369,10 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
 
   // loop over all input sets
   vtkIdType ptOffset = 0;
-  vtkIdType cellOffset = 0;
+  vtkIdType vertOffset = 0;
+  vtkIdType linesOffset = 0;
+  vtkIdType polysOffset = 0;
+  vtkIdType stripsOffset = 0;
   countPD = countCD = 0;
   for (idx = 0; idx < numInputs; ++idx)
     {
@@ -401,20 +444,53 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
         ++countPD;
         }
 
+
       if (ds->GetNumberOfCells() > 0)
         {
+        // These are the cellIDs at which each of the cell types start.
+        vtkIdType linesIndex = ds->GetNumberOfVerts();
+        vtkIdType polysIndex = linesIndex + ds->GetNumberOfLines();
+        vtkIdType stripsIndex = polysIndex + ds->GetNumberOfPolys();
+
         // cell data could be made efficient like the point data,
         // but I will wait on that.
         // copy cell data
         for (cellId=0; cellId < numCells; cellId++)
           {
-          outputCD->CopyData(cellList,inCD,countCD,cellId,cellId+cellOffset);
+          vtkIdType outCellId = 0;
+          if (cellId < linesIndex)
+            {
+            outCellId = vertOffset;
+            vertOffset++;
+            }
+          else if (cellId < polysIndex)
+            {
+            // outCellId = number of lines we already added + total number of 
+            // verts expected in the output.
+            outCellId = linesOffset + numVerts;
+            linesOffset++;
+            }
+          else if (cellId < stripsIndex)
+            {
+            // outCellId = number of polys we already added + total number of 
+            // verts and lines expected in the output.
+            outCellId = polysOffset + numLines + numVerts;
+            polysOffset++;
+            }
+          else
+            {
+            // outCellId = number of tstrips we already added + total number of 
+            // polys, verts and lines expected in the output.
+            outCellId = stripsOffset + numPolys + numLines + numVerts;
+            stripsOffset++;
+            }
+          outputCD->CopyData(cellList,inCD,countCD,cellId,outCellId);
           }
         ++countCD;
-        
+
         // copy the cells
         pPolys = this->AppendCells(pPolys, inPolys, ptOffset);
-        
+
         // These other cell arrays could be made efficient like polys ...
         for (inVerts->InitTraversal(); inVerts->GetNextCell(npts,pts); )
           {
@@ -424,7 +500,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
             newVerts->InsertCellPoint(pts[i]+ptOffset);
             }
           }
-        
+
         for (inLines->InitTraversal(); inLines->GetNextCell(npts,pts); )
           {
           newLines->InsertNextCell(npts);
@@ -444,7 +520,6 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
           }
         }
       ptOffset += numPts;
-      cellOffset += numCells;
       }
     }
   
@@ -666,19 +741,20 @@ void vtkAppendPolyData::AppendDifferentPoints(vtkDataArray *dest,
     //
 
     case VTK_DOUBLE:
-      dDest = (double*)(dest->GetVoidPointer(offset*src->GetNumberOfComponents()));
+      dDest = static_cast<double*>(
+        dest->GetVoidPointer(offset*src->GetNumberOfComponents()));
       //
       switch (src->GetDataType())
         {
         case VTK_FLOAT:
-          fSrc = (float*)(src->GetVoidPointer(0));
+          fSrc = static_cast<float*>(src->GetVoidPointer(0));
           for (p=0; p<vals; p++)
             {
-              dDest[p] = (double) fSrc[p];
+            dDest[p] = static_cast<double>(fSrc[p]);
             }
           break;
         case VTK_DOUBLE:
-          dSrc = (double*)(src->GetVoidPointer(0));
+          dSrc = static_cast<double*>(src->GetVoidPointer(0));
           memcpy(dDest, dSrc, vals*sizeof(double));
           break;
         default:
@@ -705,7 +781,7 @@ vtkIdType *vtkAppendPolyData::AppendCells(vtkIdType *pDest, vtkCellArray *src,
     return pDest;
     }
 
-  pSrc = (vtkIdType*)(src->GetPointer());
+  pSrc = src->GetPointer();
   end = pSrc + src->GetNumberOfConnectivityEntries();
   pNum = pSrc;
   
