@@ -29,7 +29,7 @@
 #include "vtkTransform.h"
 #include "vtkUnsignedCharArray.h"
 
-vtkCxxRevisionMacro(vtkGlyph3D, "$Revision: 1.123.4.1 $");
+vtkCxxRevisionMacro(vtkGlyph3D, "$Revision: 1.125 $");
 vtkStandardNewMacro(vtkGlyph3D);
 
 //----------------------------------------------------------------------------
@@ -151,7 +151,7 @@ int vtkGlyph3D::RequestData(
     }
   else
     {
-    inGhostLevels = ((vtkUnsignedCharArray*)temp)->GetPointer(0);
+    inGhostLevels =static_cast<vtkUnsignedCharArray *>(temp)->GetPointer(0);
     }
 
   requestedGhostLevel =
@@ -361,7 +361,7 @@ int vtkGlyph3D::RequestData(
     scalex = scaley = scalez = 1.0;
     if ( ! (inPtId % 10000) )
       {
-      this->UpdateProgress ((double)inPtId/numPts);
+      this->UpdateProgress(static_cast<double>(inPtId)/numPts);
       if (this->GetAbortExecute())
         {
         break;
@@ -432,7 +432,7 @@ int vtkGlyph3D::RequestData(
         value = vMag;
         }
       
-      index = (int) ((double)(value - this->Range[0]) * numberOfSources / den);
+      index = static_cast<int>((value - this->Range[0])*numberOfSources / den);
       index = (index < 0 ? 0 :
               (index >= numberOfSources ? (numberOfSources-1) : index));
       
@@ -512,7 +512,7 @@ int vtkGlyph3D::RequestData(
           vNew[0] = (v[0]+vMag) / 2.0;
           vNew[1] = v[1] / 2.0;
           vNew[2] = v[2] / 2.0;
-          trans->RotateWXYZ((double)180.0,vNew[0],vNew[1],vNew[2]);
+          trans->RotateWXYZ(180.0,vNew[0],vNew[1],vNew[2]);
           }
         }
       }
