@@ -27,7 +27,7 @@
 #include "vtkTriangleStrip.h"
 #include "vtkPriorityQueue.h"
 
-vtkCxxRevisionMacro(vtkPolyDataNormals, "$Revision: 1.68.24.1 $");
+vtkCxxRevisionMacro(vtkPolyDataNormals, "$Revision: 1.70 $");
 vtkStandardNewMacro(vtkPolyDataNormals);
 
 // Construct with feature angle=30, splitting and consistency turned on, 
@@ -339,8 +339,7 @@ int vtkPolyDataNormals::RequestData(
     //  edges found, split mesh creating new nodes.  Update polygon 
     // connectivity.
     //
-    this->CosAngle = cos ((double) 
-                          vtkMath::DegreesToRadians() * this->FeatureAngle);
+      this->CosAngle = cos( vtkMath::RadiansFromDegrees( this->FeatureAngle) );
     //  Splitting will create new points.  We have to create index array 
     // to map new points into old points.
     //
