@@ -16,6 +16,7 @@
 
 #include "vtkCellData.h"
 #include "vtkCellData.h"
+#include "vtkDataArray.h"
 #include "vtkDataSet.h"
 #include "vtkDataSetAttributes.h"
 #include "vtkFieldData.h"
@@ -23,20 +24,15 @@
 #include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
+#include <ctype.h>
 
-vtkCxxRevisionMacro(vtkRearrangeFields, "$Revision: 1.14 $");
+vtkCxxRevisionMacro(vtkRearrangeFields, "$Revision: 1.20 $");
 vtkStandardNewMacro(vtkRearrangeFields);
 
 typedef vtkRearrangeFields::Operation Operation;
 
 // Used by AddOperation() and RemoveOperation() designed to be used 
 // from other language bindings.
-char vtkRearrangeFields::AttributeNames[vtkDataSetAttributes::NUM_ATTRIBUTES][10] 
-=  { "SCALARS",
-     "VECTORS",
-     "NORMALS",
-     "TCOORDS",
-     "TENSORS" };
 char vtkRearrangeFields::OperationTypeNames[2][5] 
 = { "COPY",
     "MOVE" };
@@ -44,6 +40,8 @@ char vtkRearrangeFields::FieldLocationNames[3][12]
 = { "DATA_OBJECT",
     "POINT_DATA",
     "CELL_DATA" };
+char vtkRearrangeFields::AttributeNames[vtkDataSetAttributes::NUM_ATTRIBUTES][10]  = { {0} };
+
 
 
 //--------------------------------------------------------------------------
@@ -54,6 +52,20 @@ vtkRearrangeFields::vtkRearrangeFields()
   this->Head = 0;
   this->Tail = 0;
   this->LastId = 0;
+  //convert the attribute names to uppercase for local use
+  if (vtkRearrangeFields::AttributeNames[0][0] == 0) 
+    {
+    for (int i = 0; i < vtkDataSetAttributes::NUM_ATTRIBUTES; i++)
+      {
+      int l = static_cast<int>(
+        strlen(vtkDataSetAttributes::GetAttributeTypeAsString(i)));
+      for (int c = 0; c < l && c < 10; c++)
+        {
+        vtkRearrangeFields::AttributeNames[i][c] = 
+          toupper(vtkDataSetAttributes::GetAttributeTypeAsString(i)[c]);
+        }
+      }
+    }
 }
 
 vtkRearrangeFields::~vtkRearrangeFields()
@@ -180,7 +192,7 @@ void vtkRearrangeFields::ApplyOperation(Operation* op, vtkDataSet* input,
       vtkWarningMacro("Can not apply operation " << op->Id
                       << ": Input has to be vtkDataSetAttributes.");
       }
-    outputFD->AddArray(dsa->GetAttribute(op->AttributeType));
+    outputFD->AddArray(dsa->GetAbstractAttribute(op->AttributeType));
     // If moving the array, make sure that it is not copied
     // with PassData()
     if ( op->OperationType == vtkRearrangeFields::MOVE )
