@@ -27,7 +27,7 @@
 #include "vtkPolyData.h"
 #include "vtkTransform.h"
 
-vtkCxxRevisionMacro(vtkTensorGlyph, "$Revision: 1.57.12.1 $");
+vtkCxxRevisionMacro(vtkTensorGlyph, "$Revision: 1.60 $");
 vtkStandardNewMacro(vtkTensorGlyph);
 
 // Construct object with scaling on and scale factor 1.0. Eigenvalues are 
@@ -47,12 +47,22 @@ vtkTensorGlyph::vtkTensorGlyph()
   this->Length = 1.0;
 
   this->SetNumberOfInputPorts(2);
+
+  // by default, process active point tensors
+  this->SetInputArrayToProcess(0, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::TENSORS);
+
+  // by default, process active point scalars
+  this->SetInputArrayToProcess(1, 0, 0, vtkDataObject::FIELD_ASSOCIATION_POINTS,
+                               vtkDataSetAttributes::SCALARS);
 }
 
+//----------------------------------------------------------------------------
 vtkTensorGlyph::~vtkTensorGlyph()
 {
 }
 
+//----------------------------------------------------------------------------
 int vtkTensorGlyph::RequestData(
   vtkInformation *vtkNotUsed(request),
   vtkInformationVector **inputVector,
@@ -113,8 +123,8 @@ int vtkTensorGlyph::RequestData(
 
   pd = input->GetPointData();
   outPD = output->GetPointData();
-  inTensors = pd->GetTensors();
-  inScalars = pd->GetScalars();
+  inTensors = this->GetInputArrayToProcess(0, inputVector);
+  inScalars = this->GetInputArrayToProcess(1, inputVector);
   numPts = input->GetNumberOfPoints();
 
   if ( !inTensors || numPts < 1 )
@@ -428,6 +438,7 @@ int vtkTensorGlyph::RequestData(
   return 1;
 }
 
+//----------------------------------------------------------------------------
 void vtkTensorGlyph::SetSourceConnection(int id, vtkAlgorithmOutput* algOutput)
 {
   if (id < 0)
@@ -453,11 +464,13 @@ void vtkTensorGlyph::SetSourceConnection(int id, vtkAlgorithmOutput* algOutput)
     }
 }
 
+//----------------------------------------------------------------------------
 void vtkTensorGlyph::SetSource(vtkPolyData *source)
 {
   this->SetInput(1, source);
 }
 
+//----------------------------------------------------------------------------
 vtkPolyData *vtkTensorGlyph::GetSource()
 {
   if (this->GetNumberOfInputConnections(1) < 1)
@@ -467,6 +480,7 @@ vtkPolyData *vtkTensorGlyph::GetSource()
   return vtkPolyData::SafeDownCast(this->GetExecutive()->GetInputData(1, 0));
 }
 
+//----------------------------------------------------------------------------
 int vtkTensorGlyph::FillInputPortInformation(int port, vtkInformation *info)
 {
   if (port == 1)
@@ -478,6 +492,7 @@ int vtkTensorGlyph::FillInputPortInformation(int port, vtkInformation *info)
   return 1;
 }
 
+//----------------------------------------------------------------------------
 void vtkTensorGlyph::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
