@@ -39,7 +39,7 @@
 
 #define VTK_MAX_PLOTS 50
 
-vtkCxxRevisionMacro(vtkXYPlotActor, "$Revision: 1.59 $");
+vtkCxxRevisionMacro(vtkXYPlotActor, "$Revision: 1.69 $");
 vtkStandardNewMacro(vtkXYPlotActor);
 
 vtkCxxSetObjectMacro(vtkXYPlotActor,TitleTextProperty,vtkTextProperty);
@@ -82,8 +82,11 @@ vtkXYPlotActor::vtkXYPlotActor()
   this->AxisTitleTextProperty = vtkTextProperty::New();
   this->AxisTitleTextProperty->ShallowCopy(this->AxisLabelTextProperty);
 
-  this->LabelFormat = new char[8]; 
-  sprintf(this->LabelFormat,"%s","%-#6.3g");
+  this->XLabelFormat = new char[8]; 
+  sprintf(this->XLabelFormat,"%s","%-#6.3g");
+
+  this->YLabelFormat = new char[8]; 
+  sprintf(this->YLabelFormat,"%s","%-#6.3g");
 
   this->Logx = 0;
   
@@ -176,6 +179,12 @@ vtkXYPlotActor::vtkXYPlotActor()
 
   this->CachedSize[0] = 0;
   this->CachedSize[1] = 0;
+
+  this->AdjustXLabels = 1;
+  this->AdjustYLabels = 1;
+  this->AdjustTitlePosition = 1;
+  this->TitlePosition[0] = 0.5;
+  this->TitlePosition[1] = 0.9;
 }
 
 //----------------------------------------------------------------------------
@@ -214,7 +223,8 @@ vtkXYPlotActor::~vtkXYPlotActor()
   this->SetTitle(0);
   this->SetXTitle(0);
   this->SetYTitle(0);
-  this->SetLabelFormat(0);
+  this->SetXLabelFormat(0);
+  this->SetYLabelFormat(0);
 
   this->XAxis->Delete();
   this->YAxis->Delete();
@@ -579,12 +589,12 @@ int vtkXYPlotActor::RenderOpaqueGeometry(vtkViewport *viewport)
     if (this->AxisLabelTextProperty &&
         this->AxisLabelTextProperty->GetMTime() > this->BuildTime)
       {
-      if (this->XAxis->GetTitleTextProperty())
+      if (this->XAxis->GetLabelTextProperty())
         {
         this->XAxis->GetLabelTextProperty()->ShallowCopy(
           this->AxisLabelTextProperty);
         }
-      if (this->YAxis->GetTitleTextProperty())
+      if (this->YAxis->GetLabelTextProperty())
         {
         this->YAxis->GetLabelTextProperty()->ShallowCopy(
           this->AxisLabelTextProperty);
@@ -627,9 +637,18 @@ int vtkXYPlotActor::RenderOpaqueGeometry(vtkViewport *viewport)
       range[0] = this->XRange[0];
       range[1] = this->XRange[1];
       }
+    
+    if ( this->AdjustXLabels )
+      {
+      vtkAxisActor2D::ComputeRange(range, xRange, this->NumberOfXLabels,
+                                   numTicks, interval);
+      }
+    else
+      {
+      xRange[0] = range[0];
+      xRange[1] = range[1];
+      }
 
-    vtkAxisActor2D::ComputeRange(range, xRange, this->NumberOfXLabels,
-                                 numTicks, interval);
     if ( !this->ExchangeAxes )
       {
       this->XComputedRange[0] = xRange[0];
@@ -674,8 +693,17 @@ int vtkXYPlotActor::RenderOpaqueGeometry(vtkViewport *viewport)
       yrange[0] = this->YRange[0];
       yrange[1] = this->YRange[1];
       }
-    vtkAxisActor2D::ComputeRange(yrange, yRange, this->NumberOfYLabels,
-                                 numTicks, interval);
+
+    if ( this->AdjustYLabels )
+      {
+      vtkAxisActor2D::ComputeRange(yrange, yRange, this->NumberOfYLabels,
+                                   numTicks, interval);
+      }
+    else
+      {
+      yRange[0] = yrange[0];
+      yRange[1] = yrange[1];
+      }
 
     if ( !this->ExchangeAxes )
       {
@@ -716,15 +744,22 @@ int vtkXYPlotActor::RenderOpaqueGeometry(vtkViewport *viewport)
           this->TitleTextProperty);
         }
 
-      vtkAxisActor2D::SetFontSize(viewport, 
-                                  this->TitleMapper, 
-                                  size, 
-                                  1.0,
-                                  stringSize);
+      vtkTextMapper::SetRelativeFontSize(this->TitleMapper, viewport, 
+                                         size, stringSize, 0.015);
 
-      this->TitleActor->GetPositionCoordinate()->SetValue(
-        pos[0] + 0.5 * (pos2[0] - pos[0]) - stringSize[0] / 2.0, 
-        pos2[1] - stringSize[1] / 2.0);
+      if (this->AdjustTitlePosition)
+        {
+        this->TitleActor->GetPositionCoordinate()->SetCoordinateSystemToViewport();
+        this->TitleActor->GetPositionCoordinate()->SetValue(
+          pos[0] + 0.5 * (pos2[0] - pos[0]) - stringSize[0] / 2.0, 
+          pos2[1] - stringSize[1] / 2.0);
+        }
+      else
+        {
+        this->TitleActor->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
+        this->TitleActor->GetPositionCoordinate()->SetValue(
+          this->TitlePosition[0], this->TitlePosition[1]);
+        }
 
       this->TitleActor->SetProperty(this->GetProperty());
       }
@@ -760,6 +795,14 @@ int vtkXYPlotActor::RenderOpaqueGeometry(vtkViewport *viewport)
   return renderedSomething;
 }
 
+//-----------------------------------------------------------------------------
+// Description:
+// Does this prop have some translucent polygonal geometry?
+int vtkXYPlotActor::HasTranslucentPolygonalGeometry()
+{
+  return 0;
+}
+
 //----------------------------------------------------------------------------
 const char *vtkXYPlotActor::GetXValuesAsString()
 {
@@ -908,7 +951,8 @@ void vtkXYPlotActor::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "Number Of X Labels: " << this->NumberOfXLabels << "\n";
   os << indent << "Number Of Y Labels: " << this->NumberOfYLabels << "\n";
 
-  os << indent << "Label Format: " << this->LabelFormat << "\n";
+  os << indent << "X Label Format: " << this->XLabelFormat << "\n";
+  os << indent << "Y Label Format: " << this->YLabelFormat << "\n";
   os << indent << "Border: " << this->Border << "\n";
   
   os << indent << "X Range: ";
@@ -953,6 +997,17 @@ void vtkXYPlotActor::PrintSelf(ostream& os, vtkIndent indent)
   this->LegendActor->PrintSelf( os << endl, i2);
   os << indent << "Glyph Source:";
   this->GlyphSource->PrintSelf( os << endl, i2);
+
+  os << indent << "AdjustXLabels: " 
+     << this->AdjustXLabels << endl;
+  os << indent << "AdjustYLabels: " 
+     << this->AdjustYLabels << endl;
+  os << indent << "AdjustTitlePosition: " 
+     << this->AdjustTitlePosition << endl;
+  os << indent << "TitlePosition: " 
+     << this->TitlePosition[0] << " "
+     << this->TitlePosition[1] << " "
+     << endl;
 }
 
 //----------------------------------------------------------------------------
@@ -970,6 +1025,11 @@ void vtkXYPlotActor::ComputeXRange(double range[2], double *lengths)
         (ds = this->InputList->GetNextDataSet(dsit)); dsNum++)
     {
     numPts = ds->GetNumberOfPoints();
+    if (numPts == 0)
+      {
+      vtkErrorMacro(<<"No scalar data to plot!");
+      continue;
+      }
 
     if ( this->XValues != VTK_XYPLOT_INDEX )
       {
@@ -1103,6 +1163,26 @@ void vtkXYPlotActor::ComputeYRange(double range[2])
     }//over all datasets
 }
 
+//----------------------------------------------------------------------------
+static inline int vtkXYPlotActorGetComponent(vtkFieldData* field,
+  vtkIdType tuple, int component, double* val)
+{
+  int array_comp;
+  int array_index = field->GetArrayContainingComponent(component, array_comp);
+  if (array_index < 0)
+    {
+    return 0;
+    }
+  vtkDataArray* da = field->GetArray(array_index);
+  if (!da)
+    {
+    // non-numeric array.
+    return 0;
+    }
+  *val = da->GetComponent(tuple, array_comp);
+  return 1;
+}
+
 //----------------------------------------------------------------------------
 void vtkXYPlotActor::ComputeDORange(double xrange[2], double yrange[2], 
                                     double *lengths)
@@ -1112,21 +1192,34 @@ void vtkXYPlotActor::ComputeDORange(double xrange[2], double yrange[2],
   vtkFieldData *field;
   int doNum, numColumns;
   vtkIdType numTuples, numRows, num, ptId, maxNum;
-  double maxLength=0.0, x, y, xPrev = 0.0;
+  double maxLength=0.0;
+  double x = 0.0;
+  double y = 0.0;
+  double xPrev = 0.0;
   vtkDataArray *array;
-
+  
+  // NOTE: FieldData can have non-numeric arrays. However, XY plot can only 
+  // work on numeric arrays (or vtkDataArray subclasses). 
+  
   xrange[0] = yrange[0] = VTK_DOUBLE_MAX;
   xrange[1] = yrange[1] = -VTK_DOUBLE_MAX;
   vtkCollectionSimpleIterator doit;
   for ( doNum=0, maxNum=0, this->DataObjectInputList->InitTraversal(doit); 
         (dobj = this->DataObjectInputList->GetNextDataObject(doit)); doNum++)
     {
+
     lengths[doNum] = 0.0;
     field = dobj->GetFieldData();
-    numColumns = field->GetNumberOfComponents(); //number of "columns"
+    numColumns = field->GetNumberOfComponents();  //number of "columns"
+      // numColumns includes the components for non-numeric arrays as well.
     for (numRows = VTK_LARGE_ID, i=0; i<field->GetNumberOfArrays(); i++)
       {
       array = field->GetArray(i);
+      if (!array)
+        {
+        // non-numeric array, skip.
+        continue;
+        }
       numTuples = array->GetNumberOfTuples();
       if ( numTuples < numRows )
         {
@@ -1142,13 +1235,24 @@ void vtkXYPlotActor::ComputeDORange(double xrange[2], double yrange[2],
       // gather the information to form a plot
       for ( ptId=0; ptId < num; ptId++ )
         {
+        int status = 0;
+      
         if ( this->DataObjectPlotMode == VTK_XYPLOT_ROW )
           {
-          x = field->GetComponent(this->XComponent->GetValue(doNum), ptId);
+          // x = field->GetComponent(this->XComponent->GetValue(doNum), ptId);
+          status = ::vtkXYPlotActorGetComponent(field,
+            this->XComponent->GetValue(doNum), ptId, &x);
           }
         else //if ( this->DataObjectPlotMode == VTK_XYPLOT_COLUMN )
           {
-          x = field->GetComponent(ptId, this->XComponent->GetValue(doNum));
+          // x = field->GetComponent(ptId, this->XComponent->GetValue(doNum));
+          status = ::vtkXYPlotActorGetComponent(field,
+            ptId, this->XComponent->GetValue(doNum), &x);
+          }
+        if (!status)
+          {
+          // requested component falls in a non-numeric array, skip it.
+          continue;
           }
         if ( ptId == 0 )
           {
@@ -1203,13 +1307,24 @@ void vtkXYPlotActor::ComputeDORange(double xrange[2], double yrange[2],
     // Get the y-values
     for ( ptId=0; ptId < num; ptId++ )
       {
+      int status = 0;
       if ( this->DataObjectPlotMode == VTK_XYPLOT_ROW )
         {
-        y = field->GetComponent(this->YComponent->GetValue(doNum), ptId);
+        //y = field->GetComponent(this->YComponent->GetValue(doNum), ptId);
+        status = ::vtkXYPlotActorGetComponent(field,
+          this->YComponent->GetValue(doNum), ptId, &y);
         }
       else //if ( this->DataObjectPlotMode == VTK_XYPLOT_COLUMN )
         {
-        y = field->GetComponent(ptId, this->YComponent->GetValue(doNum));
+        //y = field->GetComponent(ptId, this->YComponent->GetValue(doNum));
+        status = ::vtkXYPlotActorGetComponent(field,
+          ptId, this->YComponent->GetValue(doNum), &y);
+        }
+      if (!status)
+        {
+        // requested component falls in non-numeric array.
+        // skip.
+        continue;
         }
       if ( y < yrange[0] )
         {
@@ -1442,9 +1557,15 @@ void vtkXYPlotActor::CreatePlotData(int *pos, int *pos2, double xRange[2],
       // determine the shape of the field
       field = dobj->GetFieldData();
       numColumns = field->GetNumberOfComponents(); //number of "columns"
+      // numColumns also includes non-numeric array components.
       for (numRows = VTK_LARGE_ID, i=0; i<field->GetNumberOfArrays(); i++)
         {
         array = field->GetArray(i);
+        if (!array)
+          {
+          // skip non-numeric arrays.
+          continue;
+          }
         numTuples = array->GetNumberOfTuples();
         if ( numTuples < numRows )
           {
@@ -1462,15 +1583,42 @@ void vtkXYPlotActor::CreatePlotData(int *pos, int *pos2, double xRange[2],
       // gather the information to form a plot
       for ( numLinePts=0, length=0.0, ptId=0; ptId < numPts; ptId++ )
         {
+        int status1, status2;
         if ( this->DataObjectPlotMode == VTK_XYPLOT_ROW )
           {
-          x[0] = field->GetComponent(this->XComponent->GetValue(doNum),ptId);
-          xyz[1] = field->GetComponent(this->YComponent->GetValue(doNum),ptId);
+          //x[0] = field->GetComponent(this->XComponent->GetValue(doNum),ptId);
+          //xyz[1] = field->GetComponent(this->YComponent->GetValue(doNum),ptId);
+          status1 = ::vtkXYPlotActorGetComponent(field,
+            this->XComponent->GetValue(doNum), ptId, &x[0]);
+          status2 = ::vtkXYPlotActorGetComponent(field,
+            this->YComponent->GetValue(doNum), ptId, &xyz[1]);
           }
         else //if ( this->DataObjectPlotMode == VTK_XYPLOT_COLUMN )
           {
-          x[0] = field->GetComponent(ptId, this->XComponent->GetValue(doNum));
-          xyz[1] = field->GetComponent(ptId, this->YComponent->GetValue(doNum));
+          //x[0] = field->GetComponent(ptId, this->XComponent->GetValue(doNum));
+          //xyz[1] = field->GetComponent(ptId, this->YComponent->GetValue(doNum));
+
+          status1 = ::vtkXYPlotActorGetComponent(field,
+            ptId, this->XComponent->GetValue(doNum), &x[0]);
+
+          if (!status1)
+            {
+            vtkWarningMacro(<< this->XComponent->GetValue(doNum) << " is a non-numeric component.");
+            }
+          
+          status2 = ::vtkXYPlotActorGetComponent(field,
+            ptId, this->YComponent->GetValue(doNum), &xyz[1]);
+
+          if (!status2)
+            {
+            vtkWarningMacro(<< this->YComponent->GetValue(doNum) << " is a non-numeric component.");
+            }
+          }
+        if (!status1 || !status2)
+          {
+          // component is non-numeric.
+          // Skip it.
+          continue;
           }
 
         switch (this->XValues)
@@ -1629,13 +1777,11 @@ void vtkXYPlotActor::PlaceAxes(vtkViewport *viewport, int *size,
   // Estimate the padding around the X and Y axes
   tprop->ShallowCopy(axisX->GetTitleTextProperty());
   textMapper->SetInput(axisX->GetTitle());
-  vtkAxisActor2D::SetFontSize(
-    viewport, textMapper, size, fontFactorX, titleSizeX);
+  vtkTextMapper::SetRelativeFontSize(textMapper, viewport, size, titleSizeX, 0.015*fontFactorX);
 
   tprop->ShallowCopy(axisY->GetTitleTextProperty());
   textMapper->SetInput(axisY->GetTitle());
-  vtkAxisActor2D::SetFontSize(
-    viewport, textMapper, size, fontFactorY, titleSizeY);
+  vtkTextMapper::SetRelativeFontSize(textMapper, viewport, size, titleSizeY, 0.015*fontFactorY);
 
   // At this point the thing to do would be to actually ask the Y axis
   // actor to return the largest label.
@@ -1644,16 +1790,14 @@ void vtkXYPlotActor::PlaceAxes(vtkViewport *viewport, int *size,
   sprintf(str2, axisY->GetLabelFormat(), axisY->GetAdjustedRange()[1]);
   tprop->ShallowCopy(axisY->GetLabelTextProperty());
   textMapper->SetInput(strlen(str1) > strlen(str2) ? str1 : str2);
-  vtkAxisActor2D::SetFontSize(
-    viewport, textMapper, size, labelFactorY * fontFactorY, labelSizeY);
+  vtkTextMapper::SetRelativeFontSize(textMapper, viewport, size, labelSizeY, 0.015*labelFactorY*fontFactorY);
 
   // We do only care of the height of the label in the X axis, so let's
   // use the min for example
   sprintf(str1, axisX->GetLabelFormat(), axisX->GetAdjustedRange()[0]);
   tprop->ShallowCopy(axisX->GetLabelTextProperty());
   textMapper->SetInput(str1);
-  vtkAxisActor2D::SetFontSize(
-    viewport, textMapper, size, labelFactorX * fontFactorX, labelSizeX);
+  vtkTextMapper::SetRelativeFontSize(textMapper, viewport, size, labelSizeX, 0.015*labelFactorX*fontFactorX);
 
   tickOffsetX = axisX->GetTickOffset();
   tickOffsetY = axisY->GetTickOffset();
@@ -2081,41 +2225,142 @@ double *vtkXYPlotActor::TransformPoint(int pos[2], int pos2[2],
 
   return xNew;
 }
-    
+
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetXTitlePosition(double position)
+{
+  this->XAxis->SetTitlePosition(position);
+}
+
+//----------------------------------------------------------------------------
+double vtkXYPlotActor::GetXTitlePosition()
+{
+  return this->XAxis->GetTitlePosition();
+}
+
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetYTitlePosition(double position)
+{
+  this->YAxis->SetTitlePosition(1.0-position);
+}
+
+//----------------------------------------------------------------------------
+double vtkXYPlotActor::GetYTitlePosition()
+{
+  return this->YAxis->GetTitlePosition();
+}
+
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetAdjustXLabels(int adjust)
+{
+  this->AdjustXLabels = adjust;
+  this->XAxis->SetAdjustLabels(adjust);
+}
+
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetAdjustYLabels(int adjust)
+{
+  this->AdjustYLabels = adjust;
+  this->YAxis->SetAdjustLabels(adjust);
+}
+
 //----------------------------------------------------------------------------
 void vtkXYPlotActor::SetLabelFormat(const char* _arg)
 {
-  if (this->LabelFormat == NULL && _arg == NULL) 
+  this->SetXLabelFormat(_arg);
+  this->SetYLabelFormat(_arg);
+}
+    
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetXLabelFormat(const char* _arg)
+{
+  if (this->XLabelFormat == NULL && _arg == NULL) 
     { 
     return;
     }
 
-  if (this->LabelFormat && _arg && (!strcmp(this->LabelFormat,_arg))) 
+  if (this->XLabelFormat && _arg && (!strcmp(this->XLabelFormat,_arg))) 
     { 
     return;
     }
 
-  if (this->LabelFormat) 
+  if (this->XLabelFormat) 
     { 
-    delete [] this->LabelFormat; 
+    delete [] this->XLabelFormat; 
     }
 
   if (_arg)
     {
-    this->LabelFormat = new char[strlen(_arg)+1];
-    strcpy(this->LabelFormat,_arg);
+    this->XLabelFormat = new char[strlen(_arg)+1];
+    strcpy(this->XLabelFormat,_arg);
     }
   else
     {
-    this->LabelFormat = NULL;
+    this->XLabelFormat = NULL;
     }
 
-  this->XAxis->SetLabelFormat(this->LabelFormat);
-  this->YAxis->SetLabelFormat(this->LabelFormat);
+  this->XAxis->SetLabelFormat(this->XLabelFormat);
 
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetYLabelFormat(const char* _arg)
+{
+  if (this->YLabelFormat == NULL && _arg == NULL) 
+    { 
+    return;
+    }
+
+  if (this->YLabelFormat && _arg && (!strcmp(this->YLabelFormat,_arg))) 
+    { 
+    return;
+    }
+
+  if (this->YLabelFormat) 
+    { 
+    delete [] this->YLabelFormat; 
+    }
+
+  if (_arg)
+    {
+    this->YLabelFormat = new char[strlen(_arg)+1];
+    strcpy(this->YLabelFormat,_arg);
+    }
+  else
+    {
+    this->YLabelFormat = NULL;
+    }
+
+  this->YAxis->SetLabelFormat(this->YLabelFormat);
+
+  this->Modified();
+}
+
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetNumberOfXMinorTicks(int num)
+{
+  this->XAxis->SetNumberOfMinorTicks(num);
+}
+
+//----------------------------------------------------------------------------
+int vtkXYPlotActor::GetNumberOfXMinorTicks()
+{
+  return this->XAxis->GetNumberOfMinorTicks();
+}
+
+//----------------------------------------------------------------------------
+void vtkXYPlotActor::SetNumberOfYMinorTicks(int num)
+{
+  this->YAxis->SetNumberOfMinorTicks(num);
+}
+
+//----------------------------------------------------------------------------
+int vtkXYPlotActor::GetNumberOfYMinorTicks()
+{
+  return this->YAxis->GetNumberOfMinorTicks();
+}
+
 //----------------------------------------------------------------------------
 void vtkXYPlotActor::PrintAsCSV(ostream &os)
 {
@@ -2129,6 +2374,8 @@ void vtkXYPlotActor::PrintAsCSV(ostream &os)
     {
     vtkIdType numPts = ds->GetNumberOfPoints();
     scalars = ds->GetPointData()->GetScalars(this->SelectedInputScalars[dsNum]);
+    os << this->SelectedInputScalars[dsNum] << ",";
+
     component = this->SelectedInputScalarsComponent->GetValue(dsNum);
     for ( vtkIdType ptId=0; ptId < numPts; ptId++ )
       {
@@ -2143,6 +2390,24 @@ void vtkXYPlotActor::PrintAsCSV(ostream &os)
         }
       }
     os << endl;
+
+    if (dsNum == this->InputList->GetNumberOfItems()-1)
+      {
+      os << "X or T,";
+      for ( vtkIdType ptId=0; ptId < numPts; ptId++ )
+        {
+        double *x = ds->GetPoint(ptId);
+        if( ptId == 0 )
+          {
+          os << x[0];
+          }
+        else
+          {
+          os << "," << x[0];
+          }
+        }
+      os << endl;
+      }
+
     }
 }
-
