@@ -15,14 +15,17 @@
 #include "vtkGenericEnSightReader.h"
 
 #include "vtkCallbackCommand.h"
+#include "vtkCompositeDataPipeline.h"
 #include "vtkDataArrayCollection.h"
 #include "vtkDataArraySelection.h"
-#include "vtkDataSet.h"
 #include "vtkEnSight6BinaryReader.h"
 #include "vtkEnSight6Reader.h"
 #include "vtkEnSightGoldBinaryReader.h"
 #include "vtkEnSightGoldReader.h"
+#include "vtkMultiBlockDataSet.h"
 #include "vtkIdListCollection.h"
+#include "vtkInformation.h"
+#include "vtkInformationVector.h"
 #include "vtkObjectFactory.h"
 
 #include <vtkstd/string>
@@ -30,7 +33,7 @@
 #include <assert.h>
 #include <ctype.h> /* isspace */
 
-vtkCxxRevisionMacro(vtkGenericEnSightReader, "$Revision: 1.74 $");
+vtkCxxRevisionMacro(vtkGenericEnSightReader, "$Revision: 1.86 $");
 vtkStandardNewMacro(vtkGenericEnSightReader);
 
 vtkCxxSetObjectMacro(vtkGenericEnSightReader,TimeSets, 
@@ -77,6 +80,7 @@ vtkGenericEnSightReader::vtkGenericEnSightReader()
   this->NumberOfComplexVectorsPerElement = 0;
   
   this->TimeValue = 0;
+
   this->MinimumTimeValue = 0;
   this->MaximumTimeValue = 0;
   
@@ -88,6 +92,8 @@ vtkGenericEnSightReader::vtkGenericEnSightReader()
 
   this->ByteOrder = FILE_UNKNOWN_ENDIAN;
   
+  this->ParticleCoordinatesByIndex = 0;
+
   this->EnSightVersion = -1;
   
   this->PointDataArraySelection = vtkDataArraySelection::New();
@@ -106,7 +112,7 @@ vtkGenericEnSightReader::vtkGenericEnSightReader()
   this->SelectionModifiedDoNotCallModified = 0;
   this->TranslationTable = new TranslationTableType;
 
-  this->SetNumberOfOutputPorts(0);
+  this->SetNumberOfInputPorts(0);
 }
 
 //----------------------------------------------------------------------------
@@ -172,19 +178,33 @@ vtkGenericEnSightReader::~vtkGenericEnSightReader()
 }
 
 //----------------------------------------------------------------------------
-void vtkGenericEnSightReader::Execute()
+int vtkGenericEnSightReader::RequestData(
+  vtkInformation *vtkNotUsed(request),
+  vtkInformationVector **vtkNotUsed(inputVector),
+  vtkInformationVector *outputVector)
 {
   int i;
 
   if ( !this->Reader )
     {
-    return;
+    return 0;
     }
 
+  vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
   // Set the real reader's data array selections from ours.
   this->SetReaderDataArraySelectionSetsFromSelf();
   
   this->Reader->SetTimeValue(this->GetTimeValue());
+  this->Reader->UpdateInformation();
+  vtkInformation* tmpOutInfo =
+    this->Reader->GetExecutive()->GetOutputInformation(0);
+  if (outInfo->Has(vtkStreamingDemandDrivenPipeline::UPDATE_TIME_STEPS()))
+    {
+    tmpOutInfo->CopyEntry(
+      outInfo,
+      vtkStreamingDemandDrivenPipeline::UPDATE_TIME_STEPS());
+    }
   this->Reader->Update();
 
   this->NumberOfScalarsPerNode = this->Reader->GetNumberOfScalarsPerNode();
@@ -209,53 +229,11 @@ void vtkGenericEnSightReader::Execute()
     this->Reader->GetNumberOfComplexScalarsPerElement();
   this->NumberOfComplexVectorsPerElement =
     this->Reader->GetNumberOfComplexScalarsPerElement();
-  
-  for (i = 0; i < this->Reader->GetNumberOfOutputs(); i++)
-    {
-    vtkDataObject* output = this->GetOutput(i);
-    if ( ! output )
-      {
-      // Copy the output rather than setting directly.
-      // When we set directly, the internal reader looses its output.
-      // Next execute, a new output is added, 
-      // and another partid is added too.
-      vtkDataObject* tmp = this->Reader->GetOutput(i);
-      if( tmp )
-        {
-        output = tmp->NewInstance();
-        this->SetNthOutput(i, output); // law: this causes the extra partid bug
-        output->ShallowCopy(tmp);
-        output->CopyInformation(tmp);
-        output->Delete();
-        // Used later.
-        //output = NULL;
-        }
-      else
-        {
-        this->SetNthOutput(i, 0);
-        }
-      }
-    else
-      {
-      // We don't know the number of outputs or whole extent of the
-      // internal reader's data until after it executes.  Therefore,
-      // the update extent of the reader is set to empty.  Since the
-      // reader ignores the update extent anyway, it reads correctly,
-      // but then this shallow copy destroys this reader's update
-      // extent.  Save it and restore.
-      int tempExtent[6];
-      output->GetUpdateExtent(tempExtent);
-      output->ShallowCopy(this->Reader->GetOutput(i));
-      output->SetUpdateExtent(tempExtent);
-      }
-    // Since most unstructured filters in VTK generate all their data once,
-    // make it the default.
-    // protected: if ( output->GetExtentType() == VTK_PIECES_EXTENT )
-    if (output && ( output->IsA("vtkPolyData") || output->IsA("vtkUnstructuredGrid")))
-      {
-      output->SetMaximumNumberOfPieces(1);
-      }
-    }
+
+  vtkMultiBlockDataSet *output = vtkMultiBlockDataSet::SafeDownCast(
+    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+  output->ShallowCopy(this->Reader->GetOutput());
+
   for (i = 0; i < this->Reader->GetNumberOfVariables(); i++)
     {
     this->AddVariableDescription(this->Reader->GetDescription(i));
@@ -269,6 +247,8 @@ void vtkGenericEnSightReader::Execute()
     this->AddComplexVariableType(this->Reader->GetComplexVariableType(i));
     this->NumberOfComplexVariables++;
     }
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -287,6 +267,7 @@ void vtkGenericEnSightReader::SetTimeValue(float value)
 int vtkGenericEnSightReader::DetermineEnSightVersion()
 {
   char line[256], subLine[256], subLine1[256], subLine2[256], binaryLine[81];
+  char *binaryLinePtr;
   int stringRead;
   int timeSet = 1, fileSet = 1;
   int xtimeSet= 1, xfileSet= 1;
@@ -378,10 +359,15 @@ int vtkGenericEnSightReader::DetermineEnSightVersion()
               }
             if (strrchr(fileName, '*') != NULL)
               {
-              // reopen case file; find right time set and fill in
+              // RE-open case file; find right time set and fill in
               // wildcards from there if possible; if not, then find right
               // file set and fill in wildcards from there.
-              this->ReplaceWildcards(fileName, timeSet, fileSet);
+              if ( this->ReplaceWildcards(fileName, timeSet, fileSet) == 0 )
+                {
+                vtkErrorMacro(
+                  "upon DetermineEnSightVersion()'s call to ReplaceWildCards()");
+                return -1;
+                }
               }
             sfilename = "";
             if (this->FilePath)
@@ -414,7 +400,15 @@ int vtkGenericEnSightReader::DetermineEnSightVersion()
           
             this->ReadBinaryLine(binaryLine);
             binaryLine[80] = '\0';
-            sscanf(binaryLine, " %*s %s", subLine);
+            // because fortran stores 4 length bytes at the start, 
+            // if the strlen is less than 4, skip the first 4
+            // and jump to the start of the actual string
+            binaryLinePtr = &binaryLine[0];
+            if (strlen(binaryLine)<4) 
+              {
+              binaryLinePtr = &binaryLine[4];
+              }
+            sscanf(binaryLinePtr, " %*s %s", subLine);
             // If the file is ascii, there might not be a null
             // terminator. This leads to a UMR in sscanf
             if (strncmp(subLine,"Binary",6) == 0 ||
@@ -576,7 +570,14 @@ void vtkGenericEnSightReader::SetCaseFileName(const char* fileName)
   
   // strip off the path and save it as FilePath if it was included in the
   // filename
-  if ((endingSlash = strrchr(this->CaseFileName, '/')))
+  endingSlash = strrchr(this->CaseFileName, '/');
+  if(endingSlash == NULL)
+    {
+    // check Windows directory separator
+    endingSlash = strrchr(this->CaseFileName, '\\');
+    }
+
+  if (endingSlash)
     {
     position = endingSlash - this->CaseFileName + 1;
     path = new char[position + 1];
@@ -590,7 +591,6 @@ void vtkGenericEnSightReader::SetCaseFileName(const char* fileName)
     delete [] path;
     delete [] newFileName;
     }
-      
 }
 
 //----------------------------------------------------------------------------
@@ -604,7 +604,7 @@ int vtkGenericEnSightReader::ReadLine(char result[256])
     {
     return 0;
     }
-  
+
   return 1;
 }
 
@@ -657,25 +657,10 @@ int vtkGenericEnSightReader::ReadNextDataLine(char result[256])
 }
 
 //----------------------------------------------------------------------------
-void vtkGenericEnSightReader::Update()
-{
-  int i;
-  
-  this->UpdateInformation();
-  this->Execute();
-  
-  for (i = 0; i < this->GetNumberOfOutputs(); i++)
-    {
-    if ( this->GetOutput(i) )
-      {
-      this->GetOutput(i)->DataHasBeenGenerated();
-      this->GetOutput(i)->SetUpdateExtentToWholeExtent();
-      }
-    }
-}
-
-//----------------------------------------------------------------------------
-void vtkGenericEnSightReader::ExecuteInformation()
+int vtkGenericEnSightReader::RequestInformation(
+  vtkInformation *request,
+  vtkInformationVector **inputVector,
+  vtkInformationVector *outputVector)
 {
   int version = this->DetermineEnSightVersion();
   int createReader = 1;
@@ -760,7 +745,7 @@ void vtkGenericEnSightReader::ExecuteInformation()
     {
     vtkErrorMacro("Error determining EnSightVersion");
     this->EnSightVersion = -1;
-    return;
+    return 0;
     }
   this->EnSightVersion = version;
   
@@ -769,9 +754,29 @@ void vtkGenericEnSightReader::ExecuteInformation()
   this->Reader->SetReadAllVariables(this->ReadAllVariables);
   this->Reader->SetCaseFileName(this->GetCaseFileName());
   this->Reader->SetFilePath(this->GetFilePath());
-  this->Reader->SetByteOrder(this->ByteOrder);
-  this->Reader->UpdateInformation();
   
+  // The following line, explicitly initializing this->ByteOrder to
+  // FILE_UNKNOWN_ENDIAN,  MUST !!NOT!! be removed as it is used to
+  // force vtkEnSightGoldBinaryReader::ReadPartId(...) to determine
+  // the actual endian type. Otherwise the endian type, the default
+  // value from combobox 'Byte Order' of the user interface -------
+  // FILE_BIG_ENDIAN unless the user manually toggles the combobox,
+  // would be forwarded to  this->Reader->ByteOrder through the next
+  // line and therefore would prevent vtkEnSightGoldBinaryReader::
+  // ReadPartId(...) from automatically checking the endian type. As
+  // a consequence, little-endian files such as the one mentioned in
+  // bug #0008237 would not be loadable. The following line might be
+  // removed ONLY WHEN the combobox is removed through
+  // ParaViews\Servers\ServerManager\Resources\readers.xml.
+  // Thus it is highly suggested that the following line be retained
+  // to guarantee the fix to bug #0007424 -- automatic determination
+  // of the endian type.
+  this->ByteOrder = FILE_UNKNOWN_ENDIAN;
+  
+  this->Reader->SetByteOrder(this->ByteOrder);
+  this->Reader->RequestInformation(request, inputVector, outputVector);
+  this->Reader->SetParticleCoordinatesByIndex(this->ParticleCoordinatesByIndex);
+
   this->SetTimeSets(this->Reader->GetTimeSets());
   if(!this->TimeValueInitialized)
     {
@@ -781,7 +786,9 @@ void vtkGenericEnSightReader::ExecuteInformation()
   this->MaximumTimeValue = this->Reader->GetMaximumTimeValue();
   
   // Copy new data array selections from internal reader.
-  this->SetDataArraySelectionSetsFromReader();  
+  this->SetDataArraySelectionSetsFromReader();
+
+  return 1;
 }
 
 //----------------------------------------------------------------------------
@@ -1040,99 +1047,200 @@ int vtkGenericEnSightReader::GetComplexVariableType(int n)
 }
 
 //----------------------------------------------------------------------------
-void vtkGenericEnSightReader::ReplaceWildcards(char* fileName, int timeSet,
-                                               int fileSet)
+int vtkGenericEnSightReader::ReplaceWildcards(char* fileName, int timeSet,
+                                              int fileSet)
 {
-  char line[256], subLine[256];
-  int cmpTimeSet, cmpFileSet, fileNameNum;
+  char line[256],  subLine[256];
+  int  cmpTimeSet, cmpFileSet, fileNameNum, lineReadResult, lineScanResult;
   
   vtkstd::string sfilename;
-  if (this->FilePath)
+  if ( this->FilePath )
     {
     sfilename = this->FilePath;
-    if (sfilename.at(sfilename.length()-1) != '/')
+    if (  sfilename.at( sfilename.length() - 1 ) != '/'  )
       {
       sfilename += "/";
       }
     sfilename += this->CaseFileName;
-    vtkDebugMacro("full path to case file: " 
-                  << sfilename.c_str());
+    vtkDebugMacro( "full path to case file: " << sfilename.c_str() );
     }
   else
     {
     sfilename = this->CaseFileName;
     }
   
-  this->IS = new ifstream(sfilename.c_str(), ios::in);
+  // We have got a valid CASE file name
+  this->IS = new ifstream( sfilename.c_str(), ios::in );
+  
+  // Below is a revamped version of the code in support of inline & non-inline
+  // file name numbers, in a CASE file, of which the first one is obtained to
+  // make a geometry file name, through wildcards replacement, used to determine
+  // the specific EnSight version.
 
-  // We already know we have a valid case file if we've gotten to this point.
-  
-  this->ReadLine(line);
-  while (strncmp(line, "TIME", 4) != 0)
+  // Locate the 'TIME' section
+  do
     {
-    this->ReadLine(line);
+    if ( this->ReadNextDataLine(line) == 0 )
+      {
+      vtkErrorMacro(
+        "ReplaceWildCards() failed to find the 'TIME' section!" );
+      delete this->IS;
+      this->IS = NULL;
+      return 0;
+      }
+    } while ( strncmp(line, "TIME", 4) != 0 );
+  
+    
+  // Locate the very 'time set' entry by the index
+  cmpTimeSet = -10000;
+  do
+    {
+    if ( this->ReadNextDataLine(line) == 0 )
+      {
+      vtkErrorMacro(
+        "ReplaceWildCards() failed to find the target 'time set' entry!" );
+      delete this->IS;
+      this->IS = NULL;
+      return 0;
+      }
+      
+      // 'time set: <int>' --- where to obtain cmpTimeSet, a time set index
+      lineScanResult = sscanf(line, "%*s %s %d", subLine, &cmpTimeSet); 
+    } while ( lineScanResult != 2 || strncmp(line, "time", 4) != 0 ||
+              strncmp(subLine, "set", 3) != 0 || cmpTimeSet != timeSet );
+               
+  // Skip 'time set: <int>' and 'number of steps: <int>' to go to
+  // 'filename xxx: ...' --- where to obtain the actual file name number(s)
+  for ( int  i = 0;  i < 2;  i ++ )
+    {
+    lineReadResult = this->ReadNextDataLine(line);
+    if ( lineReadResult == 0 ||
+         // check 'filename xxx: ...' upon the second line (i = 1)
+         ( i == 1 && 
+           ( strncmp(line, "filename", 8)    != 0 ||
+             sscanf(line, "%*s %s", subLine) != 1
+           )
+         )
+       )
+      {
+      vtkErrorMacro(
+        "ReplaceWildCards() failed to find the target 'filename ...: ...' entry!" );
+      delete this->IS;
+      this->IS = NULL;
+      return 0;
+      }
     }
   
-  this->ReadNextDataLine(line);
-  sscanf(line, " %*s %*s %d", &cmpTimeSet);
-  while (cmpTimeSet != timeSet)
-    {
-    this->ReadNextDataLine(line);
-    this->ReadNextDataLine(line);
-    sscanf(line, " %s", subLine);
-    if (strncmp(subLine, "filename", 8) == 0)
-      {
-      this->ReadNextDataLine(line);
-      }
-    if (strncmp(subLine, "filename", 8) == 0)
-      {
-      this->ReadNextDataLine(line);
-      }
-    sscanf(line, " %*s %*s %d", &cmpTimeSet);
-    }
+  fileNameNum = -10000;
   
-  this->ReadNextDataLine(line); // number of timesteps
-  this->ReadNextDataLine(line);
-  sscanf(line, " %s", subLine);
-  if (strncmp(subLine, "filename", 8) == 0)
+  // 'filename numbers: ...'
+  if ( strncmp(subLine, "numbers", 7) == 0 )
     {
-    sscanf(line, " %*s %s", subLine);
-    if (strncmp(subLine, "start", 5) == 0)
+    // The filename number(s) may be provided on the line(s) following
+    // 'filename numbers:', as is usually the case --- not "inline". Thus we
+    // need to go to the FIRST line that indeed contains the filename number(s).
+    // Note that we only need to obtain the FIRST file name number since a
+    // single geometry file allows us to determine the EnSight version. This is
+    // based on the reasonable assumption that all geometry files referenced by
+    // a CASE file have the same EnSight version.
+    
+    // not "inline"
+    if ( sscanf(line, "%*s %*s %d", &fileNameNum) != 1 )
       {
-      sscanf(line, " %*s %*s %*s %d", &fileNameNum);
+      // let's go to the next VALID line that might be several empty lines apart
+      if ( this->ReadNextDataLine(line) == 0 )
+        {
+        vtkErrorMacro(
+          "ReplaceWildCards() failed to obtain any non-inline file name number!" );
+        delete this->IS;
+        this->IS = NULL;
+        return 0;
+        }
+      
+      // obtain the first file name number from the next valid line  
+      sscanf(line, "%d", &fileNameNum);
       }
-    else
-      {
-      sscanf(line, " %*s %*s %d", &fileNameNum);
-      }
-    this->ReplaceWildcardsHelper(fileName, fileNameNum);
     }
+  // 'filename start number: ...' --- followed by 'filename increment: ...'
   else
     {
-    while (strncmp(line, "FILE", 4) != 0)
+    char  subSubLine[256];
+    lineScanResult = sscanf( line, "%*s %s %s %d", 
+                             subLine, subSubLine, &fileNameNum );
+    if ( lineScanResult != 3 ||
+         strncmp(subLine,    "start",  5) != 0 ||
+         strncmp(subSubLine, "number", 6) != 0
+       )
       {
-      this->ReadLine(line);
+      vtkErrorMacro(
+        "ReplaceWildCards() failed to find 'filename start number: <int>'!" );
+      delete this->IS;
+      this->IS = NULL;
+      return 0;
       }
-    this->ReadNextDataLine(line);
-    sscanf(line, " %*s %*s %d", &cmpFileSet);
-    while (cmpFileSet != fileSet)
-      {
-      this->ReadNextDataLine(line);
-      this->ReadNextDataLine(line);
-      sscanf(line, " %s", subLine);
-      if (strncmp(subLine, "filename", 8) == 0)
-        {
-        this->ReadNextDataLine(line);
-        }
-      sscanf(line, " %*s %*s %d", &cmpFileSet);
-      }
-    this->ReadNextDataLine(line);
-    sscanf(line, " %*s %*s %d", &fileNameNum);
-    this->ReplaceWildcardsHelper(fileName, fileNameNum);
     }
   
+  // Let's resort to the 'FILE' section, just in case of a failure so far
+  if ( fileNameNum == -10000 )
+    {
+    // Locate the 'FILE' section
+    do
+      {
+      if ( this->ReadNextDataLine(line) == 0 )
+        {
+        vtkErrorMacro(
+          "ReplaceWildCards() failed to find the optional 'FILE' section!" );
+        delete this->IS;
+        this->IS = NULL;
+        return 0;
+        }
+      } while ( strncmp(line, "FILE", 4) != 0 );
+      
+    // Locate the very 'file set' entry by the index
+    cmpFileSet = -10000;
+    do
+      {
+      if ( this->ReadNextDataLine(line) == 0 )
+        {
+        vtkErrorMacro(
+          "ReplaceWildCards() failed to find the target 'file set' entry!" );
+        delete this->IS;
+        this->IS = NULL;
+        return 0;
+        }
+    
+      // 'file set: <int>' --- to obtain cmpFileSet, a file set index
+      lineScanResult = sscanf(line, "%*s %s %d", subLine, &cmpFileSet); 
+      } while ( lineScanResult != 2 || strncmp(line, "file", 4) != 0 ||
+                strncmp(subLine, "set", 3) != 0 || cmpFileSet != fileSet );
+        
+    // Skip 'file set: <int>' to go to
+    // 'filename index: <int>' --- where to obtain ONE actual file name
+    // Note that we here do NOT allow any non-'inline' scenarios since
+    // there is ONE AND ONLY ONE integer value, within a 'filename index: <int>'
+    // entry, that is used to specify a file name index. Thus any violation
+    // of this reasonable assumption is considered to use an invalid EnSight
+    // format that needs to be corrected by the EnSight CASE file user.
+    lineReadResult = this->ReadNextDataLine(line);
+    lineScanResult = sscanf( line, "%*s %s %d", subLine, &fileNameNum );
+    if ( lineReadResult == 0               || lineScanResult != 2 ||
+         strncmp(line, "filename", 8) != 0 || strncmp(subLine, "index", 5) != 0
+       )
+      {
+      vtkErrorMacro(
+        "ReplaceWildCards() failed to find 'filename index: <int>'!" );
+      delete this->IS;
+      this->IS = NULL;
+      return 0;
+      }   
+    }
+  
+  // So far we have got a file name index
+  this->ReplaceWildcardsHelper(fileName, fileNameNum);
   delete this->IS;
   this->IS = NULL;
+  
+  return  1;
 }
 
 //----------------------------------------------------------------------------
@@ -1275,10 +1383,13 @@ void vtkGenericEnSightReader::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "TimeSets: " << this->TimeSets << endl;
   os << indent << "ReadAllVariables: " << this->ReadAllVariables << endl;
   os << indent << "ByteOrder: " << this->ByteOrder << endl;
+  os << indent << "ParticleCoordinatesByIndex: " << this->ParticleCoordinatesByIndex << endl;
   os << indent << "CellDataArraySelection: " << this->CellDataArraySelection 
      << endl;
   os << indent << "PointDataArraySelection: " << this->PointDataArraySelection 
      << endl;
+  os << indent << "GeometryFileName: " << 
+     (this->GeometryFileName ? this->GeometryFileName : "(none)") << endl;
 }
 
 //----------------------------------------------------------------------------
@@ -1491,10 +1602,18 @@ void vtkGenericEnSightReader::SetCellArrayStatus(const char* name, int status)
 //----------------------------------------------------------------------------
 int vtkGenericEnSightReader::InsertNewPartId(int partId)
 {
-  int lastId = this->TranslationTable->PartIdMap.size();
-  this->TranslationTable->PartIdMap.insert( 
+  int lastId = static_cast<int>(this->TranslationTable->PartIdMap.size());
+  this->TranslationTable->PartIdMap.insert(
     vtkstd::map<int,int>::value_type(partId, lastId));
   lastId = this->TranslationTable->PartIdMap[partId];
   //assert( lastId == this->PartIdTranslationTable[partId] );
   return lastId;
 }
+
+//----------------------------------------------------------------------------
+int vtkGenericEnSightReader::FillOutputPortInformation(int vtkNotUsed(port),
+                                                       vtkInformation* info)
+{
+  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkMultiBlockDataSet");
+  return 1;
+}
