@@ -68,8 +68,9 @@ public:
 
   // For Tcl:
   // From C++ use GetPatientAge + GetAgeAsField
-  // Those function parse a DICOM string, and return the value of the number expressed
-  // this is either expressed in year, month or days. Thus if a string is expressed in years
+  // Those function parse a DICOM string, and return the value of the number
+  // expressed this is either expressed in year, month or days. Thus if a
+  // string is expressed in years 
   // GetPatientAgeDay/GetPatientAgeWeek/GetPatientAgeMonth will return 0 
   int GetPatientAgeYear();
   int GetPatientAgeMonth();
@@ -292,35 +293,40 @@ public:
   vtkSetStringMacro(Exposure);
   vtkGetStringMacro(Exposure);
 
-  // Interface to allow insertion of user define values, for instance in DICOM one would want to 
+  // Description:
+  // Get the direction cosine (default to 1,0,0,0,1,0)
+  vtkSetVector6Macro(DirectionCosine,double);
+  vtkGetVector6Macro(DirectionCosine,double);
+
+  // Interface to allow insertion of user define values, for instance in DICOM
+  // one would want to 
   // store the Protocol Name (0018,1030), in this case one would do:
   // AddUserDefinedValue( "Protocol Name", "T1W/SE/1024" );
-  void AddUserDefinedValue(const char *name, const char *value);
-  // Get a particular user value
-  const char *GetUserDefinedValue(const char *name);
-  // Get the number of user defined values
-  unsigned int GetNumberOfUserDefinedValues();
-  // Get a name/value by index
-  const char *GetUserDefinedNameByIndex(unsigned int idx);
-  const char *GetUserDefinedValueByIndex(unsigned int idx);
-
-  // Description:
-  // Copy the contents of p to this instance.
-  virtual void DeepCopy(vtkMedicalImageProperties *p);
+  virtual void AddUserDefinedValue(const char *name, const char *value);
+  virtual const char *GetUserDefinedValue(const char *name);
+  virtual unsigned int GetNumberOfUserDefinedValues();
+  virtual const char *GetUserDefinedNameByIndex(unsigned int idx);
+  virtual const char *GetUserDefinedValueByIndex(unsigned int idx);
+  virtual void RemoveAllUserDefinedValues();
 
   // Description:
   // Add/Remove/Query the window/level presets that may have been associated
   // to a medical image. Window is also known as 'width', level is also known
   // as 'center'. The same window/level pair can not be added twice.
-  // As a convenience, a comment (aka Explanation) can be associated to a preset.
-  // For ex: DICOM Window Center (0028,1050) = 00045\000470
+  // As a convenience, a comment (aka Explanation) can be associated to
+  // a preset.
+  // For ex:
+  // \verbatim
+  //         DICOM Window Center (0028,1050) = 00045\000470
   //         DICOM Window Width  (0028,1051) = 0106\03412
   //         DICOM Window Center Width Explanation (0028,1055) = WINDOW1\WINDOW2
-  virtual void AddWindowLevelPreset(double w, double l);
+  // \endverbatim
+  virtual int AddWindowLevelPreset(double w, double l);
   virtual void RemoveWindowLevelPreset(double w, double l);
   virtual void RemoveAllWindowLevelPresets();
   virtual int GetNumberOfWindowLevelPresets();
   virtual int HasWindowLevelPreset(double w, double l);
+  virtual int GetWindowLevelPresetIndex(double w, double l);
   virtual int GetNthWindowLevelPreset(int idx, double *w, double *l);
   virtual double* GetNthWindowLevelPreset(int idx);
   virtual void SetNthWindowLevelPresetComment(int idx, const char *comment);
@@ -328,8 +334,9 @@ public:
 
   // Description: 
   // Mapping from a sliceidx within a volumeidx into a DICOM Instance UID
-  // Some DICOM reader can populate this structure so that later on from a slice index
-  // in a vtkImageData volume we can backtrack and find out which 2d slice it was coming from
+  // Some DICOM reader can populate this structure so that later on from 
+  // a slice index in a vtkImageData volume we can backtrack and find out
+  // which 2d slice it was coming from
   const char *GetInstanceUIDFromSliceID(int volumeidx, int sliceid);
   void SetInstanceUIDFromSliceID(int volumeidx, int sliceid, const char *uid);
   
@@ -338,17 +345,21 @@ public:
   // not found.
   int GetSliceIDFromInstanceUID(int &volumeidx, const char *uid);
 
-//BTX
+  //BTX
   typedef enum {
     AXIAL = 0,
     CORONAL,
     SAGITTAL
   } OrientationType;
-//ETX
+  //ETX
   int GetOrientationType(int volumeidx);
   void SetOrientationType(int volumeidx, int orientation);
   static const char *GetStringFromOrientationType(unsigned int type);
 
+  // Description:
+  // Copy the contents of p to this instance.
+  virtual void DeepCopy(vtkMedicalImageProperties *p);
+
 protected:
   vtkMedicalImageProperties();
   ~vtkMedicalImageProperties();
@@ -384,6 +395,7 @@ protected:
   char *StudyDescription;
   char *StudyID;
   char *XRayTubeCurrent;
+  double DirectionCosine[6];
 
   // Description:
   // PIMPL Encapsulation for STL containers
