@@ -33,7 +33,7 @@
 #include <ctype.h>
 #include <sys/stat.h>
 
-vtkCxxRevisionMacro(vtkXMLParser, "$Revision: 1.27.2.2 $");
+vtkCxxRevisionMacro(vtkXMLParser, "$Revision: 1.30 $");
 vtkStandardNewMacro(vtkXMLParser);
 
 //----------------------------------------------------------------------------
@@ -531,13 +531,3 @@ void vtkXMLParserEndElement(void* parser, const char *name)
   // just casts the user data to a vtkXMLParser and calls EndElement.
   static_cast<vtkXMLParser*>(parser)->EndElement(name);
 }
-
-//----------------------------------------------------------------------------
-void vtkXMLParserCharacterDataHandler(void* parser, const char* data,
-                                      int length)
-{
-  // Character data handler that is registered with the XML_Parser.
-  // This just casts the user data to a vtkXMLParser and calls
-  // CharacterDataHandler.
-  static_cast<vtkXMLParser*>(parser)->CharacterDataHandler(data, length);
-}
