@@ -35,7 +35,7 @@
 
 #include <assert.h>
 
-vtkCxxRevisionMacro(vtkXMLUnstructuredDataWriter, "$Revision: 1.22 $");
+vtkCxxRevisionMacro(vtkXMLUnstructuredDataWriter, "$Revision: 1.24 $");
 
 //----------------------------------------------------------------------------
 vtkXMLUnstructuredDataWriter::vtkXMLUnstructuredDataWriter()
@@ -138,7 +138,7 @@ int vtkXMLUnstructuredDataWriter::ProcessRequest(vtkInformation* request,
       }
 
     int result = 1;
-    if (this->CurrentPiece == 0 && this->CurrentTimeIndex == 0 || this->WritePiece >= 0)
+    if ((this->CurrentPiece == 0 && this->CurrentTimeIndex == 0) || this->WritePiece >= 0)
       {
       // We are just starting to write.  Do not call
       // UpdateProgressDiscrete because we want a 0 progress callback the
@@ -652,7 +652,10 @@ vtkXMLUnstructuredDataWriter::WriteCellsAppendedData(vtkCellArray* cells,
                                                      int timestep,
                                                      OffsetsManagerGroup *cellsManager)
 {
-  this->ConvertCells(cells);
+  if (cells)
+    {
+    this->ConvertCells(cells);
+    }
   
   // Split progress by cell connectivity, offset, and type arrays.
   float progressRange[2] = {0,0};
