@@ -18,6 +18,8 @@
 #include "vtkXMLDataElement.h"
 #include "vtkXMLDataParser.h"
 
+#include <vtksys/ios/sstream>
+
 #if !defined(_WIN32) || defined(__CYGWIN__)
 # include <unistd.h> /* unlink */
 #else
@@ -29,7 +31,7 @@
 typedef vtkstd::vector<vtkXMLDataElement*> vtkXMLUtilitiesDataElementContainer;
 
 vtkStandardNewMacro(vtkXMLUtilities);
-vtkCxxRevisionMacro(vtkXMLUtilities, "$Revision: 1.6.6.1 $");
+vtkCxxRevisionMacro(vtkXMLUtilities, "$Revision: 1.10 $");
 
 #define  VTK_XML_UTILITIES_FACTORED_POOL_NAME "FactoredPool"
 #define  VTK_XML_UTILITIES_FACTORED_NAME      "Factored"
@@ -418,11 +420,11 @@ vtkXMLUtilities::ReadElementFromString(const char *str, int encoding)
     return 0;
     }
 
-  strstream strstr;
+  vtksys_ios::stringstream strstr;
   strstr << str;
   vtkXMLDataElement *res = 
     vtkXMLUtilities::ReadElementFromStream(strstr, encoding);
-  strstr.rdbuf()->freeze(0);
+
   return res;
 }
 
@@ -439,6 +441,44 @@ vtkXMLUtilities::ReadElementFromFile(const char *filename, int encoding)
   return vtkXMLUtilities::ReadElementFromStream(is, encoding);
 }
 
+//----------------------------------------------------------------------------
+void vtkXMLUtilities::ReadElementFromAttributeArray(
+        vtkXMLDataElement *element,
+        const char** atts,
+        int encoding)
+{
+  if(atts)
+    {
+    // If the target encoding is VTK_ENCODING_NONE or VTK_ENCODING_UNKNOWN, 
+    // then keep the internal/default encoding, otherwise encode each
+    // attribute using that new format
+
+    if (encoding != VTK_ENCODING_NONE && encoding != VTK_ENCODING_UNKNOWN)
+      {
+      element->SetAttributeEncoding(encoding);
+      }
+
+    // Process each attributes returned by Expat in UTF-8 encoding, and
+    // convert them to our encoding
+
+    for (int i = 0; atts[i] && atts[i + 1]; i += 2)
+      {
+      if (element->GetAttributeEncoding() == VTK_ENCODING_UTF_8)
+        {
+        element->SetAttribute(atts[i], atts[i + 1]);
+        }
+      else
+        {
+        vtksys_ios::ostringstream str;
+        vtkXMLUtilities::EncodeString(
+          atts[i+1], VTK_ENCODING_UTF_8, str, element->GetAttributeEncoding(), 0);
+        str << ends;
+        element->SetAttribute(atts[i], str.str().c_str());
+        }
+      }
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkXMLUtilitiesFindSimilarElementsInternal(
   vtkXMLDataElement *elem, 
@@ -584,13 +624,13 @@ int vtkXMLUtilities::FactorElementsInternal(vtkXMLDataElement *tree,
   char buffer[5];
   sprintf(buffer, "%02d_", pool->GetNumberOfNestedElements());
 
-  ostrstream id;
-  id << buffer << tree->GetName() << ends;
+  vtksys_ios::ostringstream id;
+  id << buffer << tree->GetName();
     
   vtkXMLDataElement *factored = vtkXMLDataElement::New();
   factored->SetName(VTK_XML_UTILITIES_FACTORED_NAME);
   factored->SetAttributeEncoding(pool->GetAttributeEncoding());
-  factored->SetAttribute("Id", id.str());
+  factored->SetAttribute("Id", id.str().c_str());
   pool->AddNestedElement(factored);
   factored->Delete();
 
@@ -605,16 +645,14 @@ int vtkXMLUtilities::FactorElementsInternal(vtkXMLDataElement *tree,
     similar_trees[i]->RemoveAllNestedElements();
     similar_trees[i]->SetCharacterData(NULL, 0);
     similar_trees[i]->SetName(VTK_XML_UTILITIES_FACTORED_REF_NAME);
-    similar_trees[i]->SetAttribute("Id", id.str());
+    similar_trees[i]->SetAttribute("Id", id.str().c_str());
     }
 
   tree->RemoveAllAttributes();
   tree->RemoveAllNestedElements();
   tree->SetCharacterData(NULL, 0);
   tree->SetName(VTK_XML_UTILITIES_FACTORED_REF_NAME);
-  tree->SetAttribute("Id", id.str());
-    
-  id.rdbuf()->freeze(0);
+  tree->SetAttribute("Id", id.str().c_str());
 
   delete [] similar_trees;
 
