@@ -14,6 +14,7 @@
 =========================================================================*/
 #include "vtkImageBlend.h"
 
+#include "vtkAlgorithmOutput.h"
 #include "vtkImageData.h"
 #include "vtkImageStencilData.h"
 #include "vtkInformation.h"
@@ -22,7 +23,7 @@
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkPointData.h"
 
-vtkCxxRevisionMacro(vtkImageBlend, "$Revision: 1.38.4.2 $");
+vtkCxxRevisionMacro(vtkImageBlend, "$Revision: 1.49 $");
 vtkStandardNewMacro(vtkImageBlend);
 
 //----------------------------------------------------------------------------
@@ -48,6 +49,30 @@ vtkImageBlend::~vtkImageBlend()
   this->OpacityArrayLength = 0;
 }
 
+//----------------------------------------------------------------------------
+void vtkImageBlend::ReplaceNthInputConnection(int idx,
+                                              vtkAlgorithmOutput *input)
+{
+  if (idx < 0 || idx >= this->GetNumberOfInputConnections(0))
+    {
+    vtkErrorMacro("Attempt to replace connection idx " << idx
+                  << " of input port " << 0 << ", which has only "
+                  << this->GetNumberOfInputConnections(0)
+                  << " connections.");
+    return;
+    }
+
+  if (!input || !input->GetProducer())
+    {
+    vtkErrorMacro("Attempt to replace connection index " << idx
+                  << " for input port " << 0 << " with " <<
+                  (!input ? "a null input." : "an input with no producer."));
+    return;
+    }
+
+  this->SetNthInputConnection(0, idx, input);
+}
+
 //----------------------------------------------------------------------------
 // The default vtkImageAlgorithm semantics are that SetInput() puts
 // each input on a different port, we want all the image inputs to
@@ -107,7 +132,7 @@ void vtkImageBlend::SetOpacity(int idx, double opacity)
 
   if (idx >= this->OpacityArrayLength)
     {
-    newLength = idx + 1; 
+    newLength = idx + 1;
     newArray = new double[newLength];
     for (i = 0; i < this->OpacityArrayLength; i++)
       {
@@ -131,7 +156,7 @@ void vtkImageBlend::SetOpacity(int idx, double opacity)
     this->Modified();
     }
 }
-    
+
 //----------------------------------------------------------------------------
 double vtkImageBlend::GetOpacity(int idx)
 {
@@ -140,7 +165,7 @@ double vtkImageBlend::GetOpacity(int idx)
     return 1.0;
     }
   return this->Opacity[idx];
-}    
+}
 
 //----------------------------------------------------------------------------
 int vtkImageBlend::RequestInformation (
@@ -199,14 +224,14 @@ int vtkImageBlend::RequestUpdateExtent(
 {
   // get the info objects
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
-  
+
   // default input extent will be that of output extent
   int inExt[6];
-  int *outExt = 
+  int *outExt =
     outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_EXTENT());
 
   int whichInput;
-  for (whichInput = 0; whichInput < this->GetNumberOfInputConnections(0); 
+  for (whichInput = 0; whichInput < this->GetNumberOfInputConnections(0);
        whichInput++)
     {
     int *inWextent;
@@ -219,6 +244,7 @@ int vtkImageBlend::RequestUpdateExtent(
   return 1;
 }
 
+//----------------------------------------------------------------------------
 int vtkImageBlend::RequestData(
   vtkInformation* request,
   vtkInformationVector** inputVector,
@@ -233,9 +259,9 @@ int vtkImageBlend::RequestData(
     vtkImageData *outData = static_cast<vtkImageData *>(
       info->Get(vtkDataObject::DATA_OBJECT()));
     info = inputVector[0]->GetInformationObject(0);
-    vtkImageData *inData = 
+    vtkImageData *inData =
       static_cast<vtkImageData*>(info->Get(vtkDataObject::DATA_OBJECT()));
-    
+
     outData->SetExtent(inData->GetExtent());
     outData->GetPointData()->PassData(inData->GetPointData());
     this->DataWasPassed = 1;
@@ -261,8 +287,8 @@ int vtkImageBlend::RequestData(
 template <class T>
 inline int vtkBlendGetNextExtent(vtkImageStencilData *stencil,
                                  int &r1, int &r2, int rmin, int rmax,
-                                 int yIdx, int zIdx, 
-                                 T *&outPtr, T *&inPtr, 
+                                 int yIdx, int zIdx,
+                                 T *&outPtr, T *&inPtr,
                                  int outScalars, int inScalars,
                                  int &iter)
 {
@@ -301,7 +327,7 @@ inline int vtkBlendGetNextExtent(vtkImageStencilData *stencil,
 //----------------------------------------------------------------------------
 // This templated function executes the filter for any type of data.
 template <class T>
-void vtkImageBlendExecute(vtkImageBlend *self, int extent[6], 
+void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
                           vtkImageData *inData, T *inPtr,
                           vtkImageData *outData, T *outPtr,
                           double opacity, int id)
@@ -340,11 +366,11 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
   inC = inData->GetNumberOfScalarComponents();
   outC = outData->GetNumberOfScalarComponents();
 
-  target = (unsigned long)((extent[3] - extent[2] + 1)*
-                           (extent[5] - extent[4] + 1)/50.0);
+  target = static_cast<unsigned long>((extent[3] - extent[2] + 1)*
+                                      (extent[5] - extent[4] + 1)/50.0);
   target++;
-  
-  // Get increments to march through data 
+
+  // Get increments to march through data
   inData->GetContinuousIncrements(extent, inIncX, inIncY, inIncZ);
   outData->GetContinuousIncrements(extent, outIncX, outIncY, outIncZ);
 
@@ -353,7 +379,7 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
     {
     for (idxY = extent[2]; !self->AbortExecute && idxY <= extent[3]; idxY++)
       {
-      if (!id) 
+      if (!id)
         {
         if (!(count%target))
           {
@@ -376,7 +402,7 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
             outPtr[0] = T(outPtr[0]*f + inPtr[0]*r);
             outPtr[1] = T(outPtr[1]*f + inPtr[1]*r);
             outPtr[2] = T(outPtr[2]*f + inPtr[2]*r);
-            outPtr += outC; 
+            outPtr += outC;
             inPtr += inC;
             }
           }
@@ -392,7 +418,7 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
             outPtr[0] = T(outPtr[0]*f + inPtr[0]*r);
             outPtr[1] = T(outPtr[1]*f + inPtr[1]*r);
             outPtr[2] = T(outPtr[2]*f + inPtr[2]*r);
-            outPtr += outC; 
+            outPtr += outC;
             inPtr += inC;
             }
           }
@@ -410,7 +436,7 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
             outPtr[0] = T(outPtr[0]*f + (*inPtr)*r);
             outPtr[1] = T(outPtr[1]*f + (*inPtr)*r);
             outPtr[2] = T(outPtr[2]*f + (*inPtr)*r);
-            outPtr += outC; 
+            outPtr += outC;
             inPtr += 2;
             }
           }
@@ -426,7 +452,7 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
             outPtr[0] = T(outPtr[0]*f + (*inPtr)*r);
             outPtr[1] = T(outPtr[1]*f + (*inPtr)*r);
             outPtr[2] = T(outPtr[2]*f + (*inPtr)*r);
-            outPtr += outC; 
+            outPtr += outC;
             inPtr++;
             }
           }
@@ -442,7 +468,7 @@ void vtkImageBlendExecute(vtkImageBlend *self, int extent[6],
             r = opacity*(inPtr[1]-minA);
             f = 1.0-r;
             *outPtr = T((*outPtr)*f + (*inPtr)*r);
-            outPtr += outC; 
+            outPtr += outC;
             inPtr += 2;
             }
           }
@@ -493,15 +519,15 @@ void vtkImageBlendExecuteChar(vtkImageBlend *self, int extent[6],
 
   // round opacity to a value in the range [0,256], because division
   // by 256 can be efficiently achieved by bit-shifting by 8 bits
-  o = (unsigned short)(256*opacity + 0.5);
+  o = static_cast<unsigned short>(256*opacity + 0.5);
   r = o;
   f = 256 - o;
 
   inC = inData->GetNumberOfScalarComponents();
   outC = outData->GetNumberOfScalarComponents();
 
-  target = (unsigned long)((extent[3] - extent[2] + 1)*
-                           (extent[5] - extent[4] + 1)/50.0);
+  target = static_cast<unsigned long>((extent[3] - extent[2] + 1)*
+                                      (extent[5] - extent[4] + 1)/50.0);
   target++;  
 
   // Get increments to march through data 
@@ -666,8 +692,9 @@ void vtkImageBlendCopyData(vtkImageData *inData, vtkImageData *outData,
   int rowLength;
   unsigned char *inPtr, *inPtr1, *outPtr;
  
-  inPtr = (unsigned char *) inData->GetScalarPointerForExtent(ext);
-  outPtr = (unsigned char *) outData->GetScalarPointerForExtent(ext);
+  inPtr = static_cast<unsigned char *>(inData->GetScalarPointerForExtent(ext));
+  outPtr =
+    static_cast<unsigned char *>(outData->GetScalarPointerForExtent(ext));
  
   // Get increments to march through inData
   inData->GetIncrements(inIncX, inIncY, inIncZ);
@@ -708,8 +735,8 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
   unsigned long count = 0;
   unsigned long target;
 
-  target = (unsigned long)((extent[3] - extent[2] + 1)*
-                           (extent[5] - extent[4] + 1)/50.0);
+  target = static_cast<unsigned long>((extent[3] - extent[2] + 1)*
+                                      (extent[5] - extent[4] + 1)/50.0);
   target++;
   
   // Get increments to march through data 
@@ -726,7 +753,7 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
   tmpData->GetContinuousIncrements(extent, tmpIncX, tmpIncY, tmpIncZ);
   tmpC = tmpData->GetNumberOfScalarComponents();
 
-  double* tmpPtr = (double *)tmpData->GetScalarPointerForExtent(extent);  
+  double* tmpPtr = static_cast<double *>(tmpData->GetScalarPointerForExtent(extent));
 
   // Opacity
   double minA, maxA;
@@ -740,8 +767,8 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
     }
   else
     {
-    minA = (double)inData->GetScalarTypeMin();
-    maxA = (double)inData->GetScalarTypeMax();
+    minA = static_cast<double>(inData->GetScalarTypeMin());
+    maxA = static_cast<double>(inData->GetScalarTypeMax());
     }
 
   r = opacity;
@@ -772,12 +799,12 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
           { 
           for (int idxX = extent[0]; idxX <= extent[1]; idxX++)
             {
-            r = opacity * ((double)inPtr[3] - minA);
+            r = opacity * (static_cast<double>(inPtr[3]) - minA);
             if (r > threshold) 
               {
-              tmpPtr[0] += (double)inPtr[0] * r;
-              tmpPtr[1] += (double)inPtr[1] * r;
-              tmpPtr[2] += (double)inPtr[2] * r;
+              tmpPtr[0] += static_cast<double>(inPtr[0]) * r;
+              tmpPtr[1] += static_cast<double>(inPtr[1]) * r;
+              tmpPtr[2] += static_cast<double>(inPtr[2]) * r;
               tmpPtr[3] += r;
               }
             tmpPtr += 4; 
@@ -790,9 +817,9 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
           { 
           for (int idxX = extent[0]; idxX <= extent[1]; idxX++)
             {
-            tmpPtr[0] += (double)inPtr[0] * r;
-            tmpPtr[1] += (double)inPtr[1] * r;
-            tmpPtr[2] += (double)inPtr[2] * r;
+            tmpPtr[0] += static_cast<double>(inPtr[0]) * r;
+            tmpPtr[1] += static_cast<double>(inPtr[1]) * r;
+            tmpPtr[2] += static_cast<double>(inPtr[2]) * r;
             tmpPtr[3] += r;
             tmpPtr += 4; 
             inPtr += inC;
@@ -804,12 +831,12 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
           { 
           for (int idxX = extent[0]; idxX <= extent[1]; idxX++)
             {
-            r = opacity * ((double)inPtr[1] - minA);
+            r = opacity * (static_cast<double>(inPtr[1]) - minA);
             if (r > threshold) 
               {
-              tmpPtr[0] += (double)(*inPtr) * r;
-              tmpPtr[1] += (double)(*inPtr) * r;
-              tmpPtr[2] += (double)(*inPtr) * r;
+              tmpPtr[0] += static_cast<double>(*inPtr) * r;
+              tmpPtr[1] += static_cast<double>(*inPtr) * r;
+              tmpPtr[2] += static_cast<double>(*inPtr) * r;
               tmpPtr[3] += r;
               }
             tmpPtr += 4; 
@@ -822,9 +849,9 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
           { 
           for (int idxX = extent[0]; idxX <= extent[1]; idxX++)
             {
-            tmpPtr[0] += (double)(*inPtr) * r;
-            tmpPtr[1] += (double)(*inPtr) * r;
-            tmpPtr[2] += (double)(*inPtr) * r;
+            tmpPtr[0] += static_cast<double>(*inPtr) * r;
+            tmpPtr[1] += static_cast<double>(*inPtr) * r;
+            tmpPtr[2] += static_cast<double>(*inPtr) * r;
             tmpPtr[3] += r;
             tmpPtr += 4; 
             inPtr++;
@@ -837,10 +864,10 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
         { 
         for (int idxX = extent[0]; idxX <= extent[1]; idxX++)
           {
-          r = opacity * ((double)inPtr[1] - minA);
+          r = opacity * (static_cast<double>(inPtr[1]) - minA);
           if (r > threshold) 
             {
-            tmpPtr[0] = (double)(*inPtr) * r;
+            tmpPtr[0] = static_cast<double>(*inPtr) * r;
             tmpPtr[1] += r;
             }
           tmpPtr += 2; 
@@ -853,7 +880,7 @@ void vtkImageBlendCompoundExecute(vtkImageBlend *self,
         { 
         for (int idxX = extent[0]; idxX <= extent[1]; idxX++)
           {
-          tmpPtr[0] = (double)(*inPtr) * r;
+          tmpPtr[0] = static_cast<double>(*inPtr) * r;
           tmpPtr[1] += r;
           tmpPtr += 2; 
           inPtr++;
@@ -892,7 +919,8 @@ void vtkImageBlendCompoundTransferExecute(vtkImageBlend *self,
   tmpData->GetContinuousIncrements(extent, tmpIncX, tmpIncY, tmpIncZ);
   tmpC = tmpData->GetNumberOfScalarComponents();
 
-  double* tmpPtr = (double *)tmpData->GetScalarPointerForExtent(extent);  
+  double* tmpPtr = static_cast<double *>(
+    tmpData->GetScalarPointerForExtent(extent));
 
   // Loop through output pixels
 
@@ -989,7 +1017,7 @@ void vtkImageBlend::ThreadedRequestData (
         (outData[0]->GetNumberOfScalarComponents() >= 3 ? 3 : 1) + 1);
       tmpData->SetScalarType(VTK_DOUBLE);
       tmpData->AllocateScalars();
-      memset((void *)tmpData->GetScalarPointer(), 0, 
+      memset(static_cast<void *>(tmpData->GetScalarPointer()), 0, 
              (outExt[1] - outExt[0] + 1) * 
              (outExt[3] - outExt[2] + 1) * 
              (outExt[5] - outExt[4] + 1) *
@@ -1065,8 +1093,10 @@ void vtkImageBlend::ThreadedRequestData (
           if (inData[0][idx1]->GetScalarType() == VTK_UNSIGNED_CHAR)
             {
             vtkImageBlendExecuteChar(this, extent,
-                                     inData[0][idx1], (unsigned char *)(inPtr),
-                                     outData[0], (unsigned char *)(outPtr),
+                                     inData[0][idx1],
+                                     static_cast<unsigned char *>(inPtr),
+                                     outData[0],
+                                     static_cast<unsigned char *>(outPtr),
                                      opacity, id);
             }
           else
@@ -1075,8 +1105,10 @@ void vtkImageBlend::ThreadedRequestData (
               {
               vtkTemplateMacro(
                 vtkImageBlendExecute(this, extent,
-                                     inData[0][idx1], (VTK_TT *)(inPtr),
-                                     outData[0], (VTK_TT *)(outPtr), 
+                                     inData[0][idx1],
+                                     static_cast<VTK_TT *>(inPtr),
+                                     outData[0],
+                                     static_cast<VTK_TT *>(outPtr), 
                                      opacity, id));
               default:
                 vtkErrorMacro(<< "Execute: Unknown ScalarType");
@@ -1092,7 +1124,7 @@ void vtkImageBlend::ThreadedRequestData (
               vtkImageBlendCompoundExecute(this, 
                                            extent,
                                            inData[0][idx1], 
-                                           (VTK_TT *)(inPtr), 
+                                           static_cast<VTK_TT *>(inPtr), 
                                            tmpData, 
                                            opacity,
                                            this->CompoundThreshold));
@@ -1122,7 +1154,7 @@ void vtkImageBlend::ThreadedRequestData (
           vtkImageBlendCompoundTransferExecute(this, 
                                                outExt,
                                                outData[0], 
-                                               (VTK_TT *)(outPtr), 
+                                               static_cast<VTK_TT *>(outPtr), 
                                                tmpData));
         default:
           vtkErrorMacro(<< "Execute: Unknown ScalarType");
@@ -1144,7 +1176,7 @@ void vtkImageBlend::PrintSelf(ostream& os, vtkIndent indent)
   int i;
   for (i = 0; i < this->OpacityArrayLength; i++)
     {
-    os << indent << "Opacity(" << i << "): " << this->GetOpacity(i) << endl; 
+    os << indent << "Opacity(" << i << "): " << this->GetOpacity(i) << endl;
     }
   os << indent << "Stencil: " << this->GetStencil() << endl;
   os << indent << "BlendMode: " << this->GetBlendModeAsString() << endl
