@@ -26,9 +26,6 @@
 // rendering correctly.  It can also attach itself to render windows and
 // propagate rendering events and camera views.
 //
-// This class is based on the vtkCompositeManager class, except that it can
-// handle any type of parallel rendering.
-//
 // .SECTION Note:
 // Many parallel rendering schemes do not correctly handle transparency.
 // Unless otherwise documented, assume a sub class does not.
@@ -45,12 +42,14 @@
 
 #include "vtkObject.h"
 
-class vtkRenderWindow;
-class vtkRenderer;
-class vtkUnsignedCharArray;
 class vtkDoubleArray;
-class vtkTimerLog;
 class vtkMultiProcessController;
+class vtkMultiProcessStream;
+class vtkRenderer;
+class vtkRendererCollection;
+class vtkRenderWindow;
+class vtkTimerLog;
+class vtkUnsignedCharArray;
 
 class VTK_PARALLEL_EXPORT vtkParallelRenderManager : public vtkObject
 {
@@ -123,7 +122,7 @@ public:
   virtual void RenderRMI();
   virtual void ResetCamera(vtkRenderer *ren);
   virtual void ResetCameraClippingRange(vtkRenderer *ren);
-  virtual void ComputeVisiblePropBoundsRMI();
+  virtual void ComputeVisiblePropBoundsRMI(int renderId);
 
   virtual void InitializeRMIs();
 
@@ -155,6 +154,15 @@ public:
   vtkGetMacro(RenderEventPropagation, int);
   vtkBooleanMacro(RenderEventPropagation, int);
 
+  // Description:
+  // Get/Set the default value used for RenderEventPropagation when a new
+  // instance of vtkParallelRenderManager is created.
+  // Set to true by default.
+  static void SetDefaultRenderEventPropagation(bool val)
+    { vtkParallelRenderManager::DefaultRenderEventPropagation = val; }
+  static bool GetDefaultRenderEventPropagation()
+    { return vtkParallelRenderManager::DefaultRenderEventPropagation; }
+
   // Description:
   // This is used for tiled display rendering.  When data has been
   // duplicated on all processes, then we do not need to compositing.
@@ -201,6 +209,21 @@ public:
   vtkGetMacro(RenderTime, double);
   vtkGetMacro(ImageProcessingTime, double);
 
+  // Description:
+  // By default, the state of all renderers in the root's render window is
+  // propagated to the rest of the processes.  In order for this to work, all
+  // render windows must have the same renderers in the same order.  If this is
+  // not the case, you can turn off the SyncRenderWindowRenderers.  When this
+  // flag is off, the list of renderers held by this parallel render manager
+  // (initially empty) is synced.  You can modify the list of renderers with the
+  // AddRenderer, RemoveRenderer, and RemoveAllRenderers methods.
+  vtkGetMacro(SyncRenderWindowRenderers, int);
+  vtkSetMacro(SyncRenderWindowRenderers, int);
+  vtkBooleanMacro(SyncRenderWindowRenderers, int);
+  virtual void AddRenderer(vtkRenderer *);
+  virtual void RemoveRenderer(vtkRenderer *);
+  virtual void RemoveAllRenderers();
+
   // Description:
   // If on (the default), the result of any image space manipulations are
   // written back to the render window frame buffer.  If off, the image
@@ -238,6 +261,27 @@ public:
     this->SetMagnifyImageMethod(LINEAR);
   }
 
+  // Description:
+  // Convenience functions for magnifying images.
+  virtual void MagnifyImage(vtkUnsignedCharArray *fullImage,
+                            const int fullImageSize[2],
+                            vtkUnsignedCharArray *reducedImage,
+                            const int reducedImageSize[2],
+                            const int fullImageViewport[4] = NULL,
+                            const int reducedImageViewport[4] = NULL);
+  static void MagnifyImageNearest(vtkUnsignedCharArray *fullImage,
+                                  const int fullImageSize[2],
+                                  vtkUnsignedCharArray *reducedImage,
+                                  const int reducedImageSize[2],
+                                  const int fullImageViewport[4] = NULL,
+                                  const int reducedImageViewport[4] = NULL);
+  static void MagnifyImageLinear(vtkUnsignedCharArray *fullImage,
+                                 const int fullImageSize[2],
+                                 vtkUnsignedCharArray *reducedImage,
+                                 const int reducedImageSize[2],
+                                 const int fullImageViewport[4] = NULL,
+                                 const int reducedImageViewport[4] = NULL);
+
   // Description:
   // The most appropriate way to retrieve full size image data after a
   // render.  Will work regardless of whether WriteBackImages or
@@ -276,15 +320,25 @@ public:
   vtkSetMacro(UseRGBA, int);
   vtkGetMacro(UseRGBA, int);
 
+  // Description:
+  // If ForceRenderWindowSize is set to true, the render manager will use
+  // the RenderWindowSize ivar instead of getting the size from the render window.
+  vtkSetMacro(ForceRenderWindowSize, int);
+  vtkGetMacro(ForceRenderWindowSize, int);
+
+  // Description:
+  // If ForceRenderWindowSize is set to true, the render manager will use
+  // the Size ivar instead of getting the size from the render window.
+  vtkSetVector2Macro(ForcedRenderWindowSize, int);
+  vtkGetVector2Macro(ForcedRenderWindowSize, int);
+
 //BTX
   enum Tags {
     RENDER_RMI_TAG=34532,
     COMPUTE_VISIBLE_PROP_BOUNDS_RMI_TAG=54636,
-    WIN_INFO_INT_TAG=87834,
-    WIN_INFO_DOUBLE_TAG=87835,
-    REN_INFO_INT_TAG=87836,
-    REN_INFO_DOUBLE_TAG=87837,
-    LIGHT_INFO_DOUBLE_TAG=87838,
+    WIN_INFO_TAG=87834,
+    REN_INFO_TAG=87836,
+    LIGHT_INFO_TAG=87838,
     REN_ID_TAG=58794,
     BOUNDS_TAG=23543
   };
@@ -310,14 +364,14 @@ public:
   // Description:
   // @deprecated Replaced by vtkParallelRenderManager::StartServices()
   // as of VTK 5.0.
-  VTK_LEGACY(virtual void const StartService());
+  VTK_LEGACY(virtual void StartService());
 
 #ifdef VTK_WORKAROUND_WINDOWS_MANGLE
 # undef StartServiceW
 # undef StartServiceA
   //BTX
-  VTK_LEGACY(virtual void const StartServiceA());
-  VTK_LEGACY(virtual void const StartServiceW());
+  VTK_LEGACY(virtual void StartServiceA());
+  VTK_LEGACY(virtual void StartServiceW());
   //ETX
 #endif
 
@@ -329,12 +383,49 @@ public:
 # pragma reset woff 3303
 #endif
 
+  //Description:
+  //The default is to allow the use of the back buffer for compositing.
+  //If set to false, this will prevent to manager from swapping buffers.
+  //This allows something else (for instance VisibleCellSelection) to
+  //control front/back buffer swapping.
+  vtkSetMacro(UseBackBuffer, int);
+  vtkGetMacro(UseBackBuffer, int);
+  vtkBooleanMacro(UseBackBuffer, int);
+
+  // Description:
+  // When set the render manager will synchronize the TileViewport and TileScale
+  // properties. This may not be desirable in cases where there's some other
+  // mechanism to set the tile dimensions eg. Tile displays.
+  vtkSetMacro(SynchronizeTileProperties, int);
+  vtkGetMacro(SynchronizeTileProperties, int);
+  vtkBooleanMacro(SynchronizeTileProperties, int);
+
+  // Description:
+  // INTERNAL METHODS (DON NOT USE).
+  // There are internal methods made public so that they can be called from
+  // callback functions.
+  virtual void GenericStartRenderCallback();
+  virtual void GenericEndRenderCallback();
+
+//BTX
 protected:
   vtkParallelRenderManager();
   ~vtkParallelRenderManager();
 
+
+  // Description:
+  // Add/Remove event handlers for the render window.
+  void AddRenderWindowEventHandlers();
+  void RemoveRenderWindowEventHandlers();
+
   vtkRenderWindow *RenderWindow;
   vtkMultiProcessController *Controller;
+  vtkRendererCollection *Renderers;
+
+  virtual vtkRendererCollection *GetRenderers();
+
+  int ForceRenderWindowSize;
+  int ForcedRenderWindowSize[2];
 
   // Description:
   // The "root" node's process id.  This is the node which is listening for
@@ -346,7 +437,6 @@ protected:
   int RootProcessId;
 
   int ObservingRenderWindow;
-  int ObservingRenderer;
   int ObservingAbort;
 
   unsigned long StartRenderTag;
@@ -364,7 +454,7 @@ protected:
   int MagnifyImageMethod;
 
   int UseRGBA;
-
+  int SynchronizeTileProperties;
   int FullImageSize[2];
   int ReducedImageSize[2];
 
@@ -381,6 +471,7 @@ protected:
   int ParallelRendering;
   int RenderEventPropagation;
   int UseCompositing;
+  int SyncRenderWindowRenderers;
 
   vtkTimerLog *Timer;
 
@@ -394,11 +485,29 @@ protected:
 
   // Description:
   // Used to synchronize rendering information per frame.
+  // These are old methods provided for backwords compatibility. One should look
+  // at using CollectWindowInformation(), ProcessWindowInformation() etc. for
+  // bufferred sending of information over.
   virtual void SendWindowInformation() {}
   virtual void ReceiveWindowInformation() {}
   virtual void SendRendererInformation(vtkRenderer *) {};
   virtual void ReceiveRendererInformation(vtkRenderer *) {};
 
+  // Description:
+  // Subclass should override these methods (instead of
+  // SendWindowInformation/ReceiveWindowInformation or
+  // SendRendererInformation/ReceiveRendererInformation) to collect or process
+  // meta-data to synchronize rendering information per frame.
+  // Subclass should not use the Controller directly to send receive messages
+  // in any of these methods otherwise deadlocks may ensue.
+  virtual void CollectWindowInformation(vtkMultiProcessStream&) {}
+  virtual bool ProcessWindowInformation(vtkMultiProcessStream&) 
+    { return true; }
+  virtual void CollectRendererInformation(vtkRenderer*,
+    vtkMultiProcessStream&) {}
+  virtual bool ProcessRendererInformation(vtkRenderer*,
+    vtkMultiProcessStream&) { return true; }
+
   // Description:
   // Here is a good place to handle processing of data before and after
   // render.
@@ -407,7 +516,7 @@ protected:
 
   // Description:
   // Called in satellites to set the render window size to the current
-  // FullImageSize and ReducedImageSize (or vice versa)
+  // FullImageSize and ReducedImageSize (or vice versa).
   virtual void SetRenderWindowSize();
 
   // Description:
@@ -442,28 +551,33 @@ protected:
   virtual void SetRenderWindowPixelData(vtkUnsignedCharArray *pixels,
           const int pixelDimensions[2]);
 
-//BTX
-  struct RenderWindowInfoInt
+  // Description:
+  // Returns true if the image for the given renderer should be rendered at a
+  // reduced size to be magnified later.  This method always returns true, but
+  // subclasses may render some renderers at a reduced size, magnify them, and
+  // then render the other renderers at full resolution.
+  virtual int ImageReduceRenderer(vtkRenderer *) { return 1; }
+
+  struct RenderWindowInfo
   {
     int FullSize[2];
     int ReducedSize[2];
     int NumberOfRenderers;
     int UseCompositing;
-  };
-
-  struct RenderWindowInfoDouble
-  {
+    int TileScale[2];
     double ImageReductionFactor;
     double DesiredUpdateRate;
-  };
-  
-  struct RendererInfoInt
-  {
-    int NumberOfLights;
+    double TileViewport[4];
+
+    // Save/restore the struct to/from a stream.
+    void Save(vtkMultiProcessStream& stream);
+    bool Restore(vtkMultiProcessStream& stream);
   };
 
-  struct RendererInfoDouble
+  struct RendererInfo
   {
+    int Draw;
+    int NumberOfLights;
     double Viewport[4];
     double CameraPosition[3];
     double CameraFocalPoint[3];
@@ -473,26 +587,33 @@ protected:
     double CameraViewAngle;
     double Background[3];
     double ParallelScale;
+
+    // Save/restore the struct to/from a stream.
+    void Save(vtkMultiProcessStream& stream);
+    bool Restore(vtkMultiProcessStream& stream);
   };
   
-  struct LightInfoDouble
+  struct LightInfo
   {
     double Position[3];
     double FocalPoint[3];
     double Type;
+    // Save/restore the struct to/from a stream.
+    void Save(vtkMultiProcessStream& stream);
+    bool Restore(vtkMultiProcessStream& stream);
   };
 
-  static const int WIN_INFO_INT_SIZE;
-  static const int WIN_INFO_DOUBLE_SIZE;
-  static const int REN_INFO_INT_SIZE;
-  static const int REN_INFO_DOUBLE_SIZE;
-  static const int LIGHT_INFO_DOUBLE_SIZE;
-//ETX
-
   int AddedRMIs;
+  unsigned long RenderRMIId;
+  unsigned long BoundsRMIId;
+  int UseBackBuffer;
+
+  static bool DefaultRenderEventPropagation;
+
 private:
   vtkParallelRenderManager(const vtkParallelRenderManager &); //Not implemented
   void operator=(const vtkParallelRenderManager &);  //Not implemented
+//ETX
 };
 
 #endif //__vtkParalleRenderManager_h
