@@ -417,7 +417,7 @@ const char *MinMagModToString(int minMagMode)
       break;  
     default:
       result=0;
-//      assert("check: impossible case.");
+//      assert("check: impossible case." && 0);
       break;
     }
   return result;
@@ -728,7 +728,7 @@ const char *InternalTextureFormatToString(int internalFormat)
       break;
     default:
       result=0;
-//      assert("check: impossible case.");
+//      assert("check: impossible case." && 0);
 //      cout<<"unknown"<<"(0x"<< hex << ivalue[0] << dec << ")";
       break;
     }
@@ -757,7 +757,7 @@ const char *WrapModeToString(GLenum wrapMode)
       break;
     default:
       result="";
-      assert("check: impossible case.");
+      assert("check: impossible case." && 0);
       break;
     }
   return result;
@@ -1485,7 +1485,7 @@ int FromTextureSizedInternalFormatsToBaseInternalFormat(int f)
       break;
     default:
       result=0;
-      assert("check: impossible case."); 
+      assert("check: impossible case." && 0); 
       break;
     }
   return result;
@@ -1573,7 +1573,7 @@ int FromBaseInternalFormatToFormat(int f,
         result=vtkgl::LUMINANCE_ALPHA_INTEGER_EXT;
         break;
       default:
-        assert("check: impossible case.");
+        assert("check: impossible case." && 0);
         break;
       }
     }
@@ -1596,7 +1596,7 @@ const char *TargetToString(int target)
       break;
     default:
       result="";
-      assert("check: impossible case.");
+      assert("check: impossible case." && 0);
       break;
     }
   return result;
@@ -1624,7 +1624,7 @@ const char *WrapModeToString(int wrapMode)
       break;
     default:
       result="";
-      assert("check: impossible case.");
+      assert("check: impossible case." && 0);
       break;
     }
   return result;
