@@ -31,7 +31,7 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkActor, "$Revision: 1.138 $");
+vtkCxxRevisionMacro(vtkActor, "$Revision: 1.140 $");
 
 vtkCxxSetObjectMacro(vtkActor,Texture,vtkTexture);
 vtkCxxSetObjectMacro(vtkActor,Mapper,vtkMapper);
@@ -117,6 +117,13 @@ void vtkActor::GetActors(vtkPropCollection *ac)
 // should be called from the render methods only
 int vtkActor::GetIsOpaque()
 {
+  // make sure we have a property
+  if(!this->Property)
+    {
+    // force creation of a property
+    this->GetProperty();
+    }
+  
   int result=this->Property->GetOpacity() >= 1.0;
   
   if(result)
@@ -528,3 +535,13 @@ int vtkActor::GetNumberOfParts()
 #endif
 
 
+//----------------------------------------------------------------------------
+bool vtkActor::GetSupportsSelection()
+{
+  if (this->Mapper)
+    {
+    return this->Mapper->GetSupportsSelection();
+    }
+
+  return false;
+}
