@@ -22,7 +22,7 @@
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkCamera, "$Revision: 1.109 $");
+vtkCxxRevisionMacro(vtkCamera, "$Revision: 1.118 $");
 
 //----------------------------------------------------------------------------
 // Needed when we don't use the vtkStandardNewMacro.
@@ -59,7 +59,7 @@ vtkCamera::vtkCamera()
 
   this->ParallelProjection = 0;
   this->ParallelScale = 1.0;
- 
+
   this->EyeAngle = 2.0;
   this->Stereo = 0;
   this->LeftEye = 1;
@@ -75,7 +75,7 @@ vtkCamera::vtkCamera()
 
   this->Transform = vtkPerspectiveTransform::New();
   this->ViewTransform = vtkTransform::New();
-  this->PerspectiveTransform = vtkPerspectiveTransform::New();
+  this->ProjectionTransform = vtkPerspectiveTransform::New();
   this->CameraLightTransform = vtkTransform::New();
   this->UserTransform = NULL;
 
@@ -90,7 +90,7 @@ vtkCamera::~vtkCamera()
 {
   this->Transform->Delete();
   this->ViewTransform->Delete();
-  this->PerspectiveTransform->Delete();
+  this->ProjectionTransform->Delete();
   this->CameraLightTransform->Delete();
   if (this->UserTransform)
     {
@@ -105,7 +105,7 @@ vtkCamera *vtkCamera::New()
 {
   // First try to create the object from the vtkObjectFactory
   vtkObject* ret = vtkGraphicsFactory::CreateInstance("vtkCamera");
-  return (vtkCamera*)ret;
+  return static_cast<vtkCamera *>(ret);
 }
 
 //----------------------------------------------------------------------------
@@ -215,8 +215,9 @@ void vtkCamera::SetViewUp(double x, double y, double z)
   this->ViewUp[1] = y;
   this->ViewUp[2] = z;
 
-  vtkDebugMacro(<< " ViewUp set to ( " <<  this->ViewUp[0] << ", " << this->ViewUp[1] << ", " << this->ViewUp[2] << ")");
-  
+  vtkDebugMacro(<< " ViewUp set to ( " <<  this->ViewUp[0] << ", "
+    << this->ViewUp[1] << ", " << this->ViewUp[2] << ")");
+
   this->ComputeViewTransform();
   this->ComputeCameraLightTransform();
   this->Modified();
@@ -265,7 +266,7 @@ void vtkCamera::OrthogonalizeViewUp()
 }
 
 //----------------------------------------------------------------------------
-// Set the distance of the focal point from the camera. The focal point is 
+// Set the distance of the focal point from the camera. The focal point is
 // modified accordingly. This should be positive.
 void vtkCamera::SetDistance(double d)
 {
@@ -274,15 +275,15 @@ void vtkCamera::SetDistance(double d)
     return;
     }
 
-  this->Distance = d; 
+  this->Distance = d;
 
   // Distance should be greater than .0002
-  if (this->Distance < 0.0002) 
+  if (this->Distance < 0.0002)
     {
     this->Distance = 0.0002;
     vtkDebugMacro(<< " Distance is set to minimum.");
     }
-  
+
   // we want to keep the camera pointing in the same direction
   double *vec = this->DirectionOfProjection;
 
@@ -296,7 +297,7 @@ void vtkCamera::SetDistance(double d)
   this->ComputeViewTransform();
   this->ComputeCameraLightTransform();
   this->Modified();
-}  
+}
 
 //----------------------------------------------------------------------------
 // This method must be called when the focal point or camera position changes
@@ -308,9 +309,9 @@ void vtkCamera::ComputeDistance()
 
   this->Distance = sqrt(dx*dx + dy*dy + dz*dz);
 
-  if (this->Distance < 0.0002) 
+  if (this->Distance < 1e-20) 
     {
-    this->Distance = 0.0002;
+    this->Distance = 1e-20;
     vtkDebugMacro(<< " Distance is set to minimum.");
 
     double *vec = this->DirectionOfProjection;
@@ -326,11 +327,11 @@ void vtkCamera::ComputeDistance()
   this->DirectionOfProjection[2] = dz/this->Distance;
 
   this->ComputeViewPlaneNormal();
-} 
+}
 
 //----------------------------------------------------------------------------
 // Move the position of the camera along the view plane normal. Moving
-// towards the focal point (e.g., > 1) is a dolly-in, moving away 
+// towards the focal point (e.g., > 1) is a dolly-in, moving away
 // from the focal point (e.g., < 1) is a dolly-out.
 void vtkCamera::Dolly(double amount)
 {
@@ -338,10 +339,10 @@ void vtkCamera::Dolly(double amount)
     {
     return;
     }
-  
+
   // dolly moves the camera towards the focus
   double d = this->Distance/amount;
-  
+
   this->SetPosition(this->FocalPoint[0] - d*this->DirectionOfProjection[0],
                     this->FocalPoint[1] - d*this->DirectionOfProjection[1],
                     this->FocalPoint[2] - d*this->DirectionOfProjection[2]);
@@ -391,8 +392,8 @@ void vtkCamera::Roll(double angle)
 }
 
 //----------------------------------------------------------------------------
-// Rotate the focal point about the view up vector centered at the camera's 
-// position. 
+// Rotate the focal point about the view up vector centered at the camera's
+// position.
 void vtkCamera::Yaw(double angle)
 {
   double newFocalPoint[3];
@@ -402,17 +403,17 @@ void vtkCamera::Yaw(double angle)
   // translate the camera to the origin,
   // rotate about axis,
   // translate back again
-  this->Transform->Translate(+pos[0],+pos[1],+pos[2]);   
+  this->Transform->Translate(+pos[0],+pos[1],+pos[2]);
   this->Transform->RotateWXYZ(angle,this->ViewUp);
   this->Transform->Translate(-pos[0],-pos[1],-pos[2]);
-  
+
   // now transform focal point
   this->Transform->TransformPoint(this->FocalPoint,newFocalPoint);
   this->SetFocalPoint(newFocalPoint);
 }
 
 //----------------------------------------------------------------------------
-// Rotate the focal point about the cross product of the view up vector 
+// Rotate the focal point about the cross product of the view up vector
 // and the negative of the , centered at the camera's position.
 void vtkCamera::Pitch(double angle)
 {
@@ -424,14 +425,14 @@ void vtkCamera::Pitch(double angle)
   axis[0] = this->ViewTransform->GetMatrix()->GetElement(0,0);
   axis[1] = this->ViewTransform->GetMatrix()->GetElement(0,1);
   axis[2] = this->ViewTransform->GetMatrix()->GetElement(0,2);
-  
+
   // translate the camera to the origin,
   // rotate about axis,
   // translate back again
-  this->Transform->Translate(+pos[0],+pos[1],+pos[2]);   
+  this->Transform->Translate(+pos[0],+pos[1],+pos[2]);
   this->Transform->RotateWXYZ(angle,axis);
   this->Transform->Translate(-pos[0],-pos[1],-pos[2]);
-  
+
   // now transform focal point
   this->Transform->TransformPoint(this->FocalPoint,newFocalPoint);
   this->SetFocalPoint(newFocalPoint);
@@ -447,11 +448,11 @@ void vtkCamera::Azimuth(double angle)
 
   // translate the focal point to the origin,
   // rotate about view up,
-  // translate back again  
-  this->Transform->Translate(+fp[0],+fp[1],+fp[2]);   
+  // translate back again
+  this->Transform->Translate(+fp[0],+fp[1],+fp[2]);
   this->Transform->RotateWXYZ(angle,this->ViewUp);
   this->Transform->Translate(-fp[0],-fp[1],-fp[2]);
-  
+
   // apply the transform to the position
   this->Transform->TransformPoint(this->Position,newPosition);
   this->SetPosition(newPosition);
@@ -470,14 +471,14 @@ void vtkCamera::Elevation(double angle)
   axis[0] = -this->ViewTransform->GetMatrix()->GetElement(0,0);
   axis[1] = -this->ViewTransform->GetMatrix()->GetElement(0,1);
   axis[2] = -this->ViewTransform->GetMatrix()->GetElement(0,2);
-  
+
   // translate the focal point to the origin,
   // rotate about axis,
   // translate back again
-  this->Transform->Translate(+fp[0],+fp[1],+fp[2]);   
+  this->Transform->Translate(+fp[0],+fp[1],+fp[2]);
   this->Transform->RotateWXYZ(angle,axis);
   this->Transform->Translate(-fp[0],-fp[1],-fp[2]);
-  
+
   // now transform position
   this->Transform->TransformPoint(this->Position,newPosition);
   this->SetPosition(newPosition);
@@ -485,8 +486,8 @@ void vtkCamera::Elevation(double angle)
 
 //----------------------------------------------------------------------------
 // Apply Transform to camera
-void vtkCamera::ApplyTransform(vtkTransform *t) {
-  
+void vtkCamera::ApplyTransform(vtkTransform *t)
+{
   double posOld[4], posNew[4], fpOld[4], fpNew[4], vuOld[4], vuNew[4];
 
   this->GetPosition(posOld);
@@ -512,8 +513,7 @@ void vtkCamera::ApplyTransform(vtkTransform *t) {
   this->SetPosition(posNew);
   this->SetFocalPoint(fpNew);
   this->SetViewUp(vuNew);
-  
-} 
+}
 
 //----------------------------------------------------------------------------
 //----------------------------------------------------------------------------
@@ -526,18 +526,18 @@ void vtkCamera::ApplyTransform(vtkTransform *t) {
 //----------------------------------------------------------------------------
 void vtkCamera::SetParallelProjection(int flag)
 {
-  if ( this->ParallelProjection != flag ) 
+  if ( this->ParallelProjection != flag )
     {
     this->ParallelProjection = flag;
     this->Modified();
-    this->ViewingRaysModified(); 
-    } 
+    this->ViewingRaysModified();
+    }
 }
 
 //----------------------------------------------------------------------------
 void vtkCamera::SetViewAngle(double angle)
 {
-  double min =   0.00000001;
+  double min = 0.00000001;
   double max = 179.0;
 
   if ( this->ViewAngle != angle )
@@ -573,7 +573,7 @@ void vtkCamera::SetParallelScale(double scale)
 
 //----------------------------------------------------------------------------
 // Change the ViewAngle (for perspective) or the ParallelScale (for parallel)
-// so that more or less of a scene occupies the viewport.  A value > 1 is a 
+// so that more or less of a scene occupies the viewport.  A value > 1 is a
 // zoom-in. A value < 1 is a zoom-out.
 void vtkCamera::Zoom(double amount)
 {
@@ -581,7 +581,7 @@ void vtkCamera::Zoom(double amount)
     {
     return;
     }
-  
+
   if (this->ParallelProjection)
     {
     this->SetParallelScale(this->ParallelScale/amount);
@@ -596,7 +596,7 @@ void vtkCamera::Zoom(double amount)
 void vtkCamera::SetClippingRange(double nearz, double farz)
 {
   double thickness;
-  
+
   // check the order
   if ( nearz > farz )
     {
@@ -605,47 +605,48 @@ void vtkCamera::SetClippingRange(double nearz, double farz)
     nearz = farz;
     farz = temp;
     }
-  
-  // front should be greater than 0.0001
-  if (nearz < 0.0001) 
+
+  // front should be greater than 1e-20
+  if (nearz < 1e-20)
     {
-    farz += 0.0001 - nearz;
-    nearz = 0.0001;
+    farz += 1e-20 - nearz;
+    nearz = 1e-20;
     vtkDebugMacro(<< " Front clipping range is set to minimum.");
     }
-  
+
   thickness = farz - nearz;
-  
-  // thickness should be greater than 0.0001
-  if (thickness < 0.0001) 
+
+  // thickness should be greater than 1e-20
+  if (thickness < 1e-20)
     {
-    thickness = 0.0001;
+    thickness = 1e-20;
     vtkDebugMacro(<< " ClippingRange thickness is set to minimum.");
-    
+
     // set back plane
     farz = nearz + thickness;
     }
-  
-  if (nearz == this->ClippingRange[0] && 
-      farz == this->ClippingRange[1] && 
+
+  if (nearz == this->ClippingRange[0] &&
+      farz == this->ClippingRange[1] &&
       this->Thickness == thickness)
     {
     return;
     }
 
-  this->ClippingRange[0] = nearz; 
-  this->ClippingRange[1] = farz; 
+  this->ClippingRange[0] = nearz;
+  this->ClippingRange[1] = farz;
   this->Thickness = thickness;
-  
-  vtkDebugMacro(<< " ClippingRange set to ( " <<  this->ClippingRange[0] << ", "  << this->ClippingRange[1] << ")");
+
+  vtkDebugMacro(<< " ClippingRange set to ( " <<  this->ClippingRange[0]
+    << ", "  << this->ClippingRange[1] << ")");
 
   this->Modified();
-}  
+}
 
 //----------------------------------------------------------------------------
-// Set the distance between clipping planes. 
+// Set the distance between clipping planes.
 // This method adjusts the back clipping plane to the specified thickness
-// behind the front clipping plane 
+// behind the front clipping plane
 void vtkCamera::SetThickness(double s)
 {
   if (this->Thickness == s)
@@ -653,22 +654,23 @@ void vtkCamera::SetThickness(double s)
     return;
     }
 
-  this->Thickness = s; 
+  this->Thickness = s;
 
-  // thickness should be greater than 0.0001
-  if (this->Thickness < 0.0001) 
+  // thickness should be greater than 1e-20
+  if (this->Thickness < 1e-20)
     {
-    this->Thickness = 0.0001;
+    this->Thickness = 1e-20;
     vtkDebugMacro(<< " ClippingRange thickness is set to minimum.");
     }
-  
+
   // set back plane
   this->ClippingRange[1] = this->ClippingRange[0] + this->Thickness;
 
-  vtkDebugMacro(<< " ClippingRange set to ( " <<  this->ClippingRange[0] << ", " << this->ClippingRange[1] << ")");
+  vtkDebugMacro(<< " ClippingRange set to ( " <<  this->ClippingRange[0]
+    << ", " << this->ClippingRange[1] << ")");
 
   this->Modified();
-}  
+}
 
 //----------------------------------------------------------------------------
 void vtkCamera::SetWindowCenter(double x, double y)
@@ -685,14 +687,14 @@ void vtkCamera::SetWindowCenter(double x, double y)
 //----------------------------------------------------------------------------
 void vtkCamera::SetObliqueAngles(double alpha, double beta)
 {
-  alpha *= vtkMath::DoubleDegreesToRadians();
-  beta *= vtkMath::DoubleDegreesToRadians();
+  alpha = vtkMath::RadiansFromDegrees( alpha );
+  beta = vtkMath::RadiansFromDegrees( beta );
 
-  double cotbeta = cos(beta) / sin(beta);
-  double dxdz = cos(alpha) * cotbeta;
-  double dydz = sin(alpha) * cotbeta;
+  double cotbeta = cos( beta ) / sin( beta );
+  double dxdz = cos( alpha ) * cotbeta;
+  double dydz = sin( alpha ) * cotbeta;
 
-  this->SetViewShear(dxdz, dydz, 1.0);
+  this->SetViewShear( dxdz, dydz, 1.0 );
 }
 
 //----------------------------------------------------------------------------
@@ -703,144 +705,195 @@ void vtkCamera::SetObliqueAngles(double alpha, double beta)
 //
 void vtkCamera::SetViewShear(double dxdz, double dydz, double center)
 {
-  if(dxdz != this->ViewShear[0] || 
-     dydz != this->ViewShear[1] ||
+  if(dxdz   != this->ViewShear[0] ||
+     dydz   != this->ViewShear[1] ||
      center != this->ViewShear[2])
     {
-      this->Modified();
-      this->ViewingRaysModified();
+    this->Modified();
+    this->ViewingRaysModified();
 
-      this->ViewShear[0] = dxdz;
-      this->ViewShear[1] = dydz;
-      this->ViewShear[2] = center;
-      
-      this->ComputeViewPlaneNormal();
+    this->ViewShear[0] = dxdz;
+    this->ViewShear[1] = dydz;
+    this->ViewShear[2] = center;
+
+    this->ComputeViewPlaneNormal();
     }
 }
 //----------------------------------------------------------------------------
 
-void vtkCamera::SetViewShear(double d[3]) 
+void vtkCamera::SetViewShear(double d[3])
 {
   this->SetViewShear(d[0], d[1], d[2]);
 }
 
+#ifndef VTK_LEGACY_REMOVE
 //----------------------------------------------------------------------------
-// Compute the perspective transform matrix. This is used in converting 
+// Compute the projection transform matrix. This is used in converting
 // between view and world coordinates.
-void vtkCamera::ComputePerspectiveTransform(double aspect, 
+void vtkCamera::ComputePerspectiveTransform(double aspect,
                                             double nearz, double farz)
 {
-  this->PerspectiveTransform->Identity();
+  VTK_LEGACY_REPLACED_BODY(vtkCamera::ComputePerspectiveTransform,"VTK 5.4",vtkCamera::ComputeProjectionTransform);
+  this->ComputeProjectionTransform(aspect,nearz,farz);
+}
+#endif
 
-  // apply user defined transform last if there is one 
-  if (this->UserTransform)
+//----------------------------------------------------------------------------
+// Compute the projection transform matrix. This is used in converting
+// between view and world coordinates.
+void vtkCamera::ComputeProjectionTransform(double aspect,
+                                           double nearz, double farz)
+{
+  this->ProjectionTransform->Identity();
+
+  // apply user defined transform last if there is one
+  if ( this->UserTransform )
     {
-    this->PerspectiveTransform->Concatenate(this->UserTransform->GetMatrix());
+    this->ProjectionTransform->Concatenate( this->UserTransform->GetMatrix() );
     }
 
   // adjust Z-buffer range
-  this->PerspectiveTransform->AdjustZBuffer(-1, +1, nearz, farz);
+  this->ProjectionTransform->AdjustZBuffer( -1, +1, nearz, farz );
 
-  if (this->ParallelProjection)
+  if ( this->ParallelProjection)
     {
     // set up a rectangular parallelipiped
 
-    double width = this->ParallelScale*aspect;
+    double width = this->ParallelScale * aspect;
     double height = this->ParallelScale;
 
-    double xmin = (this->WindowCenter[0]-1.0)*width;
-    double xmax = (this->WindowCenter[0]+1.0)*width;
-    double ymin = (this->WindowCenter[1]-1.0)*height;
-    double ymax = (this->WindowCenter[1]+1.0)*height;
+    double xmin = ( this->WindowCenter[0] - 1.0 ) * width;
+    double xmax = ( this->WindowCenter[0] + 1.0 ) * width;
+    double ymin = ( this->WindowCenter[1] - 1.0 ) * height;
+    double ymax = ( this->WindowCenter[1] + 1.0 ) * height;
 
-    this->PerspectiveTransform->Ortho(xmin,xmax,ymin,ymax,
+    this->ProjectionTransform->Ortho( xmin, xmax, ymin, ymax,
                                       this->ClippingRange[0],
-                                      this->ClippingRange[1]);
+                                      this->ClippingRange[1] );
     }
   else
     {
     // set up a perspective frustum
 
-    double tmp = tan(this->ViewAngle*vtkMath::DoubleDegreesToRadians()/2);
+    double tmp = tan( vtkMath::RadiansFromDegrees( this->ViewAngle ) / 2. );
     double width;
     double height;
-    if (this->UseHorizontalViewAngle)
+    if ( this->UseHorizontalViewAngle )
       {
-      width = this->ClippingRange[0]*tmp;
-      height = this->ClippingRange[0]*tmp/aspect;
+      width = this->ClippingRange[0] * tmp;
+      height = this->ClippingRange[0] * tmp / aspect;
       }
     else
       {
-      width = this->ClippingRange[0]*tmp*aspect;
-      height = this->ClippingRange[0]*tmp;
+      width = this->ClippingRange[0] * tmp * aspect;
+      height = this->ClippingRange[0] * tmp;
       }
 
-    double xmin = (this->WindowCenter[0]-1.0)*width;
-    double xmax = (this->WindowCenter[0]+1.0)*width;
-    double ymin = (this->WindowCenter[1]-1.0)*height;
-    double ymax = (this->WindowCenter[1]+1.0)*height;
+    double xmin = ( this->WindowCenter[0] - 1.0 ) * width;
+    double xmax = ( this->WindowCenter[0] + 1.0 ) * width;
+    double ymin = ( this->WindowCenter[1] - 1.0 ) * height;
+    double ymax = ( this->WindowCenter[1] + 1.0 ) * height;
 
-    this->PerspectiveTransform->Frustum(xmin, xmax, ymin, ymax,
+    this->ProjectionTransform->Frustum( xmin, xmax, ymin, ymax,
                                         this->ClippingRange[0],
-                                        this->ClippingRange[1]);
+                                        this->ClippingRange[1] );
     }
 
-  if (this->Stereo)
+  if ( this->Stereo )
     {
     // set up a shear for stereo views
-    if (this->LeftEye)
+    if ( this->LeftEye )
       {
-      this->PerspectiveTransform->Stereo(-this->EyeAngle/2,
-                                         this->Distance);
+      this->ProjectionTransform->Stereo( -this->EyeAngle/2,
+                                          this->Distance );
       }
     else
       {
-      this->PerspectiveTransform->Stereo(+this->EyeAngle/2,
-                                         this->Distance);
+      this->ProjectionTransform->Stereo( +this->EyeAngle/2,
+                                          this->Distance );
       }
     }
 
-  if (this->ViewShear[0] != 0.0 || this->ViewShear[1] != 0.0) 
+  if ( this->ViewShear[0] != 0.0 || this->ViewShear[1] != 0.0 )
     {
-    this->PerspectiveTransform->Shear(this->ViewShear[0],
+    this->ProjectionTransform->Shear( this->ViewShear[0],
                                       this->ViewShear[1],
-                                      this->ViewShear[2]*this->Distance);
+                                      this->ViewShear[2] * this->Distance );
     }
-  
+
 }
 
+#ifndef VTK_LEGACY_REMOVE
 //----------------------------------------------------------------------------
-// Return the perspective transform matrix. See ComputePerspectiveTransform.
+// Return the projection transform matrix. See ComputeProjectionTransform.
 vtkMatrix4x4 *vtkCamera::GetPerspectiveTransformMatrix(double aspect,
                                                        double nearz,
                                                        double farz)
 {
-  this->ComputePerspectiveTransform(aspect, nearz, farz);
-  
-  // return the transform 
-  return this->PerspectiveTransform->GetMatrix();
+  VTK_LEGACY_REPLACED_BODY(vtkCamera::GetPerspectiveTransformMatrix,"VTK 5.4",
+                           vtkCamera::GetProjectionTransformMatrix);
+  return this->GetProjectionTransformMatrix(aspect,nearz,farz);
+}
+#endif
+
+//----------------------------------------------------------------------------
+// Return the projection transform matrix. See ComputeProjectionTransform.
+vtkMatrix4x4 *vtkCamera::GetProjectionTransformMatrix(double aspect,
+                                                      double nearz,
+                                                      double farz)
+{
+  this->ComputeProjectionTransform(aspect, nearz, farz);
+
+  // return the transform
+  return this->ProjectionTransform->GetMatrix();
 }
 
 //----------------------------------------------------------------------------
-// Return the perspective transform matrix. See ComputePerspectiveTransform.
+// Return the projection transform object. See ComputeProjectionTransform.
+vtkPerspectiveTransform *vtkCamera::GetProjectionTransformObject(double aspect,
+                                                                 double nearz,
+                                                                 double farz)
+{
+  this->ComputeProjectionTransform(aspect, nearz, farz);
+
+  // return the transform
+  return this->ProjectionTransform;
+}
+
+#ifndef VTK_LEGACY_REMOVE
+//----------------------------------------------------------------------------
+// Return the projection transform matrix. See ComputeProjectionTransform.
 vtkMatrix4x4 *vtkCamera::GetCompositePerspectiveTransformMatrix(double aspect,
                                                                 double nearz,
                                                                 double farz)
 {
-  // turn off stereo, the CompositePerspectiveTransformMatrix is used for
+  VTK_LEGACY_REPLACED_BODY(vtkCamera::GetCompositePerspectiveTransformMatrix,
+                           "VTK 5.4",
+                           vtkCamera::GetCompositeProjectionTransformMatrix);
+  return this->GetCompositeProjectionTransformMatrix(aspect,nearz,farz);
+}
+#endif
+
+//----------------------------------------------------------------------------
+// Return the projection transform matrix. See ComputeProjectionTransform.
+vtkMatrix4x4 *vtkCamera::GetCompositeProjectionTransformMatrix(double aspect,
+                                                               double nearz,
+                                                               double farz)
+{
+  // turn off stereo, the CompositeProjectionTransformMatrix is used for
   // picking, not for rendering.
   int stereo = this->Stereo;
   this->Stereo = 0;
 
   this->Transform->Identity();
-  this->Transform->Concatenate(this->GetPerspectiveTransformMatrix(aspect,
-                                                                   nearz,
-                                                                   farz));
+  this->Transform->Concatenate(this->GetProjectionTransformMatrix(aspect,
+                                                                  nearz,
+                                                                  farz));
   this->Transform->Concatenate(this->GetViewTransformMatrix());
 
   this->Stereo = stereo;
-  
-  // return the transform 
+
+  // return the transform
   return this->Transform->GetMatrix();
 }
 
@@ -848,7 +901,7 @@ vtkMatrix4x4 *vtkCamera::GetCompositePerspectiveTransformMatrix(double aspect,
 // Return the attached light transform matrix.
 vtkMatrix4x4 *vtkCamera::GetCameraLightTransformMatrix()
 {
-  // return the transform 
+  // return the transform
   return this->CameraLightTransform->GetMatrix();
 }
 
@@ -877,16 +930,26 @@ void vtkCamera::ComputeViewPlaneNormal()
 }
 
 //----------------------------------------------------------------------------
-void vtkCamera::SetViewPlaneNormal(double vtkNotUsed(x), 
+#ifndef VTK_LEGACY_REMOVE
+void vtkCamera::SetViewPlaneNormal(double vtkNotUsed(x),
                                    double vtkNotUsed(y),
                                    double vtkNotUsed(z))
 {
-  vtkWarningMacro(<< "SetViewPlaneNormal:  This method is deprecated, the view plane normal is calculated automatically.");
+  vtkWarningMacro(<< "SetViewPlaneNormal:  This method is deprecated, "
+    "the view plane normal is calculated automatically.");
 }
 
+//----------------------------------------------------------------------------
+void vtkCamera::SetViewPlaneNormal(const double [3])
+{
+  vtkWarningMacro(<< "SetViewPlaneNormal:  This method is deprecated, "
+    "the view plane normal is calculated automatically.");
+}
+#endif
+
 //----------------------------------------------------------------------------
 // Return the 6 planes (Ax + By + Cz + D = 0) that bound
-// the view frustum. 
+// the view frustum.
 void vtkCamera::GetFrustumPlanes(double aspect, double planes[24])
 {
   int i;
@@ -906,7 +969,7 @@ void vtkCamera::GetFrustumPlanes(double aspect, double planes[24])
   // get the composite perspective matrix
   vtkMatrix4x4::DeepCopy(
     *matrix, 
-    this->GetCompositePerspectiveTransformMatrix(aspect,-1,+1));
+    this->GetCompositeProjectionTransformMatrix(aspect,-1,+1));
   
   // transpose the matrix for use with normals
   vtkMatrix4x4::Transpose(*matrix,*matrix);
@@ -944,51 +1007,53 @@ void vtkCamera::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 
-  os << indent << "ClippingRange: (" << this->ClippingRange[0] << ", " 
+  os << indent << "ClippingRange: (" << this->ClippingRange[0] << ", "
      << this->ClippingRange[1] << ")\n";
   os << indent << "DirectionOfProjection: (" << this->DirectionOfProjection[0]
-     << ", " << this->DirectionOfProjection[1] 
+     << ", " << this->DirectionOfProjection[1]
      << ", " << this->DirectionOfProjection[2] << ")\n";
   os << indent << "Distance: " << this->Distance << "\n";
   os << indent << "EyeAngle: " << this->EyeAngle << "\n";
   os << indent << "FocalDisk: " << this->FocalDisk << "\n";
-  os << indent << "FocalPoint: (" << this->FocalPoint[0] << ", " 
+  os << indent << "FocalPoint: (" << this->FocalPoint[0] << ", "
      << this->FocalPoint[1] << ", " << this->FocalPoint[2] << ")\n";
   os << indent << "ViewShear: (" << this->ViewShear[0]
-     << ", " << this->ViewShear[1] 
+     << ", " << this->ViewShear[1]
      << ", " << this->ViewShear[2] << ")\n";
-  os << indent << "ParallelProjection: " << 
+  os << indent << "ParallelProjection: " <<
     (this->ParallelProjection ? "On\n" : "Off\n");
   os << indent << "ParallelScale: " << this->ParallelScale << "\n";
-  os << indent << "Position: (" << this->Position[0] << ", " 
+  os << indent << "Position: (" << this->Position[0] << ", "
      << this->Position[1] << ", " << this->Position[2] << ")\n";
   os << indent << "Stereo: " << (this->Stereo ? "On\n" : "Off\n");
+  os << indent << "Left Eye: " << this->LeftEye << endl;
   os << indent << "Thickness: " << this->Thickness << "\n";
   os << indent << "ViewAngle: " << this->ViewAngle << "\n";
-  os << indent << "UseHorizontalViewAngle: " << this->UseHorizontalViewAngle << "\n";
+  os << indent << "UseHorizontalViewAngle: " << this->UseHorizontalViewAngle
+     << "\n";
   os << indent << "UserTransform: ";
   if (this->UserTransform)
     {
-    os << this->UserTransform << "\n"; 
+    os << this->UserTransform << "\n";
     }
   else
     {
     os << "(none)\n";
     }
   os << indent << "ViewPlaneNormal: (" << this->ViewPlaneNormal[0]
-     << ", " << this->ViewPlaneNormal[1] 
+     << ", " << this->ViewPlaneNormal[1]
      << ", " << this->ViewPlaneNormal[2] << ")\n";
-  os << indent << "ViewUp: (" << this->ViewUp[0] << ", " 
+  os << indent << "ViewUp: (" << this->ViewUp[0] << ", "
      << this->ViewUp[1] << ", " << this->ViewUp[2] << ")\n";
-  os << indent << "WindowCenter: (" << this->WindowCenter[0] << ", " 
+  os << indent << "WindowCenter: (" << this->WindowCenter[0] << ", "
      << this->WindowCenter[1] << ")\n";
 }
 
-vtkMatrix4x4 *vtkCamera::GetViewTransformMatrix() 
+vtkMatrix4x4 *vtkCamera::GetViewTransformMatrix()
 { return this->ViewTransform->GetMatrix(); }
 
-double *vtkCamera::GetOrientation() 
+double *vtkCamera::GetOrientation()
 { return this->ViewTransform->GetOrientation(); };
 
-double *vtkCamera::GetOrientationWXYZ() 
+double *vtkCamera::GetOrientationWXYZ()
 { return this->ViewTransform->GetOrientationWXYZ(); };
