@@ -15,7 +15,7 @@
 #include "vtkInteractorStyleJoystickActor.h"
 
 #include "vtkCamera.h"
-#include "vtkCommand.h"
+#include "vtkCallbackCommand.h"
 #include "vtkMath.h"
 #include "vtkObjectFactory.h"
 #include "vtkRenderWindowInteractor.h"
@@ -25,7 +25,7 @@
 #include "vtkTransform.h"
 #include "vtkMatrix4x4.h"
 
-vtkCxxRevisionMacro(vtkInteractorStyleJoystickActor, "$Revision: 1.31 $");
+vtkCxxRevisionMacro(vtkInteractorStyleJoystickActor, "$Revision: 1.35 $");
 vtkStandardNewMacro(vtkInteractorStyleJoystickActor);
 
 //----------------------------------------------------------------------------
@@ -78,6 +78,7 @@ void vtkInteractorStyleJoystickActor::OnLeftButtonDown()
     return;
     }
 
+  this->GrabFocus(this->EventCallbackCommand);
   if (this->Interactor->GetShiftKey())
     {
     this->StartPan();
@@ -109,8 +110,13 @@ void vtkInteractorStyleJoystickActor::OnLeftButtonUp()
       this->EndRotate();
       break;
     }
+  if ( this->Interactor )
+    {
+    this->ReleaseFocus();
+    }
 }
 
+
 //----------------------------------------------------------------------------
 void vtkInteractorStyleJoystickActor::OnMiddleButtonDown() 
 {
@@ -124,6 +130,7 @@ void vtkInteractorStyleJoystickActor::OnMiddleButtonDown()
     return;
     }
 
+  this->GrabFocus(this->EventCallbackCommand);
   if (this->Interactor->GetControlKey())
     {
     this->StartDolly();
@@ -147,6 +154,11 @@ void vtkInteractorStyleJoystickActor::OnMiddleButtonUp()
       this->EndPan();
       break;
     }
+
+  if ( this->Interactor )
+    {
+    this->ReleaseFocus();
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -162,6 +174,7 @@ void vtkInteractorStyleJoystickActor::OnRightButtonDown()
     return;
     }
   
+  this->GrabFocus(this->EventCallbackCommand);
   this->StartUniformScale();
 }
 
@@ -172,6 +185,10 @@ void vtkInteractorStyleJoystickActor::OnRightButtonUp()
     {
     case VTKIS_USCALE:
       this->EndUniformScale();
+      if ( this->Interactor )
+        {
+        this->ReleaseFocus();
+        }
       break;
     }
 }
@@ -224,10 +241,10 @@ void vtkInteractorStyleJoystickActor::Rotate()
                                                        outsidept));
   
   double nxf = 
-    ((double)rwi->GetEventPosition()[0] - (double)disp_obj_center[0]) / radius;
+    (rwi->GetEventPosition()[0] - disp_obj_center[0]) / radius;
 
   double nyf = 
-    ((double)rwi->GetEventPosition()[1] - (double)disp_obj_center[1]) / radius;
+    (rwi->GetEventPosition()[1] - disp_obj_center[1]) / radius;
   
   if (nxf > 1.0)
     {
@@ -248,10 +265,10 @@ void vtkInteractorStyleJoystickActor::Rotate()
     }
   
   double newXAngle = 
-    asin(nxf) * vtkMath::RadiansToDegrees() / this->MotionFactor;
+    vtkMath::DegreesFromRadians( asin( nxf ) ) / this->MotionFactor;
 
   double newYAngle = 
-    asin(nyf) * vtkMath::RadiansToDegrees() / this->MotionFactor;
+    vtkMath::DegreesFromRadians( asin( nyf ) ) / this->MotionFactor;
   
   double scale[3];
   scale[0] = scale[1] = scale[2] = 1.0;
@@ -329,8 +346,7 @@ void vtkInteractorStyleJoystickActor::Spin()
   
   double *center = this->CurrentRenderer->GetCenter();
   
-  double yf = 
-    ((double)rwi->GetEventPosition()[1] - (double)disp_obj_center[1]) / (double)center[1];
+  double yf = (rwi->GetEventPosition()[1] - disp_obj_center[1]) / center[1];
 
   if (yf > 1.0)
     {
@@ -342,7 +358,7 @@ void vtkInteractorStyleJoystickActor::Spin()
     }
 
   double newAngle = 
-    asin(yf) * vtkMath::RadiansToDegrees() / this->MotionFactor;
+    vtkMath::DegreesFromRadians( asin( yf ) ) / this->MotionFactor;
 
   double scale[3];
   scale[0] = scale[1] = scale[2] = 1.0;
@@ -390,8 +406,8 @@ void vtkInteractorStyleJoystickActor::Pan()
   this->ComputeWorldToDisplay(obj_center[0], obj_center[1], obj_center[2], 
                               disp_obj_center);
 
-  this->ComputeDisplayToWorld((double)rwi->GetEventPosition()[0], 
-                              (double)rwi->GetEventPosition()[1],
+  this->ComputeDisplayToWorld(rwi->GetEventPosition()[0], 
+                              rwi->GetEventPosition()[1],
                               disp_obj_center[2],
                               new_pick_point);
   
@@ -449,9 +465,8 @@ void vtkInteractorStyleJoystickActor::Dolly()
   
   double *center = this->CurrentRenderer->GetCenter();
   
-  double yf = 
-    ((double)rwi->GetEventPosition()[1] - (double)disp_obj_center[1]) / (double)center[1];
-  double dollyFactor = pow((double)1.1, yf);
+  double yf = (rwi->GetEventPosition()[1] - disp_obj_center[1]) / center[1];
+  double dollyFactor = pow(1.1, yf);
 
   dollyFactor -= 1.0;
   motion_vector[0] = (view_point[0] - view_focus[0]) * dollyFactor;
@@ -504,9 +519,8 @@ void vtkInteractorStyleJoystickActor::UniformScale()
   
   double *center = this->CurrentRenderer->GetCenter();
     
-  double yf = 
-    ((double)rwi->GetEventPosition()[1] - (double)disp_obj_center[1]) / (double)center[1];
-  double scaleFactor = pow((double)1.1, yf);
+  double yf = (rwi->GetEventPosition()[1] - disp_obj_center[1]) / center[1];
+  double scaleFactor = pow(1.1, yf);
   
   double **rotate = NULL;
   
