@@ -60,7 +60,7 @@
 // query the vtkOpenGLExtensionManager to see if the extension is supported
 // with the ExtensionSupported method.  Valid names for extensions are
 // given in the OpenGL extension registry at
-// \ref http://www.opengl.org/registry/ .
+// http://www.opengl.org/registry/ .
 // You can also grep vtkgl.h (which will be in the binary build directory
 // if VTK is not installed) for appropriate names.  There are also
 // special extensions GL_VERSION_X_X (where X_X is replaced with a major
@@ -122,6 +122,7 @@
 #define __vtkOpenGLExtensionManager_h
 
 #include <vtkObject.h>
+#include <vtkWeakPointer.h> // needed for vtkWeakPointer.
 
 class vtkRenderWindow;
 
@@ -146,7 +147,7 @@ public:
   // Description:
   // Set/Get the render window to query extensions on.  If set to null,
   // justs queries the current render window.
-  vtkGetObjectMacro(RenderWindow, vtkRenderWindow);
+  vtkRenderWindow* GetRenderWindow();
   virtual void SetRenderWindow(vtkRenderWindow *renwin);
 
   // Description:
@@ -254,13 +255,13 @@ public:
   // vtkgl::ActiveTexture(vtkgl::TEXTURE0);
   // \endcode
   virtual void LoadCorePromotedExtension(const char *name);
-  
+//BTX
 protected:
   vtkOpenGLExtensionManager();
   virtual ~vtkOpenGLExtensionManager();
 
-  vtkRenderWindow *RenderWindow;
 
+  int OwnRenderWindow;
   char *ExtensionsString;
 
   vtkTimeStamp BuildTime;
@@ -277,6 +278,9 @@ protected:
 private:
   vtkOpenGLExtensionManager(const vtkOpenGLExtensionManager&); // Not implemented
   void operator=(const vtkOpenGLExtensionManager&); // Not implemented
+
+  vtkWeakPointer<vtkRenderWindow> RenderWindow;
+//ETX
 };
 
 #endif //__vtkOpenGLExtensionManager
