@@ -40,9 +40,8 @@
 
 #include <math.h>
 
-
 #ifndef VTK_IMPLEMENT_MESA_CXX
-vtkCxxRevisionMacro(vtkOpenGLPolyDataMapper, "$Revision: 1.106.4.2 $");
+vtkCxxRevisionMacro(vtkOpenGLPolyDataMapper, "$Revision: 1.109 $");
 vtkStandardNewMacro(vtkOpenGLPolyDataMapper);
 #endif
 
@@ -168,7 +167,7 @@ void vtkOpenGLPolyDataMapper::RenderPiece(vtkRenderer *ren, vtkActor *act)
 
   for (i = 0; i < numClipPlanes; i++)
     {
-     glEnable((GLenum)(GL_CLIP_PLANE0+i));
+     glEnable(static_cast<GLenum>(GL_CLIP_PLANE0+i));
     }
 
   if ( clipPlanes )
@@ -181,7 +180,7 @@ void vtkOpenGLPolyDataMapper::RenderPiece(vtkRenderer *ren, vtkActor *act)
     
     for (i = 0; i < numClipPlanes; i++)
       {    
-      plane = (vtkPlane *)clipPlanes->GetItemAsObject(i);
+      plane = static_cast<vtkPlane *>(clipPlanes->GetItemAsObject(i));
       
       plane->GetOrigin(origin);
       plane->GetNormal(normal);
@@ -219,7 +218,7 @@ void vtkOpenGLPolyDataMapper::RenderPiece(vtkRenderer *ren, vtkActor *act)
       planeEquation[3] = -(planeEquation[0]*origin[0]+
                            planeEquation[1]*origin[1]+
                            planeEquation[2]*origin[2]);
-      glClipPlane((GLenum)(GL_CLIP_PLANE0+i),planeEquation);
+      glClipPlane(static_cast<GLenum>(GL_CLIP_PLANE0+i),planeEquation);
       }
     
     actorMatrix->Delete();  
@@ -329,7 +328,7 @@ void vtkOpenGLPolyDataMapper::RenderPiece(vtkRenderer *ren, vtkActor *act)
     this->Timer->StopTimer();      
     }
 
-  this->TimeToDraw = (float)this->Timer->GetElapsedTime();
+  this->TimeToDraw = this->Timer->GetElapsedTime();
 
   // If the timer is not accurate enough, set it to a small
   // time so that it is not zero
@@ -340,7 +339,7 @@ void vtkOpenGLPolyDataMapper::RenderPiece(vtkRenderer *ren, vtkActor *act)
 
   for (i = 0; i < numClipPlanes; i++)
     {
-    glDisable((GLenum)(GL_CLIP_PLANE0+i));
+    glDisable(static_cast<GLenum>(GL_CLIP_PLANE0+i));
     }
 }
 
@@ -423,11 +422,12 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
     }
 }
 
+//-----------------------------------------
 
 #define vtkDrawPointsMacro(ptype,ntype,glVertFuncs,glInitFuncs) \
 { \
   vtkIdType nPts; unsigned short count = 0; \
-  ptype *points = (ptype *)voidPoints; \
+  ptype *points = static_cast<ptype *>(voidPoints);     \
   glInitFuncs \
   glBegin(GL_POINTS); \
   while (ptIds < endPtIds) \
@@ -444,7 +444,7 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
       { \
       cellNum += 10000; \
       count = 0; \
-      this->UpdateProgress((double)cellNum/this->TotalCells); \
+      this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells); \
       if (ren->GetRenderWindow()->CheckAbortStatus()) \
         { \
         noAbort = 0; \
@@ -459,7 +459,7 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
 #define vtkDrawPrimsMacro(ptype,ntype,prim,glVertFuncs,glInitFuncs) \
 { \
   vtkIdType nPts; unsigned short count = 0; \
-  ptype *points = (ptype *)voidPoints; \
+  ptype *points = static_cast<ptype *>(voidPoints);    \
   glInitFuncs \
   while (ptIds < endPtIds) \
     { \
@@ -477,7 +477,7 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
       { \
       cellNum += 10000; \
       count = 0; \
-      this->UpdateProgress((double)cellNum/this->TotalCells); \
+      this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells); \
       if (ren->GetRenderWindow()->CheckAbortStatus()) \
         { \
         noAbort = 0; \
@@ -491,10 +491,10 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
 #define vtkDrawPolysMacro(ptype,ntype,ttype,prim,glVertFuncs,glCellFuncs,glInitFuncs) \
 { \
   vtkIdType nPts; unsigned short count = 0; \
-  ptype *points = (ptype *)voidPoints; \
+  ptype *points = static_cast<ptype *>(voidPoints);    \
   GLenum previousGlFunction=GL_INVALID_VALUE; \
   glInitFuncs \
-  while (ptIds < endPtIds) \
+while (ptIds < endPtIds) \
     { \
     nPts = *ptIds; \
     ++ptIds; \
@@ -510,7 +510,246 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
       { \
       cellNum += 10000; \
       count = 0; \
-      this->UpdateProgress((double)cellNum/this->TotalCells); \
+      this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells); \
+      if (ren->GetRenderWindow()->CheckAbortStatus()) \
+        { \
+        noAbort = 0; \
+        break; \
+        } \
+      } \
+    if ((previousGlFunction != GL_TRIANGLES)  \
+        && (previousGlFunction != GL_QUADS)   \
+        && (previousGlFunction != GL_POINTS)) \
+      {  \
+      glEnd(); \
+      } \
+    } \
+  cellNum += count; \
+  if ((previousGlFunction == GL_TRIANGLES)  \
+      || (previousGlFunction == GL_QUADS)   \
+      || (previousGlFunction == GL_POINTS)) \
+    { \
+    glEnd(); \
+    } \
+}
+
+#define vtkDrawPolysMacro4Tri(ptype,ntype,ttype,prim,glVertFuncs,glCellFuncs,glInitFuncs) \
+{ \
+  vtkIdType nPts; unsigned short count = 0; \
+  ptype *points = static_cast<ptype *>(voidPoints);     \
+  GLenum previousGlFunction=GL_INVALID_VALUE; \
+  glInitFuncs \
+  \
+  double quad_center[3] = {0, 0, 0}; \
+  double quad_center_col[4] = {0, 0, 0, 0}; \
+  double quad_points[4][3]; \
+  double quad_points_col[4][4]; \
+  double dist_center[4] = {0, 0, 0, 0}; \
+  \
+  while (ptIds < endPtIds) \
+    { \
+    nPts = *ptIds; \
+  ++ptIds; \
+  /* If we don't want to draw a QUAD (ex : a triangle nPts = 3) */ \
+  if (nPts != 4) { \
+  /* Classic method */ \
+    vtkOpenGLBeginPolyTriangleOrQuad( prim, previousGlFunction, nPts ); \
+    glCellFuncs \
+    while (nPts > 0) \
+    { \
+    glVertFuncs \
+    ++ptIds; \
+    --nPts; \
+    } \
+  } \
+  /* If we want to draw a QUAD */ \
+  else { \
+  /* We launch glBegin(GL_TRIANGLES) mode in order to draw 4 triangles */ \
+    vtkOpenGLBeginPolyTriangleOrQuad( prim, previousGlFunction, 3 ); \
+    glCellFuncs \
+  /* We keep pointer on the first point of the first triangle */ \
+  /* ptIdsFirstPtQuad will be used for center calculation and for 2nd point of 4th triangle */ \
+    vtkIdType *ptIdsFirstPtQuad; \
+    ptIdsFirstPtQuad = ptIds; \
+  /* QUAD Center calculation */ \
+  /* We save the 4 QUAD points and their color */ \
+    GLfloat *vpt; \
+    GLubyte *vcol; \
+    for (int i=0; i<4; i++) { \
+      /* Position : */ \
+      vpt = points + 3**ptIds; \
+      quad_points[i][0] = vpt[0]; \
+      quad_points[i][1] = vpt[1]; \
+      quad_points[i][2] = vpt[2]; \
+      /* Color : */ \
+      vcol = colors + 4**ptIds; \
+      quad_points_col[i][0] = vcol[0]; \
+      quad_points_col[i][1] = vcol[1]; \
+      quad_points_col[i][2] = vcol[2]; \
+      quad_points_col[i][3] = vcol[3]; \
+      ++ptIds; \
+    } \
+  /* Actual calculation of QUAD center with the 4 summits */ \
+    quad_center[0] = (quad_points[0][0] + quad_points[1][0] + quad_points[2][0] + quad_points[3][0])/4; \
+    quad_center[1] = (quad_points[0][1] + quad_points[1][1] + quad_points[2][1] + quad_points[3][1])/4; \
+    quad_center[2] = (quad_points[0][2] + quad_points[1][2] + quad_points[2][2] + quad_points[3][2])/4; \
+  /* Color center calculation  (Interpolation on each component of RGB vector) */ \
+  /* Calculation of distances between center and summits */ \
+    for (int i=0; i<4; i++) { \
+      dist_center[i] = sqrt((quad_points[i][0] - quad_center[0])*(quad_points[i][0] - quad_center[0]) + \
+                (quad_points[i][1] - quad_center[1])*(quad_points[i][1] - quad_center[1]) + \
+                (quad_points[i][2] - quad_center[2])*(quad_points[i][2] - quad_center[2])); \
+    } \
+  /* Color interpolation (3 for RGB and 1 for Alpha transparency) */ \
+    for (int i=0; i<4; i++) { \
+      quad_center_col[i] = ((dist_center[3]*quad_points_col[1][i] + dist_center[1]*quad_points_col[3][i])/(dist_center[1] + dist_center[3]) + \
+                (dist_center[2]*quad_points_col[0][i] + dist_center[0]*quad_points_col[2][i])/(dist_center[2] + dist_center[0]) \
+                )/2; \
+    } \
+  /* We take pointer on the first QUAD point */ \
+    ptIds = ptIdsFirstPtQuad; \
+  /* Actual drawing of 4 triangles */ \
+    for (int i=0; i<4; i++) { \
+      /* 1st point */ \
+      glVertFuncs \
+      ++ptIds; \
+      /* 2nd point */ \
+      if (i >= 3) { /* If it is the last triangle */ \
+        /* this 2nd point = the 1st point of 1st triangle */ \
+        glColor3ubv(colors + 4**ptIdsFirstPtQuad); \
+        glVertex3fv(static_cast<float*>(points) + 3**ptIdsFirstPtQuad); \
+      } \
+      else { \
+        /* Else 2nd point = next point */ \
+        glVertFuncs \
+      } \
+      /* 3rd point */ \
+      glColor4f(quad_center_col[0],quad_center_col[1],quad_center_col[2],quad_center_col[3]); \
+      glVertex3f(quad_center[0],quad_center[1],quad_center[2]); \
+    } \
+  } /* End of if (nPts == 4) */ \
+    if (++count == 10000) \
+      { \
+      cellNum += 10000; \
+      count = 0; \
+      this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells); \
+      if (ren->GetRenderWindow()->CheckAbortStatus()) \
+        { \
+        noAbort = 0; \
+        break; \
+        } \
+      } \
+    if ((previousGlFunction != GL_TRIANGLES)  \
+        && (previousGlFunction != GL_QUADS)   \
+        && (previousGlFunction != GL_POINTS)) \
+      {  \
+      glEnd(); \
+      } \
+    } \
+  cellNum += count; \
+  if ((previousGlFunction == GL_TRIANGLES)  \
+      || (previousGlFunction == GL_QUADS)   \
+      || (previousGlFunction == GL_POINTS)) \
+    { \
+    glEnd(); \
+    } \
+}
+
+#define vtkDrawPolysMacro4TriTex(ptype,ntype,ttype,prim,glVertFuncs,glCellFuncs,glInitFuncs) \
+{ \
+  vtkIdType nPts; unsigned short count = 0; \
+  ptype *points = static_cast<ptype *>(voidPoints);     \
+  GLenum previousGlFunction=GL_INVALID_VALUE; \
+  glInitFuncs \
+  \
+double quad_center[3] = {0, 0, 0}; \
+double quad_center_tex = 0; \
+double quad_points[4][3]; \
+double quad_points_tex[4]; \
+double dist_center[4] = {0, 0, 0, 0}; \
+  \
+while (ptIds < endPtIds) \
+    { \
+    nPts = *ptIds; \
+  ++ptIds; \
+  /* If we don't want to draw a QUAD (ex : a triangle nPts = 3) */ \
+  if (nPts != 4) { \
+  /* Classic method */ \
+    vtkOpenGLBeginPolyTriangleOrQuad( prim, previousGlFunction, nPts ); \
+    glCellFuncs \
+    while (nPts > 0) \
+    { \
+    glVertFuncs \
+    ++ptIds; \
+    --nPts; \
+    } \
+  } \
+  /* If we want to draw a QUAD */ \
+  else { \
+    /* We launch glBegin(GL_TRIANGLES) mode in order to draw 4 triangles */ \
+    vtkOpenGLBeginPolyTriangleOrQuad( prim, previousGlFunction, 3 ); \
+    glCellFuncs \
+    /* We keep pointer on the first point of the first triangle */ \
+    /* ptIdsFirstPtQuad will be used for center calculation and for 2nd point of 4th triangle */ \
+    vtkIdType *ptIdsFirstPtQuad; \
+    ptIdsFirstPtQuad = ptIds; \
+  /* QUAD Center calculation */ \
+  /* We save the 4 QUAD points and their texture value */ \
+    GLfloat *vpt; \
+    GLfloat *vtex; \
+    for (int i=0; i<4; i++) { \
+      /* Position : */ \
+      vpt = points + 3**ptIds; \
+      quad_points[i][0] = vpt[0]; \
+      quad_points[i][1] = vpt[1]; \
+      quad_points[i][2] = vpt[2]; \
+      /* Texture : */ \
+      vtex = tcoords + *ptIds; \
+      quad_points_tex[i] = vtex[0]; \
+      ++ptIds; \
+    } \
+  /* Actual calculation of QUAD center with the 4 summits */ \
+    quad_center[0] = (quad_points[0][0] + quad_points[1][0] + quad_points[2][0] + quad_points[3][0])/4; \
+    quad_center[1] = (quad_points[0][1] + quad_points[1][1] + quad_points[2][1] + quad_points[3][1])/4; \
+    quad_center[2] = (quad_points[0][2] + quad_points[1][2] + quad_points[2][2] + quad_points[3][2])/4; \
+  /* Texture center calculation  (Interpolation on each component of RGB vector) */ \
+  /* Calculation of distances between center and summits */ \
+    for (int i=0; i<4; i++) { \
+      dist_center[i] = sqrt((quad_points[i][0] - quad_center[0])*(quad_points[i][0] - quad_center[0]) + \
+                (quad_points[i][1] - quad_center[1])*(quad_points[i][1] - quad_center[1]) + \
+                (quad_points[i][2] - quad_center[2])*(quad_points[i][2] - quad_center[2])); \
+    } \
+  /* Texture interpolation */ \
+    quad_center_tex = ((dist_center[3]*quad_points_tex[1] + dist_center[1]*quad_points_tex[3])/(dist_center[1] + dist_center[3]) + \
+              (dist_center[2]*quad_points_tex[0] + dist_center[0]*quad_points_tex[2])/(dist_center[2] + dist_center[0]) \
+              )/2; \
+  /* We take pointer on the first QUAD point */ \
+    ptIds = ptIdsFirstPtQuad; \
+  /* Actual drawing of 4 triangles */ \
+    for (int i=0; i<4; i++) { \
+      /* 1st point */ \
+      glVertFuncs \
+      ++ptIds; \
+      /* 2nd point */ \
+      if (i >= 3) { /* If it is the last triangle */ \
+        /* this 2nd point = the 1st point of 1st triangle */ \
+        glTexCoord1fv(tcoords + *ptIdsFirstPtQuad); \
+        glVertex3fv(points + 3**ptIdsFirstPtQuad); \
+      } \
+      else { \
+        /* Else 2nd point = next point */ \
+        glVertFuncs \
+      } \
+      /* 3rd point */ \
+      glTexCoord1f(quad_center_tex); \
+      glVertex3f(quad_center[0],quad_center[1],quad_center[2]); \
+    } \
+  } /* End of if (nPts == 4) */ \
+    if (++count == 10000) \
+      { \
+      cellNum += 10000; \
+      count = 0; \
+      this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells); \
       if (ren->GetRenderWindow()->CheckAbortStatus()) \
         { \
         noAbort = 0; \
@@ -536,7 +775,48 @@ static void vtkOpenGLBeginPolyTriangleOrQuad(GLenum aGlFunction,
 #define vtkDrawStripLinesMacro(ptype,ntype,ttype,prim,glVertFuncs,glCellFuncs,glInitFuncs) \
 { \
   vtkIdType nPts; \
-  ptype *points = (ptype *)voidPoints; \
+  ptype *points = static_cast<ptype *>(voidPoints);     \
+  vtkIdType *savedPtIds = ptIds; \
+  glInitFuncs \
+  while (ptIds < endPtIds) \
+    { \
+    glBegin(prim); \
+    nPts = *ptIds; \
+    ++ptIds; \
+    glCellFuncs \
+    while (nPts > 0) \
+      { \
+      glVertFuncs \
+      ptIds += 2; \
+      nPts -= 2; \
+      } \
+    glEnd(); \
+    ptIds += nPts; /* nPts could be 0 or -1 here */ \
+    } \
+  ptIds = savedPtIds; \
+  while (ptIds < endPtIds) \
+    { \
+    glBegin(prim); \
+    nPts = *ptIds; \
+    ++ptIds; \
+    glCellFuncs \
+    ++ptIds; \
+    --nPts; \
+    while (nPts > 0) \
+      { \
+      glVertFuncs \
+      ptIds += 2; \
+      nPts -= 2; \
+      } \
+    glEnd(); \
+    ptIds += nPts; /* nPts could be 0 or -1 here */ \
+    } \
+}
+
+#define vtkDrawStripLinesMacro(ptype,ntype,ttype,prim,glVertFuncs,glCellFuncs,glInitFuncs) \
+{ \
+  vtkIdType nPts; \
+  ptype *points = static_cast<ptype *>(voidPoints);     \
   vtkIdType *savedPtIds = ptIds; \
   glInitFuncs \
   while (ptIds < endPtIds) \
@@ -616,7 +896,7 @@ void vtkOpenGLPolyDataMapper::DrawPoints(int idx,
       vtkDrawPointsMacro(float, float, 
                          glNormal3fv(normals + 3**ptIds);
                          glVertex3fv(points + 3**ptIds);,
-                         float *normals = (float *)voidNormals;);
+                         float *normals = static_cast<float *>(voidNormals););
 
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_COLORS:
@@ -636,7 +916,7 @@ void vtkOpenGLPolyDataMapper::DrawPoints(int idx,
                          glNormal3fv(normals + 3**ptIds);
                          glColor4ubv(colors + 4**ptIds);
                          glVertex3fv(points + 3**ptIds);,
-                         float *normals = (float *)voidNormals;);
+                         float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT |
       VTK_PDM_NORMALS | VTK_PDM_COLORS | VTK_PDM_OPAQUE_COLORS:
@@ -644,7 +924,7 @@ void vtkOpenGLPolyDataMapper::DrawPoints(int idx,
                          glNormal3fv(normals + 3**ptIds);
                          glColor3ubv(colors + 4**ptIds);
                          glVertex3fv(points + 3**ptIds);,
-                         float *normals = (float *)voidNormals;);
+                         float *normals = static_cast<float *>(voidNormals););
       break;
     default:
     {
@@ -660,7 +940,7 @@ void vtkOpenGLPolyDataMapper::DrawPoints(int idx,
         {
         if (c)
           {
-          if (idx & 0x08)
+          if (idx & VTK_PDM_CELL_COLORS)
             {
             glColor4ubv(c->GetPointer(cellNum << 2));
             }
@@ -682,7 +962,7 @@ void vtkOpenGLPolyDataMapper::DrawPoints(int idx,
           }
         if (n)
           {
-          if (idx & 0x10)
+          if (idx & VTK_PDM_CELL_NORMALS)
             {
             glNormal3dv(n->GetTuple(cellNum));
             }
@@ -699,7 +979,7 @@ void vtkOpenGLPolyDataMapper::DrawPoints(int idx,
         {
         count = 0;
         // report progress
-        this->UpdateProgress((double)cellNum/this->TotalCells);
+        this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells);
         if (ren->GetRenderWindow()->CheckAbortStatus())
           {
           noAbort = 0;
@@ -761,7 +1041,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
       vtkDrawPrimsMacro(float, float, GL_LINE_STRIP,
                         glNormal3fv(normals + 3**ptIds);
                         glVertex3fv(points + 3**ptIds);,
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_COLORS:
@@ -780,7 +1060,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glColor4ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
         );
     break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
@@ -789,7 +1069,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glColor3ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
         );
     break;
     case VTK_PDM_POINT_TYPE_FLOAT | 
@@ -797,7 +1077,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
       vtkDrawPrimsMacro(float, float, GL_LINE_STRIP, 
                         glTexCoord1fv(tcoords + *ptIds);
                         glVertex3fv(points + 3**ptIds);,
-                        float *tcoords = (float *)voidTCoords;
+                        float *tcoords = static_cast<float *>(voidTCoords);
         );
     break;
     case VTK_PDM_POINT_TYPE_FLOAT | 
@@ -807,8 +1087,8 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glTexCoord1fv(tcoords + *ptIds);
                         glVertex3fv(points + 3**ptIds);,
-                        float *tcoords = (float *)voidTCoords;
-                        float *normals = (float *)voidNormals;
+                        float *tcoords = static_cast<float *>(voidTCoords);
+                        float *normals = static_cast<float *>(voidNormals);
         );
     break;
     default:
@@ -825,7 +1105,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
         {
         if (c)
           {
-          if (idx & 0x08)
+          if (idx & VTK_PDM_CELL_COLORS)
             {
             glColor4ubv(c->GetPointer(cellNum << 2));
             }
@@ -847,7 +1127,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
           }
         if (n)
           {
-          if (idx & 0x10)
+          if (idx & VTK_PDM_CELL_NORMALS)
             {
             glNormal3dv(n->GetTuple(cellNum));
             }
@@ -865,7 +1145,7 @@ void vtkOpenGLPolyDataMapper::DrawLines(int idx,
         {
         count = 0;
         // report progress
-        this->UpdateProgress((double)cellNum/this->TotalCells);
+        this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells);
         if (ren->GetRenderWindow()->CheckAbortStatus())
           {
           noAbort = 0;
@@ -944,7 +1224,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
       vtkDrawPolysMacro(float, float, float, rep,
                         glNormal3fv(normals + 3**ptIds);
                         glVertex3fv(points + 3**ptIds);,;,
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_COLORS:
       vtkDrawPolysMacro(float, float, float, rep,
@@ -953,7 +1233,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         PolyNormal,;);
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_COLORS  | VTK_PDM_OPAQUE_COLORS:
-      vtkDrawPolysMacro(float, float, float, rep, 
+      vtkDrawPolysMacro4Tri(float, float, float, rep, 
                         glColor3ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);, 
                         PolyNormal,;);
@@ -964,7 +1244,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glColor4ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);,;,
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
       VTK_PDM_NORMALS | VTK_PDM_COLORS  | VTK_PDM_OPAQUE_COLORS:
@@ -972,7 +1252,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glNormal3fv(normals + 3**ptIds); 
                         glColor3ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);,;, 
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | VTK_PDM_NORMALS | 
          VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORD_1D | VTK_PDM_TCOORDS:
@@ -980,8 +1260,8 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glTexCoord1fv(tcoords + *ptIds);
                         glVertex3fv(points + 3**ptIds);,;,
-                        float *normals = (float *)voidNormals;
-                        float *tcoords = (float *)voidTCoords;);
+                        float *normals = static_cast<float *>(voidNormals);
+                        float *tcoords = static_cast<float *>(voidTCoords););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | VTK_PDM_CELL_NORMALS | 
          VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORD_1D | VTK_PDM_TCOORDS:
@@ -989,17 +1269,17 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glTexCoord1fv(tcoords + *ptIds);
                         glVertex3fv(points + 3**ptIds);,
                         glNormal3fv(normals); normals += 3;,
-                        float *tcoords = (float *)voidTCoords;
-                        float *normals = (float *)voidNormals;
+                        float *tcoords = static_cast<float *>(voidTCoords);
+                        float *normals = static_cast<float *>(voidNormals);
                         normals += cellNum*3;);
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | 
          VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORD_1D | VTK_PDM_TCOORDS:
-      vtkDrawPolysMacro(float, float, float, rep, 
+      vtkDrawPolysMacro4TriTex(float, float, float, rep, 
                         glTexCoord1fv(tcoords + *ptIds);
                         glVertex3fv(points + 3**ptIds);,
                         PolyNormal;,
-                        float *tcoords = (float *)voidTCoords;);
+                               float *tcoords = static_cast<float *>(voidTCoords););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
         VTK_PDM_NORMALS | VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORDS:
@@ -1007,15 +1287,15 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glTexCoord2fv(tcoords + 2**ptIds);
                         glVertex3fv(points + 3**ptIds);,;,
-                        float *normals = (float *)voidNormals;
-                        float *tcoords = (float *)voidTCoords;);
+                        float *normals = static_cast<float *>(voidNormals);
+                        float *tcoords = static_cast<float *>(voidTCoords););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT|VTK_PDM_NORMAL_TYPE_FLOAT|
       VTK_PDM_CELL_NORMALS:
       vtkDrawPolysMacro(float, float, float, rep, 
                         glVertex3fv(points + 3**ptIds);, 
                         glNormal3fv(normals); normals += 3;,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
                         normals += cellNum*3;);
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT |
@@ -1024,7 +1304,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glColor4ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);,
                         glNormal3fv(normals); normals += 3;,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
                         normals += cellNum*3;);
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT |
@@ -1033,7 +1313,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glColor3ubv(colors + 4**ptIds);
                         glVertex3fv(points + 3**ptIds);,
                         glNormal3fv(normals); normals += 3;,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
                         normals += cellNum*3;);
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
@@ -1042,7 +1322,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glVertex3fv(points + 3**ptIds);,
                         glColor4ubv(colors); colors += 4;,
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
       VTK_PDM_NORMALS | VTK_PDM_COLORS  | VTK_PDM_OPAQUE_COLORS | 
@@ -1051,7 +1331,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glVertex3fv(points + 3**ptIds);,
                         glColor3ubv(colors); colors += 4;,
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
       VTK_PDM_CELL_NORMALS | VTK_PDM_COLORS | VTK_PDM_CELL_COLORS:
@@ -1059,7 +1339,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glVertex3fv(points + 3**ptIds);,
                         glNormal3fv(normals); normals += 3;
                         glColor4ubv(colors); colors += 4;,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
                         normals += cellNum*3;);
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
@@ -1069,7 +1349,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
                         glVertex3fv(points + 3**ptIds);,
                         glNormal3fv(normals); normals += 3;
                         glColor3ubv(colors); colors += 4;,
-                        float *normals = (float *)voidNormals;
+                        float *normals = static_cast<float *>(voidNormals);
                         normals += cellNum*3;);
       break;
     default:
@@ -1132,7 +1412,7 @@ void vtkOpenGLPolyDataMapper::DrawPolygons(int idx,
         {
         count = 0;
         // report progress
-        this->UpdateProgress((double)cellNum/this->TotalCells);
+        this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells);
         if (ren->GetRenderWindow()->CheckAbortStatus())
           {
           noAbort = 0;
@@ -1220,7 +1500,7 @@ void vtkOpenGLPolyDataMapper::DrawTStrips(int idx,
       vtkDrawPolysMacro(float, float, float, rep,
                         glNormal3fv(normals + 3**ptIds);
                         glVertex3fv(points + 3**ptIds);,;,
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_COLORS:
       vtkDrawPolysMacro(float, float, float, rep, 
@@ -1242,7 +1522,7 @@ void vtkOpenGLPolyDataMapper::DrawTStrips(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glColor4ubv(colors + (*ptIds << 2));
                         glVertex3fv(points + 3**ptIds);,;, 
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
       VTK_PDM_NORMALS | VTK_PDM_COLORS  | VTK_PDM_OPAQUE_COLORS:
@@ -1250,7 +1530,7 @@ void vtkOpenGLPolyDataMapper::DrawTStrips(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glColor3ubv(colors + (*ptIds << 2));
                         glVertex3fv(points + 3**ptIds);,;, 
-                        float *normals = (float *)voidNormals;);
+                        float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | VTK_PDM_NORMALS |
         VTK_PDM_TCOORD_1D | VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORDS:
@@ -1258,8 +1538,8 @@ void vtkOpenGLPolyDataMapper::DrawTStrips(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glTexCoord1fv(tcoords + *ptIds);
                         glVertex3fv(points + 3**ptIds);,;, 
-                        float *normals = (float *)voidNormals;
-                        float *tcoords = (float *)voidTCoords;);
+                        float *normals = static_cast<float *>(voidNormals);
+                        float *tcoords = static_cast<float *>(voidTCoords););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
         VTK_PDM_NORMALS | VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORDS:
@@ -1267,8 +1547,8 @@ void vtkOpenGLPolyDataMapper::DrawTStrips(int idx,
                         glNormal3fv(normals + 3**ptIds);
                         glTexCoord2fv(tcoords + 2**ptIds);
                         glVertex3fv(points + 3**ptIds);,;, 
-                        float *normals = (float *)voidNormals;
-                        float *tcoords = (float *)voidTCoords;);
+                        float *normals = static_cast<float *>(voidNormals);
+                        float *tcoords = static_cast<float *>(voidTCoords););
       break;
     default:
     {
@@ -1350,7 +1630,7 @@ void vtkOpenGLPolyDataMapper::DrawTStrips(int idx,
         {
         count = 0;
         // report progress
-        this->UpdateProgress((double)cellNum/this->TotalCells);
+        this->UpdateProgress(static_cast<double>(cellNum)/this->TotalCells);
         if (ren->GetRenderWindow()->CheckAbortStatus())
           {
           noAbort = 0;
@@ -1412,7 +1692,7 @@ void vtkOpenGLPolyDataMapperDrawTStripLines(int idx,
       vtkDrawStripLinesMacro(float, float, float, rep, 
                              glNormal3fv(normals + 3**ptIds);
                              glVertex3fv(points + 3**ptIds);,;,
-                             float *normals = (float *)voidNormals;);
+                             float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_COLORS:
       vtkDrawStripLinesMacro(float, float, float, rep, 
@@ -1434,7 +1714,7 @@ void vtkOpenGLPolyDataMapperDrawTStripLines(int idx,
                              glNormal3fv(normals + 3**ptIds);
                              glColor4ubv(colors + 4**ptIds);
                              glVertex3fv(points + 3**ptIds);,;,
-                             float *normals = (float *)voidNormals;);
+                             float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
       VTK_PDM_NORMALS | VTK_PDM_COLORS | VTK_PDM_OPAQUE_COLORS:
@@ -1442,7 +1722,7 @@ void vtkOpenGLPolyDataMapperDrawTStripLines(int idx,
                              glNormal3fv(normals + 3**ptIds); 
                              glColor3ubv(colors + 4**ptIds);
                              glVertex3fv(points + 3**ptIds);,;,
-                             float *normals = (float *)voidNormals;);
+                             float *normals = static_cast<float *>(voidNormals););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | VTK_PDM_NORMALS | 
       VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORD_1D | VTK_PDM_TCOORDS:
@@ -1450,8 +1730,8 @@ void vtkOpenGLPolyDataMapperDrawTStripLines(int idx,
                              glNormal3fv(normals + 3**ptIds);
                              glTexCoord1fv(tcoords + *ptIds);
                              glVertex3fv(points + 3**ptIds);,;,
-                             float *normals = (float *)voidNormals;
-                             float *tcoords = (float *)voidTCoords;);
+                             float *normals = static_cast<float *>(voidNormals);
+                             float *tcoords = static_cast<float *>(voidTCoords););
       break;
     case VTK_PDM_POINT_TYPE_FLOAT | VTK_PDM_NORMAL_TYPE_FLOAT | 
         VTK_PDM_NORMALS | VTK_PDM_TCOORD_TYPE_FLOAT | VTK_PDM_TCOORDS:
@@ -1459,8 +1739,8 @@ void vtkOpenGLPolyDataMapperDrawTStripLines(int idx,
                              glNormal3fv(normals + 3**ptIds);
                              glTexCoord2fv(tcoords + 2**ptIds);
                              glVertex3fv(points + 3**ptIds);,;,
-                             float *normals = (float *)voidNormals;
-                             float *tcoords = (float *)voidTCoords;);
+                             float *normals = static_cast<float *>(voidNormals);
+                             float *tcoords = static_cast<float *>(voidTCoords););
       break;
     default:
     {
@@ -1606,7 +1886,7 @@ void vtkOpenGLPolyDataMapperDrawTStripLines(int idx,
 // Draw method for OpenGL.
 int vtkOpenGLPolyDataMapper::Draw(vtkRenderer *aren, vtkActor *act)
 {
-  vtkOpenGLRenderer *ren = (vtkOpenGLRenderer *)aren;
+  vtkOpenGLRenderer *ren = static_cast<vtkOpenGLRenderer *>(aren);
   int rep, interpolation;
   float tran;
   vtkProperty *prop;
@@ -1923,16 +2203,3 @@ void vtkOpenGLPolyDataMapper::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
 }
-
-  
-
-
-
-
-
-
-
-
-
-
-
