@@ -55,6 +55,7 @@ class vtkUnsignedCharArray;
 #define VTK_STEREO_RIGHT        5
 #define VTK_STEREO_DRESDEN      6
 #define VTK_STEREO_ANAGLYPH     7
+#define VTK_STEREO_CHECKERBOARD 8
 
 #define VTK_CURSOR_DEFAULT   0
 #define VTK_CURSOR_ARROW     1
@@ -249,6 +250,9 @@ public:
     {this->SetStereoType(VTK_STEREO_DRESDEN);}
   void SetStereoTypeToAnaglyph()
     {this->SetStereoType(VTK_STEREO_ANAGLYPH);}
+  void SetStereoTypeToCheckerboard()
+    {this->SetStereoType(VTK_STEREO_CHECKERBOARD);}
+
   const char *GetStereoTypeAsString();
 
   // Description:
