@@ -45,7 +45,14 @@ class vtkCullerCollection;
 class vtkLight;
 class vtkPainter;
 class vtkIdentColoredPainter;
+class vtkHardwareSelector;
+class vtkRendererDelegate;
+class vtkRenderPass;
+
+#if !defined(VTK_LEGACY_REMOVE)
 class vtkVisibleCellSelector;
+#endif
+
 
 class VTK_RENDERING_EXPORT vtkRenderer : public vtkViewport
 {
@@ -318,6 +325,13 @@ public:
   vtkSetMacro(Layer, int);
   vtkGetMacro(Layer, int);
 
+  // Description:
+  // Normally a renderer is treated as transparent if Layer > 0. To treat a
+  // renderer at Layer 0 as transparent, set this flag to true.
+  vtkSetMacro(PreserveDepthBuffer, int);
+  vtkGetMacro(PreserveDepthBuffer, int);
+  vtkBooleanMacro(PreserveDepthBuffer, int);
+
   // Description:
   // Returns a boolean indicating if this renderer is transparent.  It is
   // transparent if it is not in the deepest layer of its render window.
@@ -426,6 +440,28 @@ public:
   // Initial value is false.
   vtkGetMacro(LastRenderingUsedDepthPeeling,int);
   
+  // Description:
+  // Set/Get a custom Render call. Allows to hook a Render call from an
+  // external project.It will be used in place of vtkRenderer::Render() if it
+  // is not NULL and its Used ivar is set to true.
+  // Initial value is NULL.
+  void SetDelegate(vtkRendererDelegate *d);
+  vtkGetObjectMacro(Delegate,vtkRendererDelegate);
+
+  //BTX
+  // Description:
+  // Set/Get a custom render pass.
+  // Initial value is NULL.
+  void SetPass(vtkRenderPass *p);
+  vtkGetObjectMacro(Pass,vtkRenderPass);
+  //ETX
+  
+  // Description:
+  // Get the current hardware selector. If the Selector is set, it implies the
+  // current render pass is for selection. Mappers/Properties may choose to
+  // behave differently when rendering for hardware selection.
+  vtkGetObjectMacro(Selector, vtkHardwareSelector);
+//BTX
 protected:
   vtkRenderer();
   ~vtkRenderer();
@@ -481,6 +517,7 @@ protected:
   // Shows what layer this renderer belongs to.  Only of interested when
   // there are layered renderers.
   int                Layer;
+  int                PreserveDepthBuffer;
 
   // Holds the result of ComputeVisiblePropBounds so that it is visible from
   // wrapped languages
@@ -572,8 +609,8 @@ protected:
   // Initial value is false.
   int LastRenderingUsedDepthPeeling;
   
+#if !defined(VTK_LEGACY_REMOVE)
   // VISIBLE CELL SELECTION ----------------------------------------
-  //BTX  
   friend class vtkVisibleCellSelector;
 
   //Description:
@@ -583,7 +620,7 @@ protected:
   enum {NOT_SELECTING = 0, COLOR_BY_PROCESSOR, COLOR_BY_ACTOR, 
         COLOR_BY_CELL_ID_HIGH, COLOR_BY_CELL_ID_MID, COLOR_BY_CELL_ID_LOW,
         COLOR_BY_VERTEX};  
-  //ETX
+
   vtkSetMacro(SelectMode, int);
   vtkSetMacro(SelectConst, unsigned int);
 
@@ -619,12 +656,30 @@ protected:
   unsigned int SelectConst;
   vtkIdentColoredPainter *IdentPainter;
   // End Ivars for visible cell selecting.
+#endif
+
+  // HARDWARE SELECTION ----------------------------------------
+  friend class vtkHardwareSelector;
+
+  // Description:
+  // Called by vtkHardwareSelector when it begins rendering for selection.
+  void SetSelector(vtkHardwareSelector* selector)
+    { this->Selector = selector; this->Modified(); }
+
+  // End Ivars for visible cell selecting.
+  vtkHardwareSelector* Selector;
 
   //---------------------------------------------------------------
+  friend class vtkRendererDelegate;
+  vtkRendererDelegate *Delegate;
+
+  friend class vtkRenderPass;
+  vtkRenderPass *Pass;
   
 private:
   vtkRenderer(const vtkRenderer&);  // Not implemented.
   void operator=(const vtkRenderer&);  // Not implemented.
+//ETX
 };
 
 inline vtkLightCollection *vtkRenderer::GetLights() {
