@@ -36,9 +36,11 @@
 #include "vtkPolyData.h"
 #include "vtkPolygon.h"
 #include "vtkProperty.h"
+#include "vtkOpenGLProperty.h"
 #include "vtkRenderer.h"
 #include "vtkRenderWindow.h"
 #include "vtkShaderDeviceAdapter.h"
+#include "vtkGLSLShaderDeviceAdapter2.h"
 #include "vtkShaderProgram.h"
 #include "vtkSmartPointer.h"
 #include "vtkTimerLog.h"
@@ -55,13 +57,15 @@ public:
     vtkDataArray* Array;
     };
   typedef vtkstd::vector<vtkInfo> InfoVector;
+  typedef vtkstd::vector<vtkDataArray *> DataArrayVector;
   InfoVector CellAttributesCache;
   InfoVector PointAttributesCache;
+  DataArrayVector MultiTextureCoords;
   vtkSmartPointer<vtkGenericVertexAttributeMapping> Mappings;
 };
 
 //-----------------------------------------------------------------------------
-vtkCxxRevisionMacro(vtkStandardPolyDataPainter, "$Revision: 1.9 $");
+vtkCxxRevisionMacro(vtkStandardPolyDataPainter, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkStandardPolyDataPainter);
 //-----------------------------------------------------------------------------
 static inline int vtkStandardPolyDataPainterGetTotalCells(vtkPolyData* pd,
@@ -96,6 +100,11 @@ void vtkStandardPolyDataPainter::PrintSelf(ostream &os, vtkIndent indent)
   this->Superclass::PrintSelf(os, indent);
 }
 
+void vtkStandardPolyDataPainter::AddMultiTextureCoordsArray(vtkDataArray * array)
+{
+  this->Internal->MultiTextureCoords.push_back(array);
+}
+
 //-----------------------------------------------------------------------------
 void vtkStandardPolyDataPainter::ProcessInformation(vtkInformation* info)
 {
@@ -112,7 +121,8 @@ void vtkStandardPolyDataPainter::ProcessInformation(vtkInformation* info)
 
 //-----------------------------------------------------------------------------
 void vtkStandardPolyDataPainter::UpdateGenericAttributesCache(
-  vtkShaderDeviceAdapter* shaderDevice)
+  vtkShaderDeviceAdapter* shaderDevice,
+  vtkGLSLShaderDeviceAdapter2* shaderDevice2)
 {
   if (this->Internal->Mappings)
     {
@@ -159,15 +169,24 @@ void vtkStandardPolyDataPainter::UpdateGenericAttributesCache(
         // This caches the attribute index.
         // This is essential since we don't want to call GetAttributeLocation in
         // glBegin()/glEnd().
-        shaderDevice->SendAttribute(vertexAttributeName, 0, 0, 0, 0);
+        if(shaderDevice)
+          {
+          shaderDevice->SendAttribute(vertexAttributeName, 0, 0, 0, 0);
+          }
+        if(shaderDevice2)
+          {
+          shaderDevice2->SendAttribute(vertexAttributeName, 0, 0, 0, 0);
+          }
         }
       }
     }
 }
 
 //-----------------------------------------------------------------------------
-void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer, vtkActor* actor, 
-  unsigned long typeflags)
+void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer,
+                                                vtkActor* actor, 
+                                                unsigned long typeflags,
+                                                bool forceCompileOnly)
 {
   if (typeflags == 0)
     {
@@ -187,33 +206,53 @@ void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer, vtkActor*
   vtkIdType startCell = 0;
   int interpolation = property->GetInterpolation();
   vtkShaderDeviceAdapter* shaderDevice=0;
-
+  vtkGLSLShaderDeviceAdapter2* shaderDevice2=0;
   this->Internal->PointAttributesCache.clear();
   this->Internal->CellAttributesCache.clear();
-  if (property->GetShading() && property->GetShaderProgram())
+  if(property->GetShading())
     {
-    // Preprocess the generic vertex attributes that we need to pass to the
-    // shader.
-    shaderDevice = property->GetShaderProgram()->GetShaderDeviceAdapter();
-    if (shaderDevice)
+    if(property->GetShaderProgram())
       {
-      shaderDevice->PrepareForRender();
+      // Preprocess the generic vertex attributes that we need to pass to the
+      // shader.
+      shaderDevice = property->GetShaderProgram()->GetShaderDeviceAdapter();
+      if (shaderDevice)
+        {
+        shaderDevice->PrepareForRender();
+        }
+      }
+    else
+      {
+      vtkOpenGLProperty *oglProperty=vtkOpenGLProperty::SafeDownCast(property);
+      if(oglProperty!=0)
+        {
+        if(oglProperty->GetCurrentShaderProgram2()!=0)
+          {
+          // Preprocess the generic vertex attributes that we need to pass to the
+          // shader.
+          shaderDevice2 = oglProperty->GetShaderDeviceAdapter2();
+          if(shaderDevice2!=0)
+            {
+            shaderDevice2->PrepareForRender();
+            }
+          }
+        }
       }
-    this->UpdateGenericAttributesCache(shaderDevice);
     }
+  this->UpdateGenericAttributesCache(shaderDevice,shaderDevice2);
 
 
   if (typeflags & vtkPainter::VERTS)
     {
     this->DrawCells(VTK_POLY_VERTEX, pd->GetVerts(), startCell,
-      shaderDevice, renderer, 0, interpolation);
+                    shaderDevice, shaderDevice2, renderer, 0, interpolation);
     }
 
   startCell += pd->GetNumberOfVerts();
   if (typeflags & vtkPainter::LINES)
     {
     this->DrawCells(VTK_POLY_LINE, pd->GetLines(), startCell,
-      shaderDevice, renderer, 0, interpolation);
+                    shaderDevice, shaderDevice2, renderer, 0, interpolation);
     }
   
   startCell += pd->GetNumberOfLines();
@@ -223,13 +262,15 @@ void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer, vtkActor*
     if (property->GetRepresentation() == VTK_WIREFRAME)
       {
       this->DrawCells(VTK_TETRA, pd->GetPolys(), startCell,
-        shaderDevice, renderer, this->BuildNormals, interpolation);
+                      shaderDevice, shaderDevice2, renderer,
+                      this->BuildNormals, interpolation);
       }
     else
 #endif
       {
       this->DrawCells(VTK_POLYGON, pd->GetPolys(), startCell,
-        shaderDevice, renderer, this->BuildNormals, interpolation);
+                      shaderDevice, shaderDevice2, renderer,
+                      this->BuildNormals, interpolation);
       }
     }
  
@@ -237,7 +278,8 @@ void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer, vtkActor*
   if (typeflags & vtkPainter::STRIPS)
     {
     this->DrawCells(VTK_TRIANGLE_STRIP, pd->GetStrips(), startCell,
-      shaderDevice, renderer, this->BuildNormals, interpolation);
+      shaderDevice, shaderDevice2, renderer, this->BuildNormals,
+                    interpolation);
     }
 
   this->Timer->StopTimer();
@@ -246,7 +288,7 @@ void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer, vtkActor*
   // let the superclass pass on the request to delegate painter.
   // Ofcouse, more than likely, this call will never have a delegate,
   // but anyways.
-  this->Superclass::RenderInternal(renderer, actor, typeflags);
+  this->Superclass::RenderInternal(renderer, actor, typeflags,forceCompileOnly);
 
 
   this->Internal->PointAttributesCache.clear();
@@ -254,11 +296,15 @@ void vtkStandardPolyDataPainter::RenderInternal(vtkRenderer* renderer, vtkActor*
 }
 
 //-----------------------------------------------------------------------------
-void vtkStandardPolyDataPainter::DrawCells(int mode, vtkCellArray *connectivity,
-                                   vtkIdType startCellId,
-                                   vtkShaderDeviceAdapter* shaderDevice,
-                                   vtkRenderer *renderer,
-                                   int buildnormals, int interpolation)
+void vtkStandardPolyDataPainter::DrawCells(
+  int mode,
+  vtkCellArray *connectivity,
+  vtkIdType startCellId,
+  vtkShaderDeviceAdapter *shaderDevice,
+  vtkGLSLShaderDeviceAdapter2 *shaderDevice2,
+  vtkRenderer *renderer,
+  int buildnormals,
+  int interpolation)
 {
   vtkPolyData* pd = this->GetInputAsPolyData();
 
@@ -351,7 +397,7 @@ void vtkStandardPolyDataPainter::DrawCells(int mode, vtkCellArray *connectivity,
       fielddata_cellId++;
       }
 
-    // Send generic attributes associated with the cell.
+    // Send generic attributes associated with the cell. Shaders style 1.
     vtkInternal::InfoVector::iterator gaIter = this->Internal->CellAttributesCache.begin();
     for (; shaderDevice && gaIter != this->Internal->CellAttributesCache.end(); ++gaIter)
       {
@@ -368,7 +414,25 @@ void vtkStandardPolyDataPainter::DrawCells(int mode, vtkCellArray *connectivity,
         (siComp>=0) ? a->GetVoidPointer(numc*cellId+siComp) :
         a->GetVoidPointer(numc*cellId));
       }
-
+    
+    // Send generic attributes associated with the cell. Shaders style 2.
+    gaIter = this->Internal->CellAttributesCache.begin();
+    for (; shaderDevice2 && gaIter != this->Internal->CellAttributesCache.end(); ++gaIter)
+      {
+      vtkDataArray* a = gaIter->Array; 
+      unsigned int mappingsIndex = gaIter->MappingsIndex;
+      int numc = a->GetNumberOfComponents();
+      int siComp = this->Internal->Mappings->GetComponent(mappingsIndex);
+      // if siComp==-1, then all components of the array are sent,
+      // otherwise only the choosen component is sent.
+      shaderDevice2->SendAttribute(
+        this->Internal->Mappings->GetAttributeName(mappingsIndex),
+        (siComp>=0)? 1: numc,
+        a->GetDataType(),
+        (siComp>=0) ? a->GetVoidPointer(numc*cellId+siComp) :
+        a->GetVoidPointer(numc*cellId));
+      }
+    
     for (vtkIdType cellpointi = 0; cellpointi < npts; cellpointi++)
       {
       vtkIdType pointId = pts[cellpointi];
@@ -425,7 +489,7 @@ void vtkStandardPolyDataPainter::DrawCells(int mode, vtkCellArray *connectivity,
           VTK_DOUBLE, polyNorm);
         }
 
-      // Send generic attributes associated with the point.
+      // Send generic attributes associated with the point. Shader style 1.
       gaIter = this->Internal->PointAttributesCache.begin();
       for (; shaderDevice && gaIter != 
         this->Internal->PointAttributesCache.end(); ++gaIter)
@@ -444,11 +508,58 @@ void vtkStandardPolyDataPainter::DrawCells(int mode, vtkCellArray *connectivity,
           a->GetVoidPointer(numc*pointId));
         }
 
-      // Send the point position as the last attribute.
-      // TODO: how to mark that point position is being sent? since,
-      // vtkDataSetAttributes doesn't define point positions and hence has
-      // no enum for that. For now, vtkPointData::NUM_ATTRIBUTES marks
-      // point positions.
+      
+      // Send generic attributes associated with the point. Shader style 2.
+      gaIter = this->Internal->PointAttributesCache.begin();
+      for (; shaderDevice2 && gaIter != 
+        this->Internal->PointAttributesCache.end(); ++gaIter)
+        {
+        vtkDataArray* a = gaIter->Array; 
+        unsigned int mappingsIndex = gaIter->MappingsIndex;
+        int numc = a->GetNumberOfComponents();
+        int siComp = this->Internal->Mappings->GetComponent(mappingsIndex);
+        // if siComp==-1, then all components of the array are sent,
+        // otherwise only the choosen component is sent.
+        shaderDevice2->SendAttribute(
+          this->Internal->Mappings->GetAttributeName(mappingsIndex),
+          (siComp>=0)? 1: numc,
+          a->GetDataType(),
+          (siComp>=0) ? a->GetVoidPointer(numc*pointId+siComp) :
+          a->GetVoidPointer(numc*pointId));
+        }
+      
+      // Check for any multitexture attributes and send them.
+      gaIter = this->Internal->PointAttributesCache.begin();
+      for (; gaIter != this->Internal->PointAttributesCache.end(); ++gaIter)
+        {
+        vtkDataArray* a = gaIter->Array; 
+        unsigned int mappingsIndex = gaIter->MappingsIndex;
+        int numc = a->GetNumberOfComponents();
+        int siComp = this->Internal->Mappings->GetComponent(mappingsIndex);
+        int textureIndex = this->Internal->Mappings->GetTextureUnit(mappingsIndex);
+
+        if(textureIndex >= 0)
+          {
+          device->SendMultiTextureCoords(
+            (siComp>=0)? 1: numc,
+            a->GetDataType(), 
+            a->GetVoidPointer(0),
+            textureIndex,
+            numc*pointId
+            );
+          }
+        }
+
+      //vtkInformationVector *inArrayVec = 
+      //  this->Information->Get(vtkAlgorithm::INPUT_ARRAYS_TO_PROCESS());
+      //int numArrays = inArrayVec->GetNumberOfInformationObjects();
+
+      //for(int i = 0; i < numArrays; i++)
+      //  {
+      //  //this->
+      //  }
+
+
       device->SendAttribute(vtkPointData::NUM_ATTRIBUTES, 3, 
         pointtype, voidpoints, 3*pointId);
       }
