@@ -156,6 +156,10 @@ public:
   // Make this windows OpenGL context the current context.
   void MakeCurrent();
 
+  // Description:
+  // Tells if this window is the current OpenGL context for the calling thread.
+  virtual bool IsCurrent();
+  
   // Description:
   // Get report of capabilities for the render window
   const char *ReportCapabilities();
