@@ -96,6 +96,10 @@ public:
   // Make this window the current OpenGL context.
   void MakeCurrent();
 
+  // Description:
+  // Tells if this window is the current OpenGL context for the calling thread.
+  virtual bool IsCurrent();
+  
   // Description:
   // If called, allow MakeCurrent() to skip cache-check when called.
   // MakeCurrent() reverts to original behavior of cache-checking     
