@@ -3,8 +3,8 @@
   Program:   DICOMParser
   Module:    $RCSfile: DICOMParser.cxx,v $
   Language:  C++
-  Date:      $Date: 2006/03/24 15:59:50 $
-  Version:   $Revision: 1.14.4.1 $
+  Date:      $Date: 2008-06-17 15:54:33 $
+  Version:   $Revision: 1.20 $
 
   Copyright (c) 2003 Matt Turek
   All rights reserved.
@@ -25,6 +25,9 @@
 #pragma warning ( push, 3 )
 #endif 
 
+#include "DICOMConfig.h"
+#include "DICOMParser.h"
+#include "DICOMCallback.h"
 
 #include <stdlib.h>
 #if !defined(__MWERKS__)
@@ -36,13 +39,9 @@
 #include <sys/types.h>
 #endif
 
+#include <string.h>
 #include <string>
 
-#include "DICOMConfig.h"
-#include "DICOMParser.h"
-#include "DICOMCallback.h"
-
-
 // Define DEBUG_DICOM to get debug messages sent to dicom_stream::cerr
 // #define DEBUG_DICOM
 
@@ -175,7 +174,7 @@ bool DICOMParser::ReadHeader() {
   this->Implementation->Datatypes.clear();
 
   long fileSize = DataFile->GetSize();
-  do 
+  do
     {
     this->ReadNextRecord(group, element, datatype);
 
@@ -194,15 +193,15 @@ bool DICOMParser::ReadHeader() {
 // return true if this is your image type, false if it is not
 //
 bool DICOMParser::IsDICOMFile(DICOMFile* file) {
-  char magic_number[4];    
+  char magic_number[4];
   file->SkipToStart();
   file->Read((void*)magic_number,4);
-  if (CheckMagic(magic_number)) 
+  if (CheckMagic(magic_number))
     {
     return(true);
-    } 
+    }
   // try with optional skip
-  else 
+  else
     {
     file->Skip(OPTIONAL_SKIP-4);
     file->Read((void*)magic_number,4);
@@ -226,8 +225,11 @@ bool DICOMParser::IsDICOMFile(DICOMFile* file) {
       bool dicom;
       if (group == 0x0002 || group == 0x0008)
         {
-        dicom_stream::cerr << "No DICOM magic number found, but file appears to be DICOM." << dicom_stream::endl;
-        dicom_stream::cerr << "Proceeding without caution."  << dicom_stream::endl;
+        dicom_stream::cerr
+          << "No DICOM magic number found, but file appears to be DICOM."
+          << dicom_stream::endl;
+        dicom_stream::cerr << "Proceeding without caution."
+          << dicom_stream::endl;
         dicom = true;
         }
       else
@@ -248,9 +250,9 @@ bool DICOMParser::IsValidRepresentation(doublebyte rep, quadbyte& len, VRTypes &
 {
   switch (rep)
     {
-    case DICOMParser::VR_AW: 
-    case DICOMParser::VR_AE: 
-    case DICOMParser::VR_AS: 
+    case DICOMParser::VR_AW:
+    case DICOMParser::VR_AE:
+    case DICOMParser::VR_AS:
     case DICOMParser::VR_CS:
     case DICOMParser::VR_UI:
     case DICOMParser::VR_DA:
@@ -263,11 +265,11 @@ bool DICOMParser::IsValidRepresentation(doublebyte rep, quadbyte& len, VRTypes &
     case DICOMParser::VR_ST:
     case DICOMParser::VR_TM:
     case DICOMParser::VR_UT: // new
-    case DICOMParser::VR_SH: 
+    case DICOMParser::VR_SH:
     case DICOMParser::VR_FL:
     case DICOMParser::VR_SL:
-    case DICOMParser::VR_AT: 
-    case DICOMParser::VR_UL: 
+    case DICOMParser::VR_AT:
+    case DICOMParser::VR_UL:
     case DICOMParser::VR_US:
     case DICOMParser::VR_SS:
     case DICOMParser::VR_FD:
@@ -277,7 +279,7 @@ bool DICOMParser::IsValidRepresentation(doublebyte rep, quadbyte& len, VRTypes &
 
     case DICOMParser::VR_OB: // OB - LE
     case DICOMParser::VR_OW:
-    case DICOMParser::VR_UN:      
+    case DICOMParser::VR_UN:
     case DICOMParser::VR_SQ:
       DataFile->ReadDoubleByte();
       len = DataFile->ReadQuadByte();
@@ -292,7 +294,7 @@ bool DICOMParser::IsValidRepresentation(doublebyte rep, quadbyte& len, VRTypes &
       DataFile->Skip(-2);
       len = DataFile->ReadQuadByte();
       mytype = DICOMParser::VR_UNKNOWN;
-      return false;  
+      return false;
     }
 }
 
@@ -303,7 +305,7 @@ void DICOMParser::ReadNextRecord(doublebyte& group, doublebyte& element, DICOMPa
   //
   // FIND A WAY TO STOP IF THERE ARE NO MORE CALLBACKS.
   //
-  // DO WE NEED TO ENSURE THAT WHEN A CALLBACK IS ADDED THAT 
+  // DO WE NEED TO ENSURE THAT WHEN A CALLBACK IS ADDED THAT
   // THE IMPLICIT TYPE MAP IS UPDATED?  ONLY IF THERE ISN'T
   // A VALUE IN THE IMPLICIT TYPE MAP.
   //
@@ -330,7 +332,7 @@ void DICOMParser::ReadNextRecord(doublebyte& group, doublebyte& element, DICOMPa
   mytype = DICOMParser::VR_UNKNOWN;
   this->IsValidRepresentation(representation, length, mytype);
 
-  DICOMParserMap::iterator iter = 
+  DICOMParserMap::iterator iter =
     Implementation->Map.find(DICOMMapKey(group,element));
 
   VRTypes callbackType;
@@ -344,7 +346,7 @@ void DICOMParser::ReadNextRecord(doublebyte& group, doublebyte& element, DICOMPa
 
     DICOMMapKey ge = (*iter).first;
     callbackType = VRTypes(((*iter).second.first));
-  
+
     if (callbackType != mytype &&
         mytype != VR_UNKNOWN)
       {
@@ -353,7 +355,7 @@ void DICOMParser::ReadNextRecord(doublebyte& group, doublebyte& element, DICOMPa
       //
       callbackType = mytype;
       }
- 
+
 #ifdef DEBUG_DICOM
     this->DumpTag(this->ParserOutputFile, group, element, callbackType, tempdata, length);
 #endif
@@ -404,8 +406,8 @@ void DICOMParser::ReadNextRecord(doublebyte& group, doublebyte& element, DICOMPa
       }
     else
       {
-      if (this->DataFile->GetPlatformIsBigEndian() == true)  
-        { 
+      if (this->DataFile->GetPlatformIsBigEndian() == true)
+        {
         switch (callbackType)
           {
           case DICOMParser::VR_OW:
@@ -416,8 +418,7 @@ void DICOMParser::ReadNextRecord(doublebyte& group, doublebyte& element, DICOMPa
             break;
           case DICOMParser::VR_FL:
           case DICOMParser::VR_FD:
-            DICOMFile::swap4((uint*) tempdata, (uint*) tempdata, length/sizeof(uint));
-            // dicom_stream::cout << "Float byte swap needed!" << dicom_stream::endl;
+            // No need to byte swap here, since these values were read by sscanf
             break;
           case DICOMParser::VR_SL:
           case DICOMParser::VR_UL:
@@ -636,7 +637,7 @@ void DICOMParser::AddDICOMTagCallbackToAllTags(DICOMCallback* cb)
   DICOMParserMap::iterator miter;
   for (miter = Implementation->Map.begin();
        miter != Implementation->Map.end();
-       miter++);
+       miter++)
   {
   dicom_stl::vector<DICOMCallback*>* callbacks = (*miter).second.second;
   callbacks->push_back(cb);
