@@ -10,17 +10,18 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: default.h,v 1.1.4.1 2007/02/06 18:51:59 barre Exp $
+ * RCS: @(#) $Id: default.h,v 1.2 2008-01-23 20:16:05 barre Exp $
  */
 
 #ifndef _DEFAULT
 #define _DEFAULT
 
-#if defined(__WIN32__) || defined(_WIN32)
+#if defined(__WIN32__) || defined(_WIN32) || \
+    defined(__CYGWIN__) || defined(__MINGW32__)
 #   include "tkWinDefault.h"
 #else
-#   if defined(MAC_TCL)
-#  include "tkMacDefault.h"
+#   if defined(MAC_OSX_TK)
+#  include "tkMacOSXDefault.h"
 #   else
 #  include "tkUnixDefault.h"
 #   endif
