@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixPort.h,v 1.1.2.2 2008-10-21 20:12:16 barre Exp $
+ * RCS: @(#) $Id: tkUnixPort.h,v 1.2 2008-01-23 20:16:05 barre Exp $
  */
 
 #ifndef _UNIXPORT
@@ -204,4 +204,15 @@
 #define TkpPrintWindowId(buf,w) \
   sprintf((buf), "%#08lx", (unsigned long) (w))
 
+/*
+ * The following declaration is used to get access to a private Tcl interface
+ * that is needed for portability reasons.
+ *
+ * Disabled for now to determined whether we really still need this.
+
+#ifndef _TCLINT
+#include <tclInt.h>
+#endif
+ */
+
 #endif /* _UNIXPORT */
