@@ -36,15 +36,10 @@
 *
 * expp - ex_get_prop: read object property 
 *
-* author - Larry A. Schoof, Sandia National Laboratories
-*          Victor R. Yarberry, Sandia National Laboratories
-*
-* environment - UNIX
-*
 * entry conditions - 
 *   input parameters:
 *       int     exoid                   exodus file id
-*       int     obj_type                type of object (element block, node
+*       ex_entity_type obj_type                type of object (element block, node
 *                                               set or side set)
 *       int     obj_id                  id of object for which value 
 *                                               is desired
@@ -56,7 +51,7 @@
 *
 * revision history - 
 *
-*  $Id: exgp.c,v 1.2 2006-11-29 18:09:13 dcthomp Exp $
+*  $Id: exgp.c,v 1.3 2009-01-16 14:32:01 utkarsh Exp $
 *
 *****************************************************************************/
 
@@ -69,19 +64,18 @@
  */
 
 int ex_get_prop (int   exoid,
-                 int   obj_type,
+                 ex_entity_type obj_type,
                  int   obj_id,
                  const char *prop_name,
                  int  *value)
 {
+   int status;
    int num_props, i, propid;
    int found = FALSE;
-   long  start[1]; 
-   nclong l_val;
+   size_t start[1]; 
+   int l_val;
    char name[MAX_VAR_NAME_LENGTH+1];
    char tmpstr[MAX_STR_LENGTH+1];
-   char obj_stype[MAX_VAR_NAME_LENGTH+1];
-   char obj_vtype[MAX_VAR_NAME_LENGTH+1];
 
    char errmsg[MAX_ERR_LENGTH];
 
@@ -91,51 +85,41 @@ int ex_get_prop (int   exoid,
 
    num_props = ex_get_num_props(exoid,obj_type);
 
-   switch (obj_type)
-   {
-     case EX_ELEM_BLOCK:
-       strcpy (obj_stype, "element block");
-       strcpy (obj_vtype, VAR_ID_EL_BLK);
-       break;
-     case EX_NODE_SET:
-       strcpy (obj_stype, "node set");
-       strcpy (obj_vtype, VAR_NS_IDS);
-       break;
-     case EX_SIDE_SET:
-       strcpy (obj_stype, "side set");
-       strcpy (obj_vtype, VAR_SS_IDS);
-       break;
-     case EX_ELEM_MAP:
-       strcpy (obj_stype, "element map");
-       strcpy (obj_vtype, VAR_EM_PROP(1));
-       break;
-     case EX_NODE_MAP:
-       strcpy (obj_stype, "node map");
-       strcpy (obj_vtype, VAR_NM_PROP(1));
-       break;
-     default:
-       exerrval = EX_BADPARAM;
-       sprintf(errmsg, "Error: object type %d not supported; file id %d",
-               obj_type, exoid);
-       ex_err("ex_get_prop",errmsg,exerrval);
-       return(EX_FATAL);
-   }
-
-   for (i=1; i<=num_props; i++)
-   {
+   for (i=1; i<=num_props; i++) {
      switch (obj_type){
        case EX_ELEM_BLOCK:
          strcpy (name, VAR_EB_PROP(i));
          break;
+       case EX_EDGE_BLOCK:
+         strcpy (name, VAR_ED_PROP(i));
+         break;
+       case EX_FACE_BLOCK:
+         strcpy (name, VAR_FA_PROP(i));
+         break;
        case EX_NODE_SET:
          strcpy (name, VAR_NS_PROP(i));
          break;
+       case EX_EDGE_SET:
+         strcpy (name, VAR_ES_PROP(i));
+         break;
+       case EX_FACE_SET:
+         strcpy (name, VAR_FS_PROP(i));
+         break;
+       case EX_ELEM_SET:
+         strcpy (name, VAR_ELS_PROP(i));
+         break;
        case EX_SIDE_SET:
          strcpy (name, VAR_SS_PROP(i));
          break;
        case EX_ELEM_MAP:
          strcpy (name, VAR_EM_PROP(i));
          break;
+       case EX_FACE_MAP:
+         strcpy (name, VAR_FAM_PROP(i));
+         break;
+       case EX_EDGE_MAP:
+         strcpy (name, VAR_EDM_PROP(i));
+         break;
        case EX_NODE_MAP:
          strcpy (name, VAR_NM_PROP(i));
          break;
@@ -147,9 +131,8 @@ int ex_get_prop (int   exoid,
          return(EX_FATAL);
      }
 
-     if ((propid = ncvarid (exoid, name)) == -1)
-     {
-       exerrval = ncerr;
+     if ((status = nc_inq_varid(exoid, name, &propid)) != NC_NOERR) {
+       exerrval = status;
        sprintf(errmsg,
           "Error: failed to locate property array %s in file id %d",
                name, exoid);
@@ -157,20 +140,17 @@ int ex_get_prop (int   exoid,
        return (EX_FATAL);
      }
 
-/*   compare stored attribute name with passed property name   */
-
+     /*   compare stored attribute name with passed property name   */
      memset(tmpstr, 0, MAX_STR_LENGTH+1);
-     if ((ncattget (exoid, propid, ATT_PROP_NAME, tmpstr)) == -1)
-     {
-       exerrval = ncerr;
+     if ((status = nc_get_att_text(exoid, propid, ATT_PROP_NAME, tmpstr)) != NC_NOERR) {
+       exerrval = status;
        sprintf(errmsg,
               "Error: failed to get property name in file id %d", exoid);
        ex_err("ex_get_prop",errmsg,exerrval);
        return (EX_FATAL);
      }
 
-     if (strcmp(tmpstr, prop_name) == 0) 
-     {
+     if (strcmp(tmpstr, prop_name) == 0) {
        found = TRUE;
        break;
      }
@@ -178,12 +158,11 @@ int ex_get_prop (int   exoid,
 
 /* if property is not found, return warning */
 
-   if (!found)
-   {
+   if (!found) {
      exerrval = EX_BADPARAM;
      sprintf(errmsg,
        "Warning: %s property %s not defined in file id %d",
-        obj_stype, prop_name, exoid);
+	     ex_name_of_object(obj_type), prop_name, exoid);
      ex_err("ex_get_prop",errmsg,exerrval);
      return (EX_WARN);
    }
@@ -192,35 +171,30 @@ int ex_get_prop (int   exoid,
 /* array at proper index; ex_id_lkup returns an index that is 1-based,   */
 /* but netcdf expects 0-based arrays so subtract 1                       */
 
-   start[0] = ex_id_lkup (exoid, obj_vtype, obj_id);
-   if (exerrval != 0) 
-   {
-     if (exerrval == EX_NULLENTITY)
-     {
+   start[0] = ex_id_lkup (exoid, obj_type, obj_id);
+   if (exerrval != 0)  {
+     if (exerrval == EX_NULLENTITY) {
        sprintf(errmsg,
               "Warning: %s id %d is NULL in file id %d",
-               obj_stype, obj_id, exoid);
+               ex_name_of_object(obj_type), obj_id, exoid);
        ex_err("ex_get_prop",errmsg,EX_MSG);
        return (EX_WARN);
-     }
-     else
-     {
-       exerrval = ncerr;
+     } else {
+       exerrval = status;
        sprintf(errmsg,
              "Error: failed to locate id %d in %s property array in file id %d",
-               obj_id, obj_stype, exoid);
+               obj_id, ex_name_of_object(obj_type), exoid);
        ex_err("ex_get_prop",errmsg,exerrval);
        return (EX_FATAL);
      }
    }
    start[0] = start[0] - 1;
 
-   if (ncvarget1 (exoid, propid, start, &l_val) == -1)
-   {
-     exerrval = ncerr;
+   if ((status = nc_get_var1_int (exoid, propid, start, &l_val)) != NC_NOERR) {
+     exerrval = status;
      sprintf(errmsg,
             "Error: failed to read value in %s property array in file id %d",
-             obj_stype, exoid);
+             ex_name_of_object(obj_type), exoid);
      ex_err("ex_get_prop",errmsg,exerrval);
      return (EX_FATAL);
    }
