@@ -32,18 +32,11 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  */
+
 /*****************************************************************************
 *
 * exgsp - ex_get_side_set_param
 *
-* author - Sandia National Laboratories
-*          Larry A. Schoof - Original
-*          James A. Schutt - 8 byte float and standard C definitions
-*          Vic Yarberry    - Added headers and error logging
-*
-*          
-* environment - UNIX
-*
 * entry conditions - 
 *   input parameters:
 *       int     exoid                   exodus file id
@@ -56,16 +49,20 @@
 *
 * revision history - 
 *
-*  $Id: exgsp.c,v 1.2 2006-11-29 18:09:13 dcthomp Exp $
+*  $Id: exgsp.c,v 1.3 2009-01-16 14:32:01 utkarsh Exp $
 *
 *****************************************************************************/
 
 #include "exodusII.h"
-#include "exodusII_int.h"
 
-/*
+/*!
  * reads the number of sides and the number of distribution factors which 
  * describe a single side set
+ * \param      exoid                   exodus file id
+ * \param      side_set_id             side set id
+ * \param[out] num_side_in_set         number of sides in the side set
+ * \param[out] num_dist_fact_in_set    number of distribution factors in the 
+ * \deprecated Use ex_get_set_param()(exoid, EX_SIDE_SET, side_set_id, num_side_in_set, num_dist_fact_in_set)
  */
 
 int ex_get_side_set_param (int  exoid,
@@ -74,5 +71,5 @@ int ex_get_side_set_param (int  exoid,
                            int *num_dist_fact_in_set)
 {
   return ex_get_set_param(exoid, EX_SIDE_SET, side_set_id, 
-        num_side_in_set, num_dist_fact_in_set);
+			  num_side_in_set, num_dist_fact_in_set);
 }
