@@ -36,12 +36,6 @@
 *
 * exgssn - ex_get_side_set_node_list
 *
-* author - Sandia National Laboratories
-*          Vic Yarberry    - Original
-*
-*
-* environment - UNIX
-*
 * entry conditions - 
 *   input parameters:
 *       int     exoid                   exodus file id
@@ -54,7 +48,7 @@
 *
 * revision history - 
 *
-*  $Id: exgssn.c,v 1.8 2006-12-03 11:08:44 dcthomp Exp $
+*  $Id: exgssn.c,v 1.10 2009-01-19 18:46:31 utkarsh Exp $
 *
 *****************************************************************************/
 
@@ -77,7 +71,7 @@ int ex_get_side_set_node_list(int exoid,
   int  num_side_sets, num_elem_blks, num_df, ndim;
   int tot_num_elem = 0, tot_num_ss_elem = 0, elem_num = 0;
   int connect_offset, side_num, node_pos;
-  int *elem_blk_ids, *connect = 0;
+  int *elem_blk_ids, *connect = NULL; 
   int *ss_elem_ndx, *ss_elem_node_ndx, *ss_parm_ndx;
   int *side_set_elem_list, *side_set_side_list;
   int elem_ctr, node_ctr, elem_num_pos;
@@ -85,17 +79,7 @@ int ex_get_side_set_node_list(int exoid,
   float fdum;
   char *cdum, elem_type[MAX_STR_LENGTH+1];
 
-  struct elem_blk_parm
-  {
-    char elem_type[MAX_STR_LENGTH+1];
-    int elem_blk_id;
-    int num_elem_in_blk;
-    int num_nodes_per_elem;
-    int num_nodes_per_side;
-    int num_attr;
-    int elem_ctr;
-    int elem_type_val;
-  } *elem_blk_parms;
+  struct elem_blk_parm  *elem_blk_parms;
 
 /* side to node translation tables - 
      These tables are used to look up the side number based on the
@@ -195,7 +179,7 @@ int ex_get_side_set_node_list(int exoid,
 
 /* Lookup index of side set id in VAR_SS_IDS array */
 
-  ex_id_lkup(exoid,VAR_SS_IDS,side_set_id);
+  ex_id_lkup(exoid,EX_SIDE_SET,side_set_id);
   if (exerrval != 0) 
   {
      if (exerrval == EX_NULLENTITY)
@@ -379,32 +363,31 @@ int ex_get_side_set_node_list(int exoid,
     elem_blk_parms[i].num_attr = num_attr;
 
     for (m=0; m < (int)strlen(elem_type); m++)
-      elem_blk_parms[i].elem_type[m] = (char)
-              toupper((int)elem_type[m]);
-    elem_blk_parms[i].elem_type[m] = '\0';
+      elem_blk_parms[i].elem_type[m] = toupper(elem_type[m]);
+    elem_blk_parms[i].elem_type[m] = NULL_ELEMENT;
 
     if (strncmp(elem_blk_parms[i].elem_type,"CIRCLE",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = CIRCLE;
       /* set side set node stride */
-        elem_blk_parms[i].num_nodes_per_side = 1;
+        elem_blk_parms[i].num_nodes_per_side[0] = 1;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"SPHERE",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = SPHERE;
       /* set side set node stride */
-        elem_blk_parms[i].num_nodes_per_side = 1;
+        elem_blk_parms[i].num_nodes_per_side[0] = 1;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"QUAD",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = QUAD;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 4)
-        elem_blk_parms[i].num_nodes_per_side = 2;
+        elem_blk_parms[i].num_nodes_per_side[0] = 2;
       else if (elem_blk_parms[i].num_nodes_per_elem == 5)
-        elem_blk_parms[i].num_nodes_per_side = 2;
+        elem_blk_parms[i].num_nodes_per_side[0] = 2;
       else
-        elem_blk_parms[i].num_nodes_per_side = 3;
+        elem_blk_parms[i].num_nodes_per_side[0] = 3;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"TRIANGLE",3) == 0)
     {
@@ -413,16 +396,16 @@ int ex_get_side_set_node_list(int exoid,
       if (ndim == 2)  /* 2d TRIs */
       {
         if (elem_blk_parms[i].num_nodes_per_elem == 3)
-          elem_blk_parms[i].num_nodes_per_side = 2;
+          elem_blk_parms[i].num_nodes_per_side[0] = 2;
         else 
-          elem_blk_parms[i].num_nodes_per_side = 3;
+          elem_blk_parms[i].num_nodes_per_side[0] = 3;
       }
       else if (ndim == 3)  /* 3d TRIs */
       {
         if (elem_blk_parms[i].num_nodes_per_elem == 3)
-          elem_blk_parms[i].num_nodes_per_side = 3;
+          elem_blk_parms[i].num_nodes_per_side[0] = 3;
         else 
-          elem_blk_parms[i].num_nodes_per_side = 6;
+          elem_blk_parms[i].num_nodes_per_side[0] = 6;
       }
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"SHELL",3) == 0)
@@ -430,64 +413,64 @@ int ex_get_side_set_node_list(int exoid,
       elem_blk_parms[i].elem_type_val = SHELL;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 2) /* KLUDGE for 2D Shells*/
-        elem_blk_parms[i].num_nodes_per_side = 2;
+        elem_blk_parms[i].num_nodes_per_side[0] = 2;
       else if (elem_blk_parms[i].num_nodes_per_elem == 4)
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else
-        elem_blk_parms[i].num_nodes_per_side = 8;
+        elem_blk_parms[i].num_nodes_per_side[0] = 8;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"HEX",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = HEX;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 8)  /* 8-node bricks */
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else if (elem_blk_parms[i].num_nodes_per_elem == 9)  /* 9-node bricks */
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else if (elem_blk_parms[i].num_nodes_per_elem == 12)  /* HEXSHELLS */
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else if (elem_blk_parms[i].num_nodes_per_elem == 27)  /* 27-node bricks */
-        elem_blk_parms[i].num_nodes_per_side = 9;
+        elem_blk_parms[i].num_nodes_per_side[0] = 9;
       else 
-        elem_blk_parms[i].num_nodes_per_side = 8;
+        elem_blk_parms[i].num_nodes_per_side[0] = 8;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"TETRA",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = TETRA;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 4)
-        elem_blk_parms[i].num_nodes_per_side = 3;
+        elem_blk_parms[i].num_nodes_per_side[0] = 3;
       else if (elem_blk_parms[i].num_nodes_per_elem == 8)
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else
-        elem_blk_parms[i].num_nodes_per_side = 6;
+        elem_blk_parms[i].num_nodes_per_side[0] = 6;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"WEDGE",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = WEDGE;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 6)
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else
-        elem_blk_parms[i].num_nodes_per_side = 8;
+        elem_blk_parms[i].num_nodes_per_side[0] = 8;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"PYRAMID",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = PYRAMID;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 5)
-        elem_blk_parms[i].num_nodes_per_side = 4;
+        elem_blk_parms[i].num_nodes_per_side[0] = 4;
       else
-        elem_blk_parms[i].num_nodes_per_side = 8;
+        elem_blk_parms[i].num_nodes_per_side[0] = 8;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"BEAM",3) == 0)
     {
       elem_blk_parms[i].elem_type_val = BEAM;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 2)
-        elem_blk_parms[i].num_nodes_per_side = 2;
+        elem_blk_parms[i].num_nodes_per_side[0] = 2;
       else 
-        elem_blk_parms[i].num_nodes_per_side = 3;
+        elem_blk_parms[i].num_nodes_per_side[0] = 3;
     }
     else if ( (strncmp(elem_blk_parms[i].elem_type,"TRUSS",3) == 0) ||
               (strncmp(elem_blk_parms[i].elem_type,"BAR",3) == 0) ||
@@ -496,21 +479,21 @@ int ex_get_side_set_node_list(int exoid,
       elem_blk_parms[i].elem_type_val = TRUSS;
       /* determine side set node stride */
       if (elem_blk_parms[i].num_nodes_per_elem == 2)
-        elem_blk_parms[i].num_nodes_per_side = 2;
+        elem_blk_parms[i].num_nodes_per_side[0] = 2;
       else 
-        elem_blk_parms[i].num_nodes_per_side = 3;
+        elem_blk_parms[i].num_nodes_per_side[0] = 3;
     }
     else if (strncmp(elem_blk_parms[i].elem_type,"NULL",3) == 0)
     {
-      elem_blk_parms[i].elem_type_val = '\0';
-      elem_blk_parms[i].num_nodes_per_side = 0;
+      elem_blk_parms[i].elem_type_val = NULL_ELEMENT;
+      elem_blk_parms[i].num_nodes_per_side[0] = 0;
       elem_blk_parms[i].num_elem_in_blk = 0;
     }
     else
     { /* unsupported element type; no problem if no sides specified for
          this element block */
       elem_blk_parms[i].elem_type_val = UNK;
-      elem_blk_parms[i].num_nodes_per_side = 0;
+      elem_blk_parms[i].num_nodes_per_side[0] = 0;
     }
     elem_blk_parms[i].elem_blk_id = elem_blk_ids[i];    /* save id */
     elem_ctr += elem_blk_parms[i].num_elem_in_blk;
@@ -562,7 +545,7 @@ int ex_get_side_set_node_list(int exoid,
   {
     for (j=0; j<num_elem_blks; j++)
     {
-      if (elem_blk_parms[j].elem_type_val != '\0')
+      if (elem_blk_parms[j].elem_type_val != NULL_ELEMENT)
       if (side_set_elem_list[i] <= elem_blk_parms[j].elem_ctr)
         break;
     }
@@ -627,7 +610,7 @@ int ex_get_side_set_node_list(int exoid,
         node_ctr += 3;  /* 3 node side */
     }
     else /* all other element types */
-      node_ctr += elem_blk_parms[j].num_nodes_per_side;
+      node_ctr += elem_blk_parms[j].num_nodes_per_side[0];
   }
 
   /* All setup, ready to go ... */
