@@ -2,16 +2,19 @@
  *  Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: libvers.c,v 1.1 2005/07/15 21:56:38 andy Exp $ */
+/* $Id: libvers.c,v 1.5 2007-08-28 11:11:13 dcthomp Exp $ */
 
+#include "ncconfig.h"
 #include "netcdf.h"
 
 /*
- * A version string.
+ * A version string. This whole function is not needed in netCDF-4,
+ * which has it's own version of this function.
  */
-#define SKIP_LEADING_GARBAGE 33  /* # of chars prior to the actual version */
+#ifndef USE_NETCDF4
+#define SKIP_LEADING_GARBAGE 33 /* # of chars prior to the actual version */
 #define XSTRING(x)  #x
-#define STRING(x)  XSTRING(x)
+#define STRING(x) XSTRING(x)
 static const char nc_libvers[] =
   "\044Id: \100(#) netcdf library version " STRING(VERSION) " of "__DATE__" "__TIME__" $";
 
@@ -21,12 +24,4 @@ nc_inq_libvers(void)
   return &nc_libvers[SKIP_LEADING_GARBAGE];
 }
 
-#if 0 /* TEST JIG */
-#include <stdio.h>
-
-main()
-{
-  (void) printf("Version: %s\n", nc_inq_libvers());
-  return 0;
-}
-#endif
+#endif /* USE_NETCDF4*/
