@@ -2,11 +2,11 @@
  *  Copyright 1996, University Corporation for Atmospheric Research
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: v1hpg.c,v 1.1 2005/07/15 21:56:39 andy Exp $ */
+/* $Id: v1hpg.c,v 1.9 2007-08-28 11:11:13 dcthomp Exp $ */
 
 #include "nc.h"
-#include <stdio.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <assert.h>
 #include "rnd.h"
@@ -14,9 +14,12 @@
 
 /*
  * This module defines the external representation
- * of the "header" of a netcdf version one file.
+ * of the "header" of a netcdf version one file and
+ * the version two variant that uses 64-bit file 
+ * offsets instead of the 32-bit file offsets in version 
+ * one files.
  * For each of the components of the NC structure,
- * There are (static) ncx_len_XXX(), ncx_put_XXX()
+ * There are (static) ncx_len_XXX(), v1h_put_XXX()
  * and v1h_get_XXX() functions. These define the
  * external representation of the components.
  * The exported entry points for the whole NC structure
@@ -31,6 +34,7 @@
 static const schar ncmagic[] = {'C', 'D', 'F', 0x02};
 static const schar ncmagic1[] = {'C', 'D', 'F', 0x01};
 
+
 /*
  * v1hs == "Version 1 Header Stream"
  *
@@ -42,11 +46,11 @@ static const schar ncmagic1[] = {'C', 'D', 'F', 0x01};
  */
 typedef struct v1hs {
   ncio *nciop;
-  off_t offset;  /* argument to nciop->get() */
+  off_t offset; /* argument to nciop->get() */
   size_t extent;  /* argument to nciop->get() */
   int flags;  /* set to RGN_WRITE for write */
-        int version;    /* either 1 for normal netcdf or 2 for 8-byte offset version */
-  void *base;  /* beginning of current buffer */
+        int version;    /* format variant: NC_FORMAT_CLASSIC or NC_FORMAT_64BIT */
+  void *base; /* beginning of current buffer */
   void *pos;  /* current position in buffer */
   void *end;  /* end of current buffer = base + extent */
 } v1hs;
@@ -81,7 +85,7 @@ fault_v1hs(v1hs *gsp, size_t extent)
 
   if(gsp->base != NULL)
   {
-    const size_t incr = (char *)gsp->pos - (char *)gsp->base;
+    const ptrdiff_t incr = (char *)gsp->pos - (char *)gsp->base;
     status = rel_v1hs(gsp);
     if(status)
       return status;
@@ -89,10 +93,10 @@ fault_v1hs(v1hs *gsp, size_t extent)
   }
   
   if(extent > gsp->extent)
-    gsp->extent = extent;  
+    gsp->extent = extent; 
 
   status = gsp->nciop->get(gsp->nciop,
-       gsp->offset, gsp->extent,
+      gsp->offset, gsp->extent,
       gsp->flags, &gsp->base);
   if(status)
     return status;
@@ -117,27 +121,29 @@ fprintf(stderr, "nextread %lu, remaining %lu\n",
   (unsigned long)((char *)gsp->end - (char *)gsp->pos));
 #endif
 
-  if((char *)gsp->pos + nextread < (char *)gsp->end)
+  if((char *)gsp->pos + nextread <= (char *)gsp->end)
     return ENOERR;
   return fault_v1hs(gsp, nextread);
 }
 
 /* End v1hs */
 
+/* Write a size_t to the header */
 static int
 v1h_put_size_t(v1hs *psp, const size_t *sp)
 {
-  int status = check_v1hs(psp, psp->version == 1 ? 4 : 8);
+  int status = check_v1hs(psp, X_SIZEOF_SIZE_T);
   if(status != ENOERR)
     return status;
   return ncx_put_size_t(&psp->pos, sp);
 }
 
 
+/* Read a size_t from the header */
 static int
 v1h_get_size_t(v1hs *gsp, size_t *sp)
 {
-  int status = check_v1hs(gsp, gsp->version == 1 ? 4 : 8);
+  int status = check_v1hs(gsp, X_SIZEOF_SIZE_T);
   if(status != ENOERR)
     return status;
   return ncx_get_size_t((const void **)(&gsp->pos), sp);
@@ -148,6 +154,7 @@ v1h_get_size_t(v1hs *gsp, size_t *sp)
 
 #define X_SIZEOF_NC_TYPE X_SIZEOF_INT
 
+/* Write a nc_type to the header */
 static int
 v1h_put_nc_type(v1hs *psp, const nc_type *typep)
 {
@@ -161,6 +168,7 @@ v1h_put_nc_type(v1hs *psp, const nc_type *typep)
 }
 
 
+/* Read a nc_type from the header */
 static int
 v1h_get_nc_type(v1hs *gsp, nc_type *typep)
 {
@@ -191,6 +199,7 @@ v1h_get_nc_type(v1hs *gsp, nc_type *typep)
 
 #define X_SIZEOF_NCTYPE X_SIZEOF_INT
 
+/* Write a NCtype to the header */
 static int
 v1h_put_NCtype(v1hs *psp, NCtype type)
 {
@@ -203,6 +212,7 @@ v1h_put_NCtype(v1hs *psp, NCtype type)
   return status;
 }
 
+/* Read a NCtype from the header */
 static int
 v1h_get_NCtype(v1hs *gsp, NCtype *typep)
 {
@@ -247,6 +257,7 @@ ncx_len_NC_string(const NC_string *ncstrp)
 }
 
 
+/* Write a NC_string to the header */
 static int
 v1h_put_NC_string(v1hs *psp, const NC_string *ncstrp)
 {
@@ -270,6 +281,7 @@ v1h_put_NC_string(v1hs *psp, const NC_string *ncstrp)
 }
 
 
+/* Read a NC_string from the header */
 static int
 v1h_get_NC_string(v1hs *gsp, NC_string **ncstrpp)
 {
@@ -336,6 +348,7 @@ ncx_len_NC_dim(const NC_dim *dimp)
 }
 
 
+/* Write a NC_dim to the header */
 static int
 v1h_put_NC_dim(v1hs *psp, const NC_dim *dimp)
 {
@@ -352,6 +365,7 @@ v1h_put_NC_dim(v1hs *psp, const NC_dim *dimp)
   return ENOERR;
 }
 
+/* Read a NC_dim from the header */
 static int
 v1h_get_NC_dim(v1hs *gsp, NC_dim **dimpp)
 {
@@ -387,6 +401,7 @@ unwind_name:
 }
 
 
+/* How much space in the header is required for this NC_dimarray? */
 static size_t
 ncx_len_NC_dimarray(const NC_dimarray *ncap)
 {
@@ -407,6 +422,7 @@ ncx_len_NC_dimarray(const NC_dimarray *ncap)
 }
 
 
+/* Write a NC_dimarray to the header */
 static int
 v1h_put_NC_dimarray(v1hs *psp, const NC_dimarray *ncap)
 {
@@ -461,6 +477,7 @@ v1h_put_NC_dimarray(v1hs *psp, const NC_dimarray *ncap)
 }
 
 
+/* Read a NC_dimarray from the header */
 static int
 v1h_get_NC_dimarray(v1hs *gsp, NC_dimarray *ncap)
 {
@@ -498,7 +515,7 @@ v1h_get_NC_dimarray(v1hs *gsp, NC_dimarray *ncap)
       status = v1h_get_NC_dim(gsp, dpp);
       if(status)
       {
-        ncap->nelems = dpp - ncap->value;
+        ncap->nelems = (size_t)(dpp - ncap->value);
         free_NC_dimarrayV(ncap);
         return status;
       }
@@ -571,6 +588,7 @@ v1h_put_NC_attrV(v1hs *psp, const NC_attr *attrp)
   return ENOERR;
 }
 
+/* Write a NC_attr to the header */
 static int
 v1h_put_NC_attr(v1hs *psp, const NC_attr *attrp)
 {
@@ -610,8 +628,6 @@ v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
   void *value = attrp->xvalue;
   size_t nget; 
 
-  assert(gsp->extent % X_ALIGN == 0);
-  
   do {
     nget = MIN(perchunk, remaining);
   
@@ -631,6 +647,7 @@ v1h_get_NC_attrV(v1hs *gsp, NC_attr *attrp)
 }
 
 
+/* Read a NC_attr from the header */
 static int
 v1h_get_NC_attr(v1hs *gsp, NC_attr **attrpp)
 {
@@ -676,6 +693,7 @@ unwind_name:
 }
 
 
+/* How much space in the header is required for this NC_attrarray? */
 static size_t
 ncx_len_NC_attrarray(const NC_attrarray *ncap)
 {
@@ -696,6 +714,7 @@ ncx_len_NC_attrarray(const NC_attrarray *ncap)
 }
 
 
+/* Write a NC_attrarray to the header */
 static int
 v1h_put_NC_attrarray(v1hs *psp, const NC_attrarray *ncap)
 {
@@ -750,6 +769,7 @@ v1h_put_NC_attrarray(v1hs *psp, const NC_attrarray *ncap)
 }
 
 
+/* Read a NC_attrarray from the header */
 static int
 v1h_get_NC_attrarray(v1hs *gsp, NC_attrarray *ncap)
 {
@@ -786,7 +806,7 @@ v1h_get_NC_attrarray(v1hs *gsp, NC_attrarray *ncap)
       status = v1h_get_NC_attr(gsp, app);
       if(status)
       {
-        ncap->nelems = app - ncap->value;
+        ncap->nelems = (size_t)(app - ncap->value);
         free_NC_attrarrayV(ncap);
         return status;
       }
@@ -824,6 +844,7 @@ ncx_len_NC_var(const NC_var *varp, size_t sizeof_off_t)
 }
 
 
+/* Write a NC_var to the header */
 static int
 v1h_put_NC_var(v1hs *psp, const NC_var *varp)
 {
@@ -857,10 +878,6 @@ v1h_put_NC_var(v1hs *psp, const NC_var *varp)
   if(status != ENOERR)
     return status;
 
-#if 0
-  fprintf(stderr, "VarWr: %s--%d bytes\n", varp->name->cp, varp->len);
-#endif
-  
   status = check_v1hs(psp, psp->version == 1 ? 4 : 8);
   if(status != ENOERR)
      return status;
@@ -872,6 +889,7 @@ v1h_put_NC_var(v1hs *psp, const NC_var *varp)
 }
 
 
+/* Read a NC_var from the header */
 static int
 v1h_get_NC_var(v1hs *gsp, NC_var **varpp)
 {
@@ -915,10 +933,6 @@ v1h_get_NC_var(v1hs *gsp, NC_var **varpp)
   if(status != ENOERR)
      goto unwind_alloc;
 
-#if 0
-  fprintf(stderr, "VarRd: %s--%d bytes\n", varp->name->cp, varp->len);
-#endif
-
   status = check_v1hs(gsp, gsp->version == 1 ? 4 : 8);
   if(status != ENOERR)
      goto unwind_alloc;
@@ -940,6 +954,7 @@ unwind_name:
 }
 
 
+/* How much space in the header is required for this NC_vararray? */
 static size_t
 ncx_len_NC_vararray(const NC_vararray *ncap, size_t sizeof_off_t)
 {
@@ -960,6 +975,7 @@ ncx_len_NC_vararray(const NC_vararray *ncap, size_t sizeof_off_t)
 }
 
 
+/* Write a NC_vararray to the header */
 static int
 v1h_put_NC_vararray(v1hs *psp, const NC_vararray *ncap)
 {
@@ -1014,6 +1030,7 @@ v1h_put_NC_vararray(v1hs *psp, const NC_vararray *ncap)
 }
 
 
+/* Read a NC_vararray from the header */
 static int
 v1h_get_NC_vararray(v1hs *gsp, NC_vararray *ncap)
 {
@@ -1051,7 +1068,7 @@ v1h_get_NC_vararray(v1hs *gsp, NC_vararray *ncap)
       status = v1h_get_NC_var(gsp, vpp);
       if(status)
       {
-        ncap->nelems = vpp - ncap->value;
+        ncap->nelems = (size_t)(vpp - ncap->value);
         free_NC_vararrayV(ncap);
         return status;
       }
@@ -1070,8 +1087,8 @@ v1h_get_NC_vararray(v1hs *gsp, NC_vararray *ncap)
  * Recompute the shapes of all variables
  * Sets ncp->begin_var to start of first variable.
  * Sets ncp->begin_rec to start of first record variable.
- * Returns -1 on error. The only possible error is an reference
- * to a non existent dimension, which would occur for a corrupt
+ * Returns -1 on error. The only possible error is a reference
+ * to a non existent dimension, which could occur for a corrupted
  * netcdf file.
  */
 static int
@@ -1079,8 +1096,8 @@ NC_computeshapes(NC *ncp)
 {
   NC_var **vpp = (NC_var **)ncp->vars.value;
   NC_var *const *const end = &vpp[ncp->vars.nelems];
-  NC_var *first_var = NULL;  /* first "non-record" var */
-  NC_var *first_rec = NULL;  /* first "record" var */
+  NC_var *first_var = NULL; /* first "non-record" var */
+  NC_var *first_rec = NULL; /* first "record" var */
   int status;
 
   ncp->begin_var = (off_t) ncp->xsz;
@@ -1096,18 +1113,19 @@ NC_computeshapes(NC *ncp)
     if(status != ENOERR)
       return(status);
 
-      if(IS_RECVAR(*vpp))  
+      if(IS_RECVAR(*vpp)) 
     {
-        if(first_rec == NULL)  
+        if(first_rec == NULL) 
         first_rec = *vpp;
       ncp->recsize += (*vpp)->len;
     }
-    else if(first_var == NULL)
+    else
     {
-      first_var = *vpp;
+            if(first_var == NULL)
+              first_var = *vpp;
       /*
        * Overwritten each time thru.
-       * Usually overwritten in first_rec != NULL clause.
+       * Usually overwritten in first_rec != NULL clause below.
        */
       ncp->begin_rec = (*vpp)->begin + (off_t)(*vpp)->len;
     }
@@ -1118,8 +1136,8 @@ NC_computeshapes(NC *ncp)
     assert(ncp->begin_rec <= first_rec->begin);
     ncp->begin_rec = first_rec->begin;
     /*
-      * for special case of exactly one record variable, pack value
-      */
+     * for special case of exactly one record variable, pack value
+     */
     if(ncp->recsize == first_rec->len)
       ncp->recsize = *first_rec->dsizes * first_rec->xsz;
   }
@@ -1142,6 +1160,7 @@ NC_computeshapes(NC *ncp)
 }
 
 
+/* How much space in the header is required for the NC data structure? */
 size_t
 ncx_len_NC(const NC *ncp, size_t sizeof_off_t)
 {
@@ -1158,10 +1177,11 @@ ncx_len_NC(const NC *ncp, size_t sizeof_off_t)
 }
 
 
+/* Write the file header */
 int
 ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
 {
-  int status = ENOERR;
+  int status;
   v1hs ps; /* the get stream */
 
   assert(ncp != NULL);
@@ -1218,13 +1238,15 @@ ncx_put_NC(const NC *ncp, void **xpp, off_t offset, size_t extent)
     status = ncx_putn_schar_schar(&ps.pos, sizeof(ncmagic), ncmagic);
   else
     status = ncx_putn_schar_schar(&ps.pos, sizeof(ncmagic1), ncmagic1);
-
   if(status != ENOERR)
     goto release;
 
-  status = ncx_put_size_t(&ps.pos, &ncp->numrecs);
+  {
+  const size_t nrecs = NC_get_numrecs(ncp);
+  status = ncx_put_size_t(&ps.pos, &nrecs);
   if(status != ENOERR)
     goto release;
+  }
 
   assert((char *)ps.pos < (char *)ps.end);
 
@@ -1247,6 +1269,7 @@ release:
 }
 
 
+/* Make the in-memory NC structure from reading the file header */
 int
 nc_get_NC(NC *ncp)
 {
@@ -1269,21 +1292,37 @@ nc_get_NC(NC *ncp)
     /*
      * Come up with a reasonable stream read size.
      */
-    size_t extent = ncp->xsz;
+    off_t filesize;
+    size_t extent;
+    
+    extent = ncp->xsz;
     if(extent <= MIN_NC_XSZ)
     {
+      status = ncio_filesize(ncp->nciop, &filesize);
+      if(status)
+          return status;
       /* first time read */
       extent = ncp->chunk;
       /* Protection for when ncp->chunk is huge;
        * no need to read hugely. */
-            if(extent > 4096)
+      if(extent > 4096)
         extent = 4096;
+      if(extent > (size_t)filesize)
+        extent = filesize;
     }
     else if(extent > ncp->chunk)
     {
       extent = ncp->chunk;
     }
-    
+
+    /*
+     * Invalidate the I/O buffers to force a read of the header
+     * region.
+     */
+    status = gs.nciop->sync(gs.nciop);
+    if(status)
+      return status;
+
     status = fault_v1hs(&gs, extent);
     if(status)
       return status;
@@ -1311,18 +1350,26 @@ nc_get_NC(NC *ncp)
       gs.version = 1;
     } else if (magic[sizeof(ncmagic)-1] == 0x2) {
       gs.version = 2;
+      fSet(ncp->flags, NC_64BIT_OFFSET);
+      /* Now we support version 2 file access on non-LFS systems -- rkr */
+#if 0
       if (sizeof(off_t) != 8) {
-        fprintf(stderr, "NETCDF WARNING: Version 2 database on 32-bit system.\n");
+        fprintf(stderr, "NETCDF WARNING: Version 2 file on 32-bit system.\n");
       }
+#endif
     } else {
       status = NC_ENOTNC;
       goto unwind_get;
     }
   }
   
-  status = ncx_get_size_t((const void **)(&gs.pos), &ncp->numrecs);
+  {
+  size_t nrecs = 0;
+  status = ncx_get_size_t((const void **)(&gs.pos), &nrecs);
   if(status != ENOERR)
     goto unwind_get;
+  NC_set_numrecs(ncp, nrecs);
+  }
 
   assert((char *)gs.pos < (char *)gs.end);
 
@@ -1341,6 +1388,8 @@ nc_get_NC(NC *ncp)
   ncp->xsz = ncx_len_NC(ncp, (gs.version == 1) ? 4 : 8);
 
   status = NC_computeshapes(ncp);
+  if(status != ENOERR)
+    goto unwind_get;
 
 unwind_get:
   (void) rel_v1hs(&gs);
