@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/VTK/VTK/Utilities/vtktiff/tif_read.c,v 1.2 2005/02/28 18:49:39 malaterre Exp $ */
+/* $Id: tif_read.c,v 1.4 2008-03-11 19:37:29 andinet Exp $ */
 
 /*
  * Copyright (c) 1988-1997 Sam Leffler
@@ -30,10 +30,9 @@
  */
 #include "tiffiop.h"
 #include <stdio.h>
-#include <assert.h>
 
-static  int TIFFFillStrip(TIFF*, tstrip_t);
-static  int TIFFFillTile(TIFF*, ttile_t);
+        int TIFFFillStrip(TIFF*, tstrip_t);
+        int TIFFFillTile(TIFF*, ttile_t);
 static  int TIFFStartStrip(TIFF*, tstrip_t);
 static  int TIFFStartTile(TIFF*, ttile_t);
 static  int TIFFCheckRead(TIFF*, int);
@@ -51,15 +50,15 @@ TIFFSeek(TIFF* tif, uint32 row, tsample_t sample)
         tstrip_t strip;
 
         if (row >= td->td_imagelength) {        /* out of range */
-                TIFFError(tif->tif_name, "%lu: Row out of range, max %lu",
-                    (u_long) row, (u_long) td->td_imagelength);
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, "%lu: Row out of range, max %lu",
+                    (unsigned long) row, (unsigned long) td->td_imagelength);
                 return (0);
         }
         if (td->td_planarconfig == PLANARCONFIG_SEPARATE) {
                 if (sample >= td->td_samplesperpixel) {
-                        TIFFError(tif->tif_name,
+                        TIFFErrorExt(tif->tif_clientdata, tif->tif_name,
                             "%lu: Sample out of range, max %lu",
-                            (u_long) sample, (u_long) td->td_samplesperpixel);
+                            (unsigned long) sample, (unsigned long) td->td_samplesperpixel);
                         return (0);
                 }
                 strip = sample*td->td_stripsperimage + row/td->td_rowsperstrip;
@@ -92,7 +91,8 @@ TIFFSeek(TIFF* tif, uint32 row, tsample_t sample)
 }
 
 int
-TEXPORT TIFFReadScanline(TIFF* tif, tdata_t buf, uint32 row, tsample_t sample)
+TEXPORT
+TIFFReadScanline(TIFF* tif, tdata_t buf, uint32 row, tsample_t sample)
 {
         int e;
 
@@ -104,7 +104,10 @@ TEXPORT TIFFReadScanline(TIFF* tif, tdata_t buf, uint32 row, tsample_t sample)
                  */
                 e = (*tif->tif_decoderow)
                     (tif, (tidata_t) buf, tif->tif_scanlinesize, sample);
-                tif->tif_row++;
+
+                /* we are now poised at the beginning of the next row */
+                tif->tif_row = row + 1;
+
                 if (e)
                         (*tif->tif_postdecode)(tif, (tidata_t) buf,
                             tif->tif_scanlinesize);
@@ -127,7 +130,7 @@ TIFFReadEncodedStrip(TIFF* tif, tstrip_t strip, tdata_t buf, tsize_t size)
         if (!TIFFCheckRead(tif, 0))
                 return (-1);
         if (strip >= td->td_nstrips) {
-                TIFFError(tif->tif_name, "%ld: Strip out of range, max %ld",
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, "%ld: Strip out of range, max %ld",
                     (long) strip, (long) td->td_nstrips);
                 return (-1);
         }
@@ -153,8 +156,9 @@ TIFFReadEncodedStrip(TIFF* tif, tstrip_t strip, tdata_t buf, tsize_t size)
                 size = stripsize;
         else if (size > stripsize)
                 size = stripsize;
-        if (TIFFFillStrip(tif, strip) && (*tif->tif_decodestrip)(tif,
-            (tidata_t) buf, size, (tsample_t)(strip / td->td_stripsperimage))) {
+        if (TIFFFillStrip(tif, strip) 
+            && (*tif->tif_decodestrip)(tif, (tidata_t) buf, size, 
+                         (tsample_t)(strip / td->td_stripsperimage)) > 0 ) {
                 (*tif->tif_postdecode)(tif, (tidata_t) buf, size);
                 return (size);
         } else
@@ -171,34 +175,35 @@ TIFFReadRawStrip1(TIFF* tif,
                 tsize_t cc;
 
                 if (!SeekOK(tif, td->td_stripoffset[strip])) {
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
                             "%s: Seek error at scanline %lu, strip %lu",
                             tif->tif_name,
-                            (u_long) tif->tif_row, (u_long) strip);
+                            (unsigned long) tif->tif_row, (unsigned long) strip);
                         return (-1);
                 }
                 cc = TIFFReadFile(tif, buf, size);
                 if (cc != size) {
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
                 "%s: Read error at scanline %lu; got %lu bytes, expected %lu",
                             tif->tif_name,
-                            (u_long) tif->tif_row,
-                            (u_long) cc,
-                            (u_long) size);
+                            (unsigned long) tif->tif_row,
+                            (unsigned long) cc,
+                            (unsigned long) size);
                         return (-1);
                 }
         } else {
                 if (td->td_stripoffset[strip] + size > tif->tif_size) {
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
     "%s: Read error at scanline %lu, strip %lu; got %lu bytes, expected %lu",
                             tif->tif_name,
-                            (u_long) tif->tif_row,
-                            (u_long) strip,
-                            (u_long) tif->tif_size - td->td_stripoffset[strip],
-                            (u_long) size);
+                            (unsigned long) tif->tif_row,
+                            (unsigned long) strip,
+                            (unsigned long) tif->tif_size - td->td_stripoffset[strip],
+                            (unsigned long) size);
                         return (-1);
                 }
-                _TIFFmemcpy(buf, tif->tif_base + td->td_stripoffset[strip], size);
+                _TIFFmemcpy(buf, tif->tif_base + td->td_stripoffset[strip],
+                            size);
         }
         return (size);
 }
@@ -216,15 +221,15 @@ TIFFReadRawStrip(TIFF* tif, tstrip_t strip, tdata_t buf, tsize_t size)
         if (!TIFFCheckRead(tif, 0))
                 return ((tsize_t) -1);
         if (strip >= td->td_nstrips) {
-                TIFFError(tif->tif_name, "%lu: Strip out of range, max %lu",
-                    (u_long) strip, (u_long) td->td_nstrips);
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, "%lu: Strip out of range, max %lu",
+                    (unsigned long) strip, (unsigned long) td->td_nstrips);
                 return ((tsize_t) -1);
         }
         bytecount = td->td_stripbytecount[strip];
         if (bytecount <= 0) {
-                TIFFError(tif->tif_name,
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name,
                     "%lu: Invalid strip byte count, strip %lu",
-                    (u_long) bytecount, (u_long) strip);
+                    (unsigned long) bytecount, (unsigned long) strip);
                 return ((tsize_t) -1);
         }
         if (size != (tsize_t)-1 && size < bytecount)
@@ -237,7 +242,7 @@ TIFFReadRawStrip(TIFF* tif, tstrip_t strip, tdata_t buf, tsize_t size)
  * The data buffer is expanded, as necessary, to
  * hold the strip's data.
  */
-static int
+int
 TIFFFillStrip(TIFF* tif, tstrip_t strip)
 {
         static const char module[] = "TIFFFillStrip";
@@ -246,13 +251,14 @@ TIFFFillStrip(TIFF* tif, tstrip_t strip)
 
         bytecount = td->td_stripbytecount[strip];
         if (bytecount <= 0) {
-                TIFFError(tif->tif_name,
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name,
                     "%lu: Invalid strip byte count, strip %lu",
-                    (u_long) bytecount, (u_long) strip);
+                    (unsigned long) bytecount, (unsigned long) strip);
                 return (0);
         }
         if (isMapped(tif) &&
-            (isFillOrder(tif, td->td_fillorder) || (tif->tif_flags & TIFF_NOBITREV))) {
+            (isFillOrder(tif, td->td_fillorder)
+             || (tif->tif_flags & TIFF_NOBITREV))) {
                 /*
                  * The image is mapped into memory and we either don't
                  * need to flip bits or the compression routine is going
@@ -272,12 +278,12 @@ TIFFFillStrip(TIFF* tif, tstrip_t strip)
                          * This error message might seem strange, but it's
                          * what would happen if a read were done instead.
                          */
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
                     "%s: Read error on strip %lu; got %lu bytes, expected %lu",
                             tif->tif_name,
-                            (u_long) strip,
-                            (u_long) tif->tif_size - td->td_stripoffset[strip],
-                            (u_long) bytecount);
+                            (unsigned long) strip,
+                            (unsigned long) tif->tif_size - td->td_stripoffset[strip],
+                            (unsigned long) bytecount);
                         tif->tif_curstrip = NOSTRIP;
                         return (0);
                 }
@@ -293,16 +299,16 @@ TIFFFillStrip(TIFF* tif, tstrip_t strip)
                 if (bytecount > tif->tif_rawdatasize) {
                         tif->tif_curstrip = NOSTRIP;
                         if ((tif->tif_flags & TIFF_MYBUFFER) == 0) {
-                                TIFFError(module,
+                                TIFFErrorExt(tif->tif_clientdata, module,
                                 "%s: Data buffer too small to hold strip %lu",
-                                    tif->tif_name, (u_long) strip);
+                                    tif->tif_name, (unsigned long) strip);
                                 return (0);
                         }
                         if (!TIFFReadBufferSetup(tif, 0,
                             TIFFroundup(bytecount, 1024)))
                                 return (0);
                 }
-                if (TIFFReadRawStrip1(tif, strip, (u_char *)tif->tif_rawdata,
+                if (TIFFReadRawStrip1(tif, strip, (unsigned char *)tif->tif_rawdata,
                     bytecount, module) != bytecount)
                         return (0);
                 if (!isFillOrder(tif, td->td_fillorder) &&
@@ -322,7 +328,8 @@ TIFFFillStrip(TIFF* tif, tstrip_t strip)
  * tile is selected by the (x,y,z,s) coordinates.
  */
 tsize_t
-TEXPORT TIFFReadTile(TIFF* tif,
+TEXPORT
+TIFFReadTile(TIFF* tif,
     tdata_t buf, uint32 x, uint32 y, uint32 z, tsample_t s)
 {
         if (!TIFFCheckRead(tif, 1) || !TIFFCheckTile(tif, x, y, z, s))
@@ -344,8 +351,8 @@ TIFFReadEncodedTile(TIFF* tif, ttile_t tile, tdata_t buf, tsize_t size)
         if (!TIFFCheckRead(tif, 1))
                 return (-1);
         if (tile >= td->td_nstrips) {
-                TIFFError(tif->tif_name, "%ld: Tile out of range, max %ld",
-                    (long) tile, (u_long) td->td_nstrips);
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, "%ld: Tile out of range, max %ld",
+                    (long) tile, (unsigned long) td->td_nstrips);
                 return (-1);
         }
         if (size == (tsize_t) -1)
@@ -370,7 +377,7 @@ TIFFReadRawTile1(TIFF* tif,
                 tsize_t cc;
 
                 if (!SeekOK(tif, td->td_stripoffset[tile])) {
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
                             "%s: Seek error at row %ld, col %ld, tile %ld",
                             tif->tif_name,
                             (long) tif->tif_row,
@@ -380,25 +387,25 @@ TIFFReadRawTile1(TIFF* tif,
                 }
                 cc = TIFFReadFile(tif, buf, size);
                 if (cc != size) {
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
             "%s: Read error at row %ld, col %ld; got %lu bytes, expected %lu",
                             tif->tif_name,
                             (long) tif->tif_row,
                             (long) tif->tif_col,
-                            (u_long) cc,
-                            (u_long) size);
+                            (unsigned long) cc,
+                            (unsigned long) size);
                         return ((tsize_t) -1);
                 }
         } else {
                 if (td->td_stripoffset[tile] + size > tif->tif_size) {
-                        TIFFError(module,
+                        TIFFErrorExt(tif->tif_clientdata, module,
     "%s: Read error at row %ld, col %ld, tile %ld; got %lu bytes, expected %lu",
                             tif->tif_name,
                             (long) tif->tif_row,
                             (long) tif->tif_col,
                             (long) tile,
-                            (u_long) tif->tif_size - td->td_stripoffset[tile],
-                            (u_long) size);
+                            (unsigned long) tif->tif_size - td->td_stripoffset[tile],
+                            (unsigned long) size);
                         return ((tsize_t) -1);
                 }
                 _TIFFmemcpy(buf, tif->tif_base + td->td_stripoffset[tile], size);
@@ -419,8 +426,8 @@ TIFFReadRawTile(TIFF* tif, ttile_t tile, tdata_t buf, tsize_t size)
         if (!TIFFCheckRead(tif, 1))
                 return ((tsize_t) -1);
         if (tile >= td->td_nstrips) {
-                TIFFError(tif->tif_name, "%lu: Tile out of range, max %lu",
-                    (u_long) tile, (u_long) td->td_nstrips);
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, "%lu: Tile out of range, max %lu",
+                    (unsigned long) tile, (unsigned long) td->td_nstrips);
                 return ((tsize_t) -1);
         }
         bytecount = td->td_stripbytecount[tile];
@@ -434,7 +441,7 @@ TIFFReadRawTile(TIFF* tif, ttile_t tile, tdata_t buf, tsize_t size)
  * The data buffer is expanded, as necessary, to
  * hold the tile's data.
  */
-static int
+int
 TIFFFillTile(TIFF* tif, ttile_t tile)
 {
         static const char module[] = "TIFFFillTile";
@@ -443,13 +450,14 @@ TIFFFillTile(TIFF* tif, ttile_t tile)
 
         bytecount = td->td_stripbytecount[tile];
         if (bytecount <= 0) {
-                TIFFError(tif->tif_name,
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name,
                     "%lu: Invalid tile byte count, tile %lu",
-                    (u_long) bytecount, (u_long) tile);
+                    (unsigned long) bytecount, (unsigned long) tile);
                 return (0);
         }
         if (isMapped(tif) &&
-            (isFillOrder(tif, td->td_fillorder) || (tif->tif_flags & TIFF_NOBITREV))) {
+            (isFillOrder(tif, td->td_fillorder)
+             || (tif->tif_flags & TIFF_NOBITREV))) {
                 /*
                  * The image is mapped into memory and we either don't
                  * need to flip bits or the compression routine is going
@@ -480,7 +488,7 @@ TIFFFillTile(TIFF* tif, ttile_t tile)
                 if (bytecount > tif->tif_rawdatasize) {
                         tif->tif_curtile = NOTILE;
                         if ((tif->tif_flags & TIFF_MYBUFFER) == 0) {
-                                TIFFError(module,
+                                TIFFErrorExt(tif->tif_clientdata, module,
                                 "%s: Data buffer too small to hold tile %ld",
                                     tif->tif_name, (long) tile);
                                 return (0);
@@ -489,8 +497,9 @@ TIFFFillTile(TIFF* tif, ttile_t tile)
                             TIFFroundup(bytecount, 1024)))
                                 return (0);
                 }
-                if (TIFFReadRawTile1(tif, tile, (u_char *)tif->tif_rawdata,
-                    bytecount, module) != bytecount)
+                if (TIFFReadRawTile1(tif, tile,
+                                     (unsigned char *)tif->tif_rawdata,
+                                     bytecount, module) != bytecount)
                         return (0);
                 if (!isFillOrder(tif, td->td_fillorder) &&
                     (tif->tif_flags & TIFF_NOBITREV) == 0)
@@ -528,7 +537,7 @@ TIFFReadBufferSetup(TIFF* tif, tdata_t bp, tsize_t size)
                 tif->tif_flags |= TIFF_MYBUFFER;
         }
         if (tif->tif_rawdata == NULL) {
-                TIFFError(module,
+                TIFFErrorExt(tif->tif_clientdata, module,
                     "%s: No space for data buffer at scanline %ld",
                     tif->tif_name, (long) tif->tif_row);
                 tif->tif_rawdatasize = 0;
@@ -590,11 +599,11 @@ static int
 TIFFCheckRead(TIFF* tif, int tiles)
 {
         if (tif->tif_mode == O_WRONLY) {
-                TIFFError(tif->tif_name, "File not open for reading");
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, "File not open for reading");
                 return (0);
         }
         if (tiles ^ isTiled(tif)) {
-                TIFFError(tif->tif_name, tiles ?
+                TIFFErrorExt(tif->tif_clientdata, tif->tif_name, tiles ?
                     "Can not read tiles from a stripped image" :
                     "Can not read scanlines from a tiled image");
                 return (0);
@@ -616,6 +625,14 @@ _TIFFSwab16BitData(TIFF* tif, tidata_t buf, tsize_t cc)
     TIFFSwabArrayOfShort((uint16*) buf, cc/2);
 }
 
+void
+_TIFFSwab24BitData(TIFF* tif, tidata_t buf, tsize_t cc)
+{
+    (void) tif;
+    assert((cc % 3) == 0);
+    TIFFSwabArrayOfTriples((uint8*) buf, cc/3);
+}
+
 void
 _TIFFSwab32BitData(TIFF* tif, tidata_t buf, tsize_t cc)
 {
@@ -631,3 +648,5 @@ _TIFFSwab64BitData(TIFF* tif, tidata_t buf, tsize_t cc)
     assert((cc & 7) == 0);
     TIFFSwabArrayOfDouble((double*) buf, cc/8);
 }
+
+/* vim: set ts=8 sts=8 sw=8 noet: */
