@@ -54,36 +54,28 @@ char IPWeventLog[] =
 "MiddleButtonReleaseEvent 179 135 0 0 0 0 i\n"
 "RightButtonPressEvent 179 135 0 0 0 0 i\n"
 "MouseMoveEvent 180 135 0 0 0 0 i\n"
+"MouseMoveEvent 181 136 0 0 0 0 i\n"
+"MouseMoveEvent 181 137 0 0 0 0 i\n"
+"MouseMoveEvent 181 138 0 0 0 0 i\n"
+"MouseMoveEvent 181 139 0 0 0 0 i\n"
+"MouseMoveEvent 181 140 0 0 0 0 i\n"
 "MouseMoveEvent 180 140 0 0 0 0 i\n"
-"MouseMoveEvent 180 145 0 0 0 0 i\n"
-"MouseMoveEvent 180 150 0 0 0 0 i\n"
-"MouseMoveEvent 180 155 0 0 0 0 i\n"
-"MouseMoveEvent 181 160 0 0 0 0 i\n"
-"MouseMoveEvent 181 165 0 0 0 0 i\n"
-"MouseMoveEvent 181 170 0 0 0 0 i\n"
-"MouseMoveEvent 181 175 0 0 0 0 i\n"
-"MouseMoveEvent 181 180 0 0 0 0 i\n"
-"MouseMoveEvent 181 185 0 0 0 0 i\n"
-"MouseMoveEvent 181 190 0 0 0 0 i\n"
-"MouseMoveEvent 181 195 0 0 0 0 i\n"
-"MouseMoveEvent 181 200 0 0 0 0 i\n"
-"MouseMoveEvent 180 207 0 0 0 0 i\n"
-"MouseMoveEvent 175 207 0 0 0 0 i\n"
-"MouseMoveEvent 170 207 0 0 0 0 i\n"
-"MouseMoveEvent 165 207 0 0 0 0 i\n"
-"MouseMoveEvent 160 207 0 0 0 0 i\n"
-"MouseMoveEvent 155 209 0 0 0 0 i\n"
-"MouseMoveEvent 150 209 0 0 0 0 i\n"
-"MouseMoveEvent 145 210 0 0 0 0 i\n"
-"MouseMoveEvent 140 211 0 0 0 0 i\n"
-"MouseMoveEvent 135 213 0 0 0 0 i\n"
-"MouseMoveEvent 130 214 0 0 0 0 i\n"
-"MouseMoveEvent 125 215 0 0 0 0 i\n"
-"MouseMoveEvent 120 216 0 0 0 0 i\n"
-"MouseMoveEvent 115 216 0 0 0 0 i\n"
-"MouseMoveEvent 110 217 0 0 0 0 i\n"
+"MouseMoveEvent 175 135 0 0 0 0 i\n"
+"MouseMoveEvent 170 130 0 0 0 0 i\n"
+"MouseMoveEvent 165 130 0 0 0 0 i\n"
+"MouseMoveEvent 160 130 0 0 0 0 i\n"
+"MouseMoveEvent 155 125 0 0 0 0 i\n"
+"MouseMoveEvent 150 120 0 0 0 0 i\n"
+"MouseMoveEvent 145 115 0 0 0 0 i\n"
+"MouseMoveEvent 140 110 0 0 0 0 i\n"
+"RightButtonReleaseEvent 140 110 0 0 0 0 i\n"
+"MouseMoveEvent 135 120 0 0 0 0 i\n"
+"MouseMoveEvent 130 135 0 0 0 0 i\n"
+"MouseMoveEvent 125 170 0 0 0 0 i\n"
+"MouseMoveEvent 120 180 0 0 0 0 i\n"
+"MouseMoveEvent 115 190 0 0 0 0 i\n"
+"MouseMoveEvent 110 200 0 0 0 0 i\n"
 "MouseMoveEvent 106 218 0 0 0 0 i\n"
-"RightButtonReleaseEvent 106 218 0 0 0 0 i\n"
 "LeftButtonPressEvent 106 218 0 0 0 0 i\n"
 "MouseMoveEvent 107 219 0 0 0 0 i\n"
 "MouseMoveEvent 110 218 0 0 0 0 i\n"
@@ -337,6 +329,46 @@ char IPWeventLog[] =
 "MiddleButtonReleaseEvent 86 274 0 0 0 0 Shift_L\n"
 ;
 
+//----------------------------------------------------------------------------
+class vtkWidgetWindowLevelCallback : public vtkCommand
+{
+public:
+  static vtkWidgetWindowLevelCallback *New()
+  { return new vtkWidgetWindowLevelCallback; }
+
+  void Execute( vtkObject *caller, unsigned long vtkNotUsed( event ),
+                void *callData )
+  {
+    vtkImagePlaneWidget* self =
+      reinterpret_cast< vtkImagePlaneWidget* >( caller );
+    if(!self) return;
+
+    double* wl = static_cast<double*>( callData );
+
+    if ( self == this->WidgetX )
+      {
+      this->WidgetY->SetWindowLevel(wl[0],wl[1],1);
+      this->WidgetZ->SetWindowLevel(wl[0],wl[1],1);
+      }
+    else if( self == this->WidgetY )
+      {
+      this->WidgetX->SetWindowLevel(wl[0],wl[1],1);
+      this->WidgetZ->SetWindowLevel(wl[0],wl[1],1);
+      }
+    else if (self == this->WidgetZ)
+      {
+      this->WidgetX->SetWindowLevel(wl[0],wl[1],1);
+      this->WidgetY->SetWindowLevel(wl[0],wl[1],1);
+      }
+  }
+
+  vtkWidgetWindowLevelCallback():WidgetX( 0 ), WidgetY( 0 ), WidgetZ ( 0 ) {}
+
+  vtkImagePlaneWidget* WidgetX;
+  vtkImagePlaneWidget* WidgetY;
+  vtkImagePlaneWidget* WidgetZ;
+};
+
 int ImagePlaneWidget( int argc, char *argv[] )
 {
   char* fname = vtkTestUtilities::ExpandDataFileName(argc, argv, "Data/headsq/quarter");
@@ -365,6 +397,7 @@ int ImagePlaneWidget( int argc, char *argv[] )
   vtkRenderer* ren2 = vtkRenderer::New();
 
   vtkRenderWindow* renWin = vtkRenderWindow::New();
+    renWin->SetMultiSamples(0);
     renWin->AddRenderer(ren2);
     renWin->AddRenderer(ren1);
 
@@ -425,6 +458,15 @@ int ImagePlaneWidget( int argc, char *argv[] )
     planeWidgetZ->DisplayTextOn();
     planeWidgetZ->On();
 
+   vtkWidgetWindowLevelCallback* cbk = vtkWidgetWindowLevelCallback::New();
+   cbk->WidgetX = planeWidgetX;
+   cbk->WidgetY = planeWidgetY;
+   cbk->WidgetZ = planeWidgetZ;
+   planeWidgetX->AddObserver( vtkCommand::EndWindowLevelEvent, cbk );
+   planeWidgetY->AddObserver( vtkCommand::EndWindowLevelEvent, cbk );
+   planeWidgetZ->AddObserver( vtkCommand::EndWindowLevelEvent, cbk );
+   cbk->Delete();
+
   double wl[2];
   planeWidgetZ->GetWindowLevel(wl);
 
