@@ -373,7 +373,7 @@ int TestOrientationMarkerWidget( int argc, char *argv[] )
     // the first prop in the collection will be the cube outline, the last
     // will be the text outlines
     //
-    if ( node && i == 0 || i == (nprops - 1) )
+    if ( node && (i == 0 || i == (nprops - 1)) )
       {
       vtkPolyData* poly = vtkPolyData::SafeDownCast(node->GetMapper()->GetInput());
       if ( poly )
@@ -463,15 +463,6 @@ int TestOrientationMarkerWidget( int argc, char *argv[] )
   //
   vtkMapper::SetResolveCoincidentTopologyToPolygonOffset();
 
-  // just for coverage
-  //
-  cube->TextEdgesOff();
-  cube->TextEdgesOn();
-  cube->CubeOff();
-  cube->CubeOn();
-  cube->FaceTextOff();
-  cube->FaceTextOn();
-
   // anatomic labelling
   //
   cube->SetXPlusFaceText ( "A" );
