@@ -40,7 +40,7 @@
 #include "vtkInteractorObserver.h"
 #include "vtkBox.h"
 
-vtkCxxRevisionMacro(vtkImplicitPlaneRepresentation, "$Revision: 1.9.2.2 $");
+vtkCxxRevisionMacro(vtkImplicitPlaneRepresentation, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkImplicitPlaneRepresentation);
 
 //----------------------------------------------------------------------------
@@ -446,6 +446,19 @@ double *vtkImplicitPlaneRepresentation::GetBounds()
   return this->BoundingBox->GetBounds();
 }
 
+//----------------------------------------------------------------------------
+void vtkImplicitPlaneRepresentation::GetActors(vtkPropCollection *pc)
+{
+  this->OutlineActor->GetActors(pc);
+  this->CutActor->GetActors(pc);
+  this->EdgesActor->GetActors(pc);
+  this->ConeActor->GetActors(pc);
+  this->LineActor->GetActors(pc);
+  this->ConeActor2->GetActors(pc);
+  this->LineActor2->GetActors(pc);
+  this->SphereActor->GetActors(pc);
+}
+
 //----------------------------------------------------------------------------
 void vtkImplicitPlaneRepresentation::ReleaseGraphicsResources(vtkWindow *w)
 {
@@ -710,7 +723,7 @@ void vtkImplicitPlaneRepresentation::Rotate(double X, double Y,
     }
   int *size = this->Renderer->GetSize();
   double l2 = (X-this->LastEventPosition[0])*(X-this->LastEventPosition[0]) + (Y-this->LastEventPosition[1])*(Y-this->LastEventPosition[1]);
-  theta = 360.0 * sqrt(l2/((double)size[0]*size[0]+size[1]*size[1]));
+  theta = 360.0 * sqrt(l2/(size[0]*size[0]+size[1]*size[1]));
 
   //Manipulate the transform to reflect the rotation
   this->Transform->Identity();
