@@ -30,9 +30,12 @@
 #include "vtkLine.h"
 #include "vtkWindow.h"
 #include "vtkObjectFactory.h"
+#include "vtkVectorText.h"
+#include "vtkFollower.h"
+#include "vtkCellPicker.h"
+#include "vtkPolyDataMapper.h"
 
-
-vtkCxxRevisionMacro(vtkLineRepresentation, "$Revision: 1.9.2.1 $");
+vtkCxxRevisionMacro(vtkLineRepresentation, "$Revision: 1.18.2.1 $");
 vtkStandardNewMacro(vtkLineRepresentation);
 
 vtkCxxSetObjectMacro(vtkLineRepresentation,HandleRepresentation,vtkPointHandleRepresentation3D);
@@ -102,17 +105,40 @@ vtkLineRepresentation::vtkLineRepresentation()
   bounds[5] = 0.5;
   this->PlaceFactor = 1.0; //overload parent's value
 
+  // The distance text annotation
+  this->DistanceAnnotationVisibility = 0;
+  this->Distance = 0.0;
+  this->DistanceAnnotationFormat = new char[8]; 
+  sprintf(this->DistanceAnnotationFormat,"%s","%-#6.3g");  
+  this->TextInput = vtkVectorText::New();
+  this->TextInput->SetText( "0" );
+  this->TextMapper = vtkPolyDataMapper::New();
+  this->TextMapper->SetInput( this->TextInput->GetOutput() );
+  this->TextActor = vtkFollower::New();
+  this->TextActor->SetMapper(this->TextMapper);
+  this->TextActor->GetProperty()->SetColor( 1.0, 0.1, 0.0 );  
+  
   // This needs to be initialized before PlaceWidget is called.
   this->InitializedDisplayPosition = 0;
-
-  // Initial creation of the widget, serves to initialize it
-  this->PlaceWidget(bounds);
+  
   this->ClampToBounds = 0;
 
   // The bounding box
   this->BoundingBox = vtkBox::New();
 
+  this->LinePicker = vtkCellPicker::New();
+  this->LinePicker->SetTolerance(0.005); //need some fluff
+  this->LinePicker->AddPickList( this->LineActor );
+  this->LinePicker->PickFromListOn();
+
   this->RepresentationState = vtkLineRepresentation::Outside;
+  this->AnnotationTextScaleInitialized = false;
+  
+  // Initial creation of the widget, serves to initialize it.
+  // Call PlaceWidget() LAST in the constructor, as this method depends on ivar
+  // values.
+  this->PlaceWidget(bounds);
+
 }
 
 //----------------------------------------------------------------------------
@@ -155,6 +181,22 @@ vtkLineRepresentation::~vtkLineRepresentation()
   this->SelectedLineProperty->Delete();
 
   this->BoundingBox->Delete();
+
+  if (this->DistanceAnnotationFormat) 
+    {
+    delete [] this->DistanceAnnotationFormat;
+    this->DistanceAnnotationFormat = NULL;
+    }
+  this->TextInput->Delete();
+  this->TextMapper->Delete();
+  this->TextActor->Delete();
+  this->LinePicker->Delete();
+}
+
+//----------------------------------------------------------------------
+double vtkLineRepresentation::GetDistance()
+{
+  return this->Distance;
 }
 
 //----------------------------------------------------------------------
@@ -502,15 +544,20 @@ int vtkLineRepresentation::ComputeInteractionState(int X, int Y, int vtkNotUsed(
 
   // Check if we are on edges
   int onLine = (vtkLine::DistanceToLine(xyz,p1,p2,t,closest) <= tol2);
-  if ( onLine )
+  if ( onLine && t < 1.0 && t > 0.0 )
     {
     this->InteractionState = vtkLineRepresentation::OnLine;
     this->SetRepresentationState(vtkLineRepresentation::OnLine);
     this->GetPoint1WorldPosition(pos1);
     this->GetPoint2WorldPosition(pos2);
-    closest[0] = pos1[0] + t*(pos2[0]-pos1[0]);
-    closest[1] = pos1[1] + t*(pos2[1]-pos1[1]);
-    closest[2] = pos1[2] + t*(pos2[2]-pos1[2]);
+
+    // Consider the two infinite lines :
+    //   L1: pos1 to pos2
+    //   L2: camera_position to focal_point
+    // The line handle will lie on the closest point on L1.
+
+    this->LinePicker->Pick(X,Y,0.0,this->Renderer);
+    this->LinePicker->GetPickPosition(closest);
     this->LineHandleRepresentation->SetWorldPosition(closest);
     }
   else
@@ -633,15 +680,38 @@ void vtkLineRepresentation::BuildRepresentation()
       this->InitializedDisplayPosition = 1;
       }
 
-    double x[3];
-    this->GetPoint1WorldPosition(x);
-    this->LineSource->SetPoint1(x);
-    this->HandleGeometry[0]->SetCenter(x);
+    double x1[3], x2[3];
+    this->GetPoint1WorldPosition(x1);
+    this->LineSource->SetPoint1(x1);
+    this->HandleGeometry[0]->SetCenter(x1);
 
-    this->GetPoint2WorldPosition(x);
-    this->LineSource->SetPoint2(x);
-    this->HandleGeometry[1]->SetCenter(x);
+    this->GetPoint2WorldPosition(x2);
+    this->LineSource->SetPoint2(x2);
+    this->HandleGeometry[1]->SetCenter(x2);
 
+    this->Distance = sqrt(vtkMath::Distance2BetweenPoints( x1, x2 ));
+
+    // Place the DistanceAnnotation right in between the two points.
+    double x[3] = { (x1[0] + x2[0])/2.0, 
+                    (x1[1] + x2[1])/2.0, 
+                    (x1[2] + x2[2])/2.0 };
+    char string[512];
+    sprintf(string, this->DistanceAnnotationFormat, this->Distance);
+    this->TextInput->SetText( string );
+    this->TextActor->SetPosition( x );
+    if (this->Renderer)
+      {
+      this->TextActor->SetCamera( this->Renderer->GetActiveCamera() );
+      }
+    
+    if (!this->AnnotationTextScaleInitialized)
+      {
+      // If a font size hasn't been specified by the user, scale the text 
+      // (font size) according to the length of the line widget.
+      this->TextActor->SetScale( 
+          this->Distance/10.0, this->Distance/10.0, this->Distance/10.0 );
+      }
+    
     this->SizeHandles();
     this->BuildTime.Modified();
     }
@@ -732,12 +802,22 @@ int vtkLineRepresentation::InBounds(double x[3])
   return 1;
 }
 
+//----------------------------------------------------------------------
+void vtkLineRepresentation::GetActors(vtkPropCollection *pc)
+{
+  this->LineActor->GetActors(pc);
+  this->Handle[0]->GetActors(pc);
+  this->Handle[1]->GetActors(pc);
+  this->TextActor->GetActors(pc);
+}
+
 //----------------------------------------------------------------------------
 void vtkLineRepresentation::ReleaseGraphicsResources(vtkWindow *w)
 {
   this->LineActor->ReleaseGraphicsResources(w);
   this->Handle[0]->ReleaseGraphicsResources(w);
   this->Handle[1]->ReleaseGraphicsResources(w);
+  this->TextActor->ReleaseGraphicsResources(w);
 }
 
 //----------------------------------------------------------------------------
@@ -748,6 +828,10 @@ int vtkLineRepresentation::RenderOpaqueGeometry(vtkViewport *v)
   count += this->LineActor->RenderOpaqueGeometry(v);
   count += this->Handle[0]->RenderOpaqueGeometry(v);
   count += this->Handle[1]->RenderOpaqueGeometry(v);
+  if (this->DistanceAnnotationVisibility)
+    {
+    count += this->TextActor->RenderOpaqueGeometry(v);
+    }
   
   return count;
 }
@@ -760,6 +844,10 @@ int vtkLineRepresentation::RenderTranslucentPolygonalGeometry(vtkViewport *v)
   count += this->LineActor->RenderTranslucentPolygonalGeometry(v);
   count += this->Handle[0]->RenderTranslucentPolygonalGeometry(v);
   count += this->Handle[1]->RenderTranslucentPolygonalGeometry(v);
+  if (this->DistanceAnnotationVisibility)
+    {
+    count += this->TextActor->RenderTranslucentPolygonalGeometry(v);
+    }
   
   return count;
 }
@@ -772,6 +860,10 @@ int vtkLineRepresentation::HasTranslucentPolygonalGeometry()
   result |= this->LineActor->HasTranslucentPolygonalGeometry();
   result |= this->Handle[0]->HasTranslucentPolygonalGeometry();
   result |= this->Handle[1]->HasTranslucentPolygonalGeometry();
+  if (this->DistanceAnnotationVisibility)
+    {
+    result |= this->TextActor->HasTranslucentPolygonalGeometry();
+    }
   
   return result;
 }
@@ -790,6 +882,19 @@ unsigned long vtkLineRepresentation::GetMTime()
   return mTime;
 }
 
+//----------------------------------------------------------------------
+void vtkLineRepresentation::SetDistanceAnnotationScale( double scale[3] )
+{
+  this->TextActor->SetScale( scale );
+  this->AnnotationTextScaleInitialized = true;
+}
+
+//----------------------------------------------------------------------
+double * vtkLineRepresentation::GetDistanceAnnotationScale()
+{
+  return this->TextActor->GetScale();
+}
+
 //----------------------------------------------------------------------------
 void vtkLineRepresentation::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -857,6 +962,26 @@ void vtkLineRepresentation::PrintSelf(ostream& os, vtkIndent indent)
   this->LineHandleRepresentation->PrintSelf(os,indent.GetNextIndent());
   
   os << indent << "Representation State: " << this->RepresentationState << "\n";
+
+  os << indent << "DistanceAnnotationVisibility: ";
+  if ( this->DistanceAnnotationVisibility )
+    {
+    os << this->DistanceAnnotationVisibility << "\n";
+    }
+  else
+    {
+    os << "(none)\n";
+    }
+  
+  os << indent << "DistanceAnnotationFormat: ";
+  if ( this->DistanceAnnotationFormat )
+    {
+    os << this->DistanceAnnotationFormat << "\n";
+    }
+  else
+    {
+    os << "(none)\n";
+    }  
   
   // this->InteractionState is printed in superclass
   // this is commented to avoid PrintSelf errors
