@@ -40,7 +40,7 @@
 #include "vtkObjectFactory.h"
 
 
-vtkCxxRevisionMacro(vtkSphereRepresentation, "$Revision: 1.8.2.2 $");
+vtkCxxRevisionMacro(vtkSphereRepresentation, "$Revision: 1.9 $");
 vtkStandardNewMacro(vtkSphereRepresentation);
 
 //----------------------------------------------------------------------------
@@ -647,8 +647,8 @@ void vtkSphereRepresentation::BuildRepresentation()
     this->HandleSource->GetCenter(hc);
     double r = sqrt( static_cast<double>(vtkMath::Distance2BetweenPoints(c,hc)) );
     r = (r<=0.0 ? 1.0 : r);
-    double theta = vtkMath::RadiansToDegrees()*atan2((hc[1]-c[1]),(hc[0]-c[0]));
-    double phi = vtkMath::RadiansToDegrees()*acos((hc[2]-c[2])/r);
+    double theta = vtkMath::DegreesFromRadians( atan2( ( hc[1] - c[1] ), ( hc[0] - c[0] ) ) );
+    double phi   = vtkMath::DegreesFromRadians( acos( ( hc[2] - c[2] ) / r ) );
     sprintf(str,"(%0.2g, %1.1f, %1.1f)", r, theta, phi);
     this->HandleTextMapper->SetInput(str);
     vtkInteractorObserver::ComputeWorldToDisplay(this->Renderer, hc[0], hc[1], hc[2], tc);
