@@ -1,4 +1,4 @@
 """Utility modules for the VTK-Python wrappers."""
 
 __all__ = ['colors', 'misc', 'vtkConstants', 'vtkImageExportToArray',
-           'vtkImageImportFromArray', 'vtkMethodParser']
+           'vtkImageImportFromArray', 'vtkMethodParser', 'numpy_support']
