@@ -65,6 +65,10 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkParallelInstantiator.h"
 #endif
 
+#ifdef VTK_USE_GEOVIS
+#include "vtkGeovisInstantiator.h"
+#endif
+
 #ifdef VTK_USE_INFOVIS
 #include "vtkInfovisInstantiator.h"
 #endif
@@ -199,6 +203,10 @@ extern "C" int Vtkwidgetstcl_Init(Tcl_Interp *interp);
 extern "C" int Vtkparalleltcl_Init(Tcl_Interp *interp);
 #endif
 
+#ifdef VTK_USE_GEOVIS
+extern "C" int Vtkgeovistcl_Init(Tcl_Interp *interp);
+#endif
+
 #ifdef VTK_USE_INFOVIS
 extern "C" int Vtkinfovistcl_Init(Tcl_Interp *interp);
 #endif
@@ -300,6 +308,13 @@ int Tcl_AppInit(Tcl_Interp *interp)
     }
 #endif
  
+#ifdef VTK_USE_GEOVIS
+  if (Vtkgeovistcl_Init(interp) == TCL_ERROR)
+    {
+    return TCL_ERROR;
+    }
+#endif
+
 #ifdef VTK_USE_INFOVIS
   if (Vtkinfovistcl_Init(interp) == TCL_ERROR)
     {
